<?php

namespace App\Http\Controllers;

use App\Models\RechargeApi;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Log;


class AbandonedCheckoutController extends Controller
{
    private $modelObj;
    private $checkoutReturn;
    private $totalPrice = 0;

    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct() {

        $this->modelObj   = new RechargeApi();
        $this->productObj = new Product();
    }

    /**
     * Create checkout
     */
    public function createAbandonedCheckout(Request $request) {
        
        $requestData = !empty($request->input('data')) ? $request->input('data') : "";
        if (empty($requestData)) {
            $message = "Endpoint parameters are missing - token/email";
            $error = "Bad request";
            $status = "Failed";
            $response = array(
                "status"       => $status,
                "message" => $message,
                "error" => $error
            );
            Log::channel('abandonedCheckoutLog')->info("createAbandonedCheckout: ".$message);
            echo json_encode($response);
            exit();
        }
        $parameterLineItemStatus = 0;
        $parameterShippingStatus = 0;
        $checkoutRequest         = [];
        $lineItems               = [];
        $shippingAddress         = [];
        $receivedData = json_decode($requestData, true);
        $checkoutEmail = !empty($receivedData['email']) ? trim($receivedData['email']) : "";
        $checkoutToken = !empty($receivedData['checkoutToken']) ? trim($receivedData['checkoutToken']) : "";
        if (empty($checkoutEmail) || empty($checkoutToken)) {
            $errorMessage = "Missing required parameters (email or token)";
            Log::channel('abandonedCheckoutLog')->info("createAbandonedCheckout: ".$errorMessage);
            return $this->returnData("Failed", $errorMessage, $request->input('callback'));
        }
        $checkoutDbRecord = DB::table('user_checkouts')
        ->select('user_email','checkout_recharge_token','checkout_details')
        ->where('user_email', '=', $checkoutEmail )
        ->where('checkout_recharge_token', '=', $checkoutToken )
        ->get();
        if (!isset($checkoutDbRecord[0]->checkout_details)) {
            $errorMessage = "Invalid token or email";
            Log::channel('abandonedCheckoutLog')->info("createAbandonedCheckout: No checkout details 
            found in DB for email => ".$checkoutEmail." and token => ".$checkoutToken);
            return $this->returnData("Failed", $errorMessage, $request->input('callback'));
        }
        $checkoutDetails = '';
        $externalCheckoutId = '';
        $externalCheckoutSource = '';
        $discountCode = '';
        if (isset($checkoutDbRecord[0]->checkout_details)) {
            $checkoutDetails = $checkoutDbRecord[0]->checkout_details;
            $checkoutDetails = json_decode($checkoutDetails);
            if (isset($checkoutDetails->checkout->shipping_address)) {
                $shippingAddress = $checkoutDetails->checkout->shipping_address;
            }
            if (isset($checkoutDetails->checkout->line_items)) {
                $lineItems = $checkoutDetails->checkout->line_items;
            }
            if(isset($checkoutDetails->checkout->external_checkout_id)){
                $externalCheckoutId = $checkoutDetails->checkout->external_checkout_id;
            }
            if(isset($checkoutDetails->checkout->external_checkout_source)){
                $externalCheckoutSource = $checkoutDetails->checkout->external_checkout_source;
            }
            if(isset($checkoutDetails->checkout->discount_code)){
                $discountCode = $checkoutDetails->checkout->discount_code;
            }
        }
        $checkoutRequest = array(
            "checkout" => array(
                "line_items"               => $lineItems,
                "shipping_address"         => $shippingAddress,
                "email"                    => !empty($checkoutEmail) ? $checkoutEmail : '',
                "external_checkout_id"     => $externalCheckoutId,
                "external_checkout_source" => $externalCheckoutSource,
                "discount_code"            => $discountCode,
            )
        );
        $properties = $lineItems[0]->properties;
        $lineItemDetails = [];
        $firstName = '';
        $quizId = '';
        if (!empty($properties)) {
            foreach ($properties as $key => $value) {   
                if (strpos($key, 'Vitamin') !== false) {
                    $explodeValue = explode('|', $value);
                    $item = new \stdClass();
                    if (!empty($explodeValue[3]) && !empty($explodeValue[2])) {
                        $item->id = trim($explodeValue[3]);
                        $item->quantity = trim($explodeValue[2]);
                        $lineItemDetails[] = $item;
                    }
                }
                if ($key == 'first_name') {
                    $firstName = $value;
                }
                if ($key == 'quiz_id') {
                    $quizId = $value;
                }
            }
        }
        foreach ($checkoutRequest['checkout'] as $key => $value) {
            if (empty($value)) {
                unset($checkoutRequest['checkout'][$key]);
            }
        }

        $checkoutReturn = $this->modelObj->createCheckout(json_encode($checkoutRequest));

        Log::channel('abandonedCheckoutLog')->info("createAbandonedCheckout: Checkout =>".
        json_encode($checkoutReturn));
        if (isset($checkoutReturn->error) || isset($checkoutReturn->errors) ||
            empty($checkoutReturn)) {
            $status = "failed";
            $error  = "Unknown error";
            if (isset($checkoutReturn->error)) {
                $error = json_encode($checkoutReturn->error);
            } else if (isset($checkoutReturn->errors)) {
                $error = json_encode($checkoutReturn->errors);
            }
            $errorMessage = "Failed to create the Checkout";
            Log::channel('abandonedCheckoutLog')->info("createAbandonedCheckout: CheckoutError =>".$error);
            return $this->returnData("Failed", $errorMessage, $request->input('callback'));
        }
        $url = env('NURISH_CHECKOUT_URL', true) . $checkoutReturn->checkout->token .
            '?myshopify_domain=' . env('SHOPIFY_STORE', true);
        echo $_REQUEST['callback'] . "({'status':'success','url': '" . $url . "','errorMessage':'',
            'lineItems': '" . json_encode($lineItemDetails) . "','firstName': '" .$firstName. "',
            'quizId': '" .$quizId. "'})";
        exit();
    }

    /**
     * Response function
     */
    public function returnData($status, $errorMessage, $callBackToken) {

        $response = array(
            "status"       => $status,
            "errorMessage" => $errorMessage
        );
        echo $callBackToken . '(' . json_encode($response) . ')';
        exit();
    }
    
    /**
     * Create checkout
     */
    public function createAbandonedCart(Request $request) {
        
        $emailAddress = !empty($request->input('email')) ? trim($request->input('email')) : "";
        $token = !empty($request->input('checkoutToken')) ? trim($request->input('checkoutToken')) : "";
        if (empty($emailAddress) && empty($token)) {
            $message = "Endpoint parameters are missing - token/email";
            $error = "Bad request";
            $status = "Failed";
            $response = array(
                "status"       => $status,
                "message" => $message,
                "error" => $error
            );
            echo json_encode($response);
            exit();
        }

        $parameterLineItemStatus = 0;
        $parameterShippingStatus = 0;
        $checkoutRequest         = array();
        $lineItems               = array();
        $shippingAddress         = array();

        $receivedData = $_REQUEST;

        if (empty($emailAddress) || empty($token)) {
            $errorMessage = "Missing required parameters (email or token)";
            Log::channel('abandonedCheckoutLog')->info("createAbandonedCart: ".$errorMessage);
            return $this->returnData("Failed", $errorMessage, $request->input('callback'));
        }
        $checkoutEmail = trim($emailAddress);
        $checkoutEmail = str_replace(' ','+',$checkoutEmail);
        $checkoutToken = trim($token);

        $checkoutDbRecord = DB::table('user_checkouts')
        ->select('user_email','checkout_recharge_token','checkout_details')
        ->where('user_email', '=', $checkoutEmail )
        ->where('checkout_recharge_token', '=', $checkoutToken )
        ->get();

        if (!isset($checkoutDbRecord[0]->checkout_details)) {
            $errorMessage = "Invalid token or email";
            Log::channel('abandonedCheckoutLog')->info("createAbandonedCart: No checkout details 
            found in DB for email => ".$checkoutEmail." and token => ".$checkoutToken);
            return $this->returnData("Failed", $errorMessage, $request->input('callback'));
        }
        $checkoutDetails = '';
        $externalCheckoutId = '';
        $externalCheckoutSource = '';
        $discountCode = '';
        if (isset($checkoutDbRecord[0]->checkout_details)) {
            $checkoutDetails = $checkoutDbRecord[0]->checkout_details;
            $checkoutDetails = json_decode($checkoutDetails);
            if (isset($checkoutDetails->checkout->shipping_address)) {
                $shippingAddress = $checkoutDetails->checkout->shipping_address;
            }
            if (isset($checkoutDetails->checkout->line_items)) {
                $lineItems = $checkoutDetails->checkout->line_items;
            }
            if (isset($checkoutDetails->checkout->external_checkout_id)) {
                $externalCheckoutId = $checkoutDetails->checkout->external_checkout_id;
            }
            if (isset($checkoutDetails->checkout->external_checkout_source)) {
                $externalCheckoutSource = $checkoutDetails->checkout->external_checkout_source;
            }
            if (isset($checkoutDetails->checkout->discount_code)) {
                $discountCode = $checkoutDetails->checkout->discount_code;
            }
        }
        $checkoutRequest = array(
            "checkout" => array(
                "line_items"               => $lineItems,
                "shipping_address"         => $shippingAddress,
                "email"                    => !empty($checkoutEmail) ? $checkoutEmail : '',
                "external_checkout_id"     => $externalCheckoutId,
                "external_checkout_source" => $externalCheckoutSource,
                "discount_code"            => $discountCode,

            )
        );
        $properties = $lineItems[0]->properties;
        $lineItemDetails = [];
        if(!empty($properties)){
            foreach($properties as $key => $value) {   
                if (strpos($key, 'Vitamin') !== false) {
                    $explodeValue = explode('|', $value);
                    $item = new \stdClass();
                    if(!empty($explodeValue[3]) && !empty($explodeValue[2])){
                        $item->id = trim($explodeValue[3]);
                        $item->quantity = trim($explodeValue[2]);
                        $lineItemDetails[] = $item;
                    }
                }
            }
        }
        foreach ($checkoutRequest['checkout'] as $key => $value) {
            if (empty($value)) {
                unset($checkoutRequest['checkout'][$key]);
            }
        }

        $checkoutReturn = $this->modelObj->createCheckout(json_encode($checkoutRequest));

        Log::channel('abandonedCheckoutLog')->info("createAbandonedCart: Checkout =>". 
        json_encode($checkoutReturn));
        if (isset($checkoutReturn->error) || isset($checkoutReturn->errors) ||
            empty($checkoutReturn)) {
            $status = "failed";
            $error  = "Unknown error";
            if (isset($checkoutReturn->error)) {
                $error = json_encode($checkoutReturn->error);
            } else if (isset($checkoutReturn->errors)) {
                $error = json_encode($checkoutReturn->errors);
            }
            $errorMessage = "Failed to create the Checkout";
            Log::channel('abandonedCheckoutLog')->info("createAbandonedCart: CheckoutError =>". $error);
            return $this->returnData("Failed", $errorMessage, $request->input('callback'));
        }
        $url = env('NURISH_CHECKOUT_URL', true) . $checkoutReturn->checkout->token .
            '?myshopify_domain=' . env('SHOPIFY_STORE', true);
        return redirect()->to($url);
        
        echo $_REQUEST['callback'] . "({'status':'success','url': '" . $url . "','errorMessage':'',
            'lineItems': '" . json_encode($lineItemDetails) . "'})";
        exit();
    }

}
