<?php

namespace App\Http\Controllers;

use App\Models\RechargeApi;
use App\Models\Shopify\Product;
use App\Models\GetPackCount;
use Illuminate\Http\Request;
use Log;

class CheckoutController extends Controller
{
    private $modelObj;
    private $packCount;
    private $checkoutReturn;
    private $productObj;
    private $totalPrice = 0;

    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct() {

        $this->modelObj   = new RechargeApi();
        $this->packCount  = new GetPackCount();
        $this->productObj = new Product();
    }

    /**
     * Create checkout
     */
    public function CreateCheckout(Request $request) {

        $requestData = $request->input('data');
        if (empty($requestData)) {
            $message = "Endpoint parameters are missing";
            $error = "Bad request";
            $status = "Failed";
            $response = array(
                "status"       => $status,
                "message" => $message,
                "error" => $error
            );
            Log::channel('checkoutLog')->info("CreateCheckout: ".$message);
            echo json_encode($response);
            exit();
        }
        $parameterShippingStatus = 1;
        $checkoutRequest         = [];
        $lineItems               = [];
        $shippingAddress         = [];
        $giftPackDetails         = [];
        $giftPackStatus          = 0;
        Log::channel('checkoutLog')->info("CreateCheckout: Request => ".$requestData);
        $receivedData = json_decode($requestData, true);
        if (!isset($receivedData['checkout']) || !isset($receivedData['checkout']['line_items'])) {
            $errorMessage = "The endpoint request is not valid";
            Log::channel('checkoutLog')->info("CreateCheckout: ".$errorMessage);
            return $this->returnData("Failed", $errorMessage, $request->input('callback'));
        }
        if (!isset($receivedData['checkout']['line_items'][0]['properties'])) {
            $errorMessage = "Missing line item properties in the request";
            Log::channel('checkoutLog')->info("CreateCheckout: ".$errorMessage);
            return $this->returnData("Failed", $errorMessage, $request->input('callback'));
        }
        $pillsInProperties = 0;
        $propertyValidity = 1;
        $shProductId = "";
        $shVariantId = "";
        $nonPillProperties = ['utm', 'override_free_shipping', 'special_pack_type', 'partner', 'campaign', 'first_name',
        'source', 'medium', 'content', 'term', 'quiz_id', 'campaign_id', 'template_id', 'future_month_free', 'corp_discount',
        'giftToSub'];
        $shippingAddressRequiredParams = ['last_name', 'address1', 'city', 'country', 'province'];

        foreach ($receivedData['checkout']['line_items'] as $checkoutValue) {
            $shProductId = !empty($checkoutValue['product_id']) ? trim($checkoutValue['product_id']) : "";
            $shVariantId = !empty($checkoutValue['variant_id']) ? trim($checkoutValue['variant_id']) : "";
            foreach ($checkoutValue['properties'] as $propertyValue) {
                if (isset($propertyValue['gift']) && !empty($propertyValue['gift'])) {
                    $giftPackStatusInRequest = !empty($propertyValue['gift']) ? trim($propertyValue['gift']) : '';
                    if ($giftPackStatusInRequest && !empty($giftPackStatusInRequest)) {
                        $giftPackStatus = 1;
                        $giftPackDetails = $propertyValue;
                    }
                    continue;
                }
                $nonPillPropertyStatus = 0;
                foreach ($nonPillProperties as $nonPillKey => $nonPillValue) {
                    if (isset($propertyValue[$nonPillValue])) {
                        $nonPillPropertyStatus = 1;
                    }
                }
                if ($nonPillPropertyStatus == 1) {
                    continue;
                }
                $requestedProductId = !empty($propertyValue['product_id']) ? trim($propertyValue['product_id']) : "";
                $requestedPdtVariantId = !empty($propertyValue['variant_id']) ? trim($propertyValue['variant_id']) : "";
                $requestedProductName = !empty($propertyValue['name']) ? trim($propertyValue['name']) : "";
                $requestedProductValue = !empty($propertyValue['value']) ? trim($propertyValue['value']) : "";
                $requestedDosage = !empty($propertyValue['dosage']) ? trim($propertyValue['dosage']) : "";
                if (!empty($requestedProductId) || !empty($requestedPdtVariantId) || !empty($requestedProductName) ||
                    !empty($requestedProductValue) ||  !empty($requestedDosage)) {
                    $pillsInProperties = 1; 
                }
                if (empty($requestedProductId) || empty($requestedPdtVariantId) || empty($requestedProductName) ||
                    empty($requestedProductValue) || empty($requestedDosage)) {
                        $propertyValidity = 0;
                }
            }   
        }
        if ($pillsInProperties == 0) {
            $errorMessage = "Not found any pills in the line item properties";
            Log::channel('checkoutLog')->info("CreateCheckout: ".$errorMessage);
            return $this->returnData("Failed", $errorMessage, $request->input('callback'));
        }
        if ($propertyValidity == 0) {
            $errorMessage = "Missing required parameters in the line item properties";
            Log::channel('checkoutLog')->info("CreateCheckout: ".$errorMessage);
            return $this->returnData("Failed", $errorMessage, $request->input('callback'));
        }
        if (empty($shProductId) || empty($shVariantId)) {
            $errorMessage = "Shopify productId or variantId is missing";
            Log::channel('checkoutLog')->info("CreateCheckout: ".$errorMessage);
            return $this->returnData("Failed", $errorMessage, $request->input('callback'));
        }
        if (isset($receivedData['checkout']['shipping_address']) || !empty($receivedData['checkout']['shipping_address'])) {
            foreach ($shippingAddressRequiredParams as $shippingAddressKey => $shippingAddressValue) {
                if (empty($receivedData['checkout']['shipping_address'][$shippingAddressValue])) {
                    $parameterShippingStatus = 0;
                }
            }
        }
        if ($parameterShippingStatus == 0) {
            $errorMessage = "Missing required parameters in the Shipping Address";
            Log::channel('checkoutLog')->info("CreateCheckout: ".$errorMessage);
            return $this->returnData("Failed", $errorMessage, $request->input('callback'));
        }
        $otp = false;
        $otpExtra = 0.9;
        if (isset($receivedData['checkout']['one_time_purchase'])) {
            $otp = trim($receivedData['checkout']['one_time_purchase']);
        }
        if ((!$otp || $otp == 'false') || $giftPackStatus == 1) {
            $requestChargeFrequency = !empty($checkoutValue['charge_interval_frequency']) ? 
            trim($checkoutValue['charge_interval_frequency']) : "";
            $requestOrderFrequency = !empty($checkoutValue['order_interval_frequency']) ? 
            trim($checkoutValue['order_interval_frequency']) : "";
            $requestIntervalUnit = !empty($checkoutValue['order_interval_unit']) ? 
            trim($checkoutValue['order_interval_unit']) : "";

            if (empty($requestChargeFrequency) || empty($requestOrderFrequency) || empty($requestIntervalUnit)) {
                $errorMessage = "Missing required charge_interval_frequency or order_interval_frequency or order_interval_unit in the request";
                Log::channel('checkoutLog')->info("CreateCheckout: ".$errorMessage);
                return $this->returnData("Failed", $errorMessage, $request->input('callback'));
            }
        }
        $utmParameter = '';
        $shippingOverRideStatus = '';
        $specialPackType = '';
        $partnerValue = '';
        $campaignValue = '';
        $firstName = '';
        $source = '';
        $medium = '';
        $content = '';
        $term = '';
        $quizId = '';
        $campaignId = '';
        $templateId = '';
        $futureMonthFree = '';
        $corp_discount = '';
        $giftToSub = '';
        if (isset($receivedData['checkout']['line_items'])) {
            $i = 0;
            foreach ($receivedData['checkout']['line_items'] as $itemValue) {
                if (isset($itemValue['properties'])) {
                    //Handling duplicate line items in properties
                    $rearrangeProperties = [];
                    $j = 1;
                    foreach ($itemValue['properties'] as $propertyValue) {
                        if (isset($propertyValue['gift']) && !empty($propertyValue['gift'])) {
                            continue;
                        }
                        if (!isset($propertyValue['utm'])
                        && !isset($propertyValue['override_free_shipping'])
                        && !isset($propertyValue['special_pack_type'])
                        && !isset($propertyValue['partner'])
                        && !isset($propertyValue['campaign'])
                        && !isset($propertyValue['first_name'])
                        && !isset($propertyValue['source'])
                        && !isset($propertyValue['medium'])
                        && !isset($propertyValue['content'])
                        && !isset($propertyValue['term'])
                        && !isset($propertyValue['quiz_id'])
                        && !isset($propertyValue['campaign_id'])
                        && !isset($propertyValue['template_id'])
                        && !isset($propertyValue['future_month_free'])
                        && !isset($propertyValue['corp_discount'])
                        && !isset($propertyValue['giftToSub'])
                        ) {
                            if(isset($rearrangeProperties[$propertyValue['variant_id']])){
                                $rearrangeProperties[$propertyValue['variant_id']]['quantity'] +=
                                $propertyValue['quantity'];

                            } else {
                                $propertyValue['name'] = 'Vitamin'. ($j<10? '0'.$j++:$j++);
                                $rearrangeProperties[$propertyValue['variant_id']] = $propertyValue;
                         }
                       } else {
                            if (isset($propertyValue['utm'])) {
                                $utmParameter = $propertyValue['utm'];
                            } elseif (isset($propertyValue['override_free_shipping'])) {
                                $shippingOverRideStatus = ($propertyValue['override_free_shipping'] == "true") ?
                                "true" : "false";
                            } elseif (isset($propertyValue['special_pack_type'])) {
                                $specialPackType = $propertyValue['special_pack_type'];
                            } elseif (isset($propertyValue['partner'])) {
                                $partnerValue = $propertyValue['partner'];
                            } elseif (isset($propertyValue['campaign'])) {
                                $campaignValue = $propertyValue['campaign'];
                            } elseif (isset($propertyValue['first_name'])) {
                                $firstName = $propertyValue['first_name'];
                            } elseif (isset($propertyValue['source'])) {
                                $source = $propertyValue['source'];
                            } elseif (isset($propertyValue['medium'])) {
                                $medium = $propertyValue['medium'];
                            } elseif (isset($propertyValue['content'])) {
                                $content = $propertyValue['content'];
                            } elseif (isset($propertyValue['term'])) {
                                $term = $propertyValue['term'];
                            } elseif (isset($propertyValue['quiz_id'])) {
                                $quizId = $propertyValue['quiz_id'];
                            } elseif (isset($propertyValue['campaign_id'])) {
                                $campaignId = $propertyValue['campaign_id'];
                            } elseif (isset($propertyValue['template_id'])) {
                                $templateId = $propertyValue['template_id'];
                            } elseif (isset($propertyValue['future_month_free'])) {
                                $futureMonthFree = $propertyValue['future_month_free'];
                            } elseif (isset($propertyValue['corp_discount'])) {
                                $corp_discount = $propertyValue['corp_discount'];
                            } elseif (isset($propertyValue['giftToSub'])) {
                                $giftToSub = $propertyValue['giftToSub'];
                            }
                        }
                    }
                    $itemValue['properties'] =  array_values($rearrangeProperties);
                }
                unset($receivedData['checkout']['line_items'][$i]);
                $receivedData['checkout']['line_items'][$i] =  $itemValue;
                $i++;
            }
        }
        $lineItemProperties = [];
        $packData = '';
        $packCountRequest = [];
        $productPackCount = '';
        $melatoninProductIds = [1837729808455, 1837729775687, 4329622405191];
        $melatoninCount = 0;
        $productVariantErrorStatus = 0;
        $variantError = '';
        $productValue = '';
        $elderberryQuantity = 0;
        $elderberryQuantityAdjustedRequest = [];
        $elderberryQuantity = $this->getElderBerryQuantity($receivedData['checkout']['line_items'][0]['properties']);
        if ($elderberryQuantity > 1) {
            $elderberryQuantityAdjustedRequest = $this->updateElderberryQuantity($receivedData['checkout']['line_items'][0]['properties']);
            $receivedData['checkout']['line_items'][0]['properties'] = $elderberryQuantityAdjustedRequest;
            Log::channel('checkoutLog')->info("CreateCheckout: Elderberry quantity adjusted request => ".json_encode($elderberryQuantityAdjustedRequest));
        }

        $shopifyProductDetails = $this->getShopifyProductDetails();

        if (empty($shopifyProductDetails) || !isset($shopifyProductDetails)) {
            $errorMessage = "Failed to get the product details from Shopify";
            Log::channel('checkoutLog')->info("CreateCheckout: ".$errorMessage);
            return $this->returnData("Failed", $errorMessage, $request->input('callback'));
        }
        if (isset($receivedData['checkout']['line_items'])) {
            if (($otp || $otp == 'true') && $otpExtra > 0) {
              $otpEmail = !empty($receivedData['checkout']['email']) ? $receivedData['checkout']['email'] : '';
              $otpExtraInFile = $this->getOTPExtra($otpEmail);
              if ($otpExtraInFile != '') {
                  $otpExtraInFile = (100-$otpExtraInFile)/100;
              }
              
              if (($otpExtraInFile != '' && $otpExtraInFile > 0) && ($otpExtraInFile != $otpExtra)) {
                  $otpExtra = $otpExtraInFile;
              }
            }
            foreach ($receivedData['checkout']['line_items'] as $itemValue) {
                if (isset($itemValue['properties'])) {
                    foreach ($itemValue['properties'] as $propertyValue) {
                        $requestedVariantId = '';
                        $variantDetails = [];
                        if (!empty($propertyValue['product_id']) && !empty($propertyValue['variant_id'])) {
                            $requestedVariantId = $propertyValue['variant_id'];
                            $variantDetails = !empty($shopifyProductDetails[$requestedVariantId]) ? 
                            $shopifyProductDetails[$requestedVariantId] : '';
                            if (empty($variantDetails) || !isset($variantDetails)) {
                                $productVariantErrorStatus = 1;
                                $productValue = $propertyValue['value'];
                                $variantError = json_encode($propertyValue);
                                break;
                            }
                            $freePill = false;
                            $lineItemProperties[$propertyValue['name']] = $propertyValue['value'] .
                                (!empty($propertyValue['dosage']) ? ' | ' . $propertyValue['dosage'] : '') .
                                (!empty($propertyValue['quantity']) ? ' | ' . $propertyValue['quantity'] : ' | 1') .
                                ' | ' . $propertyValue['variant_id'] . ' | ' . $variantDetails['sku'] . ($freePill ? ' | free_pill' : '');

                            if (empty($propertyValue['quantity'])) {
                                $productQuantity = 1;
                            } else {
                                $productQuantity = $propertyValue['quantity'];
                            }
                            if (in_array($propertyValue['product_id'], $melatoninProductIds)) {
                                $melatoninCount = $melatoninCount + $productQuantity;
                            }
                            $packCountRequest[] = array("bulkID" => $variantDetails['sku'], "qty" => $productQuantity);
                            if ($freePill) {
                                continue;
                            }
                            if (isset($variantDetails['price'])) {
                                if (($otp || $otp == 'true') && $otpExtra > 0) {
                                  $this->totalPrice = $this->totalPrice + (((float)$variantDetails['price'] * $productQuantity)/$otpExtra);
                                } else {
                                   $this->totalPrice = $this->totalPrice +  ((float)$variantDetails['price'] * $productQuantity);
                                }
                            }
                        }
                    }
                }
            }
        }
        if ($productVariantErrorStatus == 1) {
            $errorMessage = "Failed to get the requested ".$productValue." details from Shopify";
            Log::channel('checkoutLog')->info("CreateCheckout: ".$errorMessage.", Requested property:".$variantError);
            return $this->returnData("Failed", $errorMessage, $request->input('callback'));
        }
        //Setting free_shipping_threshold property to checkout LIP
        $shopifyShippingMaxThreshold = '';
        $shopifyNonDomesticShippingPrice = '';
        $subscriptionShippingZoneIds = [96075317330,94991646791];

        $shippingZones = $this->productObj->getShopifyShippingZone();

        if (!empty($shippingZones['error']) || !empty($shippingZones['errors']) ||
            !isset($shippingZones)) {
            $status = "failed";
            $error  = "Unknown error";
            if (!empty($shippingZones['error'])) {
                $error = json_encode($shippingZones['error']);
            } else if (!empty($shippingZones['errors'])) {
                $error = json_encode($shippingZones['errors']);
            } else if (!isset($shippingZones)) {
                $error = "Failed to get the Shopify shipping zone details";
            }
            Log::channel('checkoutLog')->info("CreateCheckout: Failed => " .$error);
            $response = array(
                "status"       => "Failed",
                "errorMessage" => "Failed to get the Shopify shipping zone details",
            );
            echo $request->input('callback') . '(' . json_encode($response) . ')';
            //exit();
        }
        if (isset($shippingZones['shipping_zones'])) {
            foreach ($shippingZones['shipping_zones'] as $shipValue) {
                $shippingZoneId = !empty($shipValue['id']) ? trim($shipValue['id']) : '';
                $shippingZoneName = !empty($shipValue['name']) ? strtolower(trim($shipValue['name'])) : '';
                if (strpos($shipValue['name'], ",") !== false ) {
                    $shippingZoneName = str_replace(" ", "", $shippingZoneName);
                    $shippingZoneName = str_replace(",", "-", $shippingZoneName);
                }
                if ($shippingZoneName == 'domestic' && in_array($shippingZoneId, $subscriptionShippingZoneIds)) {
                    if (!empty($shipValue['price_based_shipping_rates'])) {
                        foreach ($shipValue['price_based_shipping_rates'] as $priceValue) {
                            $shippingName = strtolower(trim($priceValue['name']));
                            $shippingName = str_replace(" ", "-", $shippingName);
                            if ($shippingName == 'standard-shipping') {
                                $shopifyShippingMaxThreshold = $priceValue['max_order_subtotal'];
                                $shopifyShippingMaxThreshold = number_format((float)$shopifyShippingMaxThreshold, 2, '.', '');

                            }
                        }
                    }
                }
            }
        }
        //Gift pack LIP setting
        $giftStatus = '';
        $gifteeFirstName = '';
        $gifteeLastName = '';
        $gitePackName = '';
        $gifteeEmail = '';
        $giftNumberChargesUntilExpiration = '';
        $giftNotificationDate = '';
        $giftNote = '';
        $giftSendersFirstName = '';
        $giftSendersLastName = '';
        $giftSubscriptionNumberOfCharges = '';
        $expireAfterSpecificNumberOfCharges = null;
        if ($giftPackStatus == 1 && !empty($giftPackDetails)) {
            $chargeIntervalFrequency = !empty($receivedData['checkout']['line_items'][0]['charge_interval_frequency']) ?
            (int)$receivedData['checkout']['line_items'][0]['charge_interval_frequency'] : '';
            $checkoutGiftPackDetails = !empty($giftPackDetails) ? 
            $giftPackDetails : '';
            $gifteeFirstName = !empty($checkoutGiftPackDetails['giftee_first_name']) ? 
            trim($checkoutGiftPackDetails['giftee_first_name']) : '';
            $gifteeLastName = !empty($checkoutGiftPackDetails['giftee_last_name']) ? 
            trim($checkoutGiftPackDetails['giftee_last_name']) : '';
            $gitePackName = !empty($checkoutGiftPackDetails['gift_pack_name']) ? 
            trim($checkoutGiftPackDetails['gift_pack_name']) : '';
            $gifteeEmail = !empty($checkoutGiftPackDetails['giftee_email']) ? 
            trim($checkoutGiftPackDetails['giftee_email']) : '';
            $giftSubscriptionNumberOfCharges = !empty($checkoutGiftPackDetails['gift_number_charges_until_expiration']) ? 
            trim($checkoutGiftPackDetails['gift_number_charges_until_expiration']) : 1;
            $giftNotificationDate = !empty($checkoutGiftPackDetails['gift_notification_date']) ? 
            trim($checkoutGiftPackDetails['gift_notification_date']) : '';
            $giftNote = !empty($checkoutGiftPackDetails['gift_note']) ? 
            trim($checkoutGiftPackDetails['gift_note']) : '';
            $giftSendersFirstName = !empty($checkoutGiftPackDetails['gift_senders_first_name']) ? 
            trim($checkoutGiftPackDetails['gift_senders_first_name']) : '';
            $giftSendersLastName = !empty($checkoutGiftPackDetails['gift_senders_last_name']) ? 
            trim($checkoutGiftPackDetails['gift_senders_last_name']) : '';
            $giftPackNumberCharges = '';
            $giftPackNumberFromFrequency = '';
            if (!empty($chargeIntervalFrequency)) {
                $giftPackNumberFromFrequency = (int)$chargeIntervalFrequency / 28;
                $giftNumberChargesUntilExpiration = $giftPackNumberFromFrequency;
                $giftPackNumberCharges = !empty($giftPackNumberFromFrequency) ? (int)$giftPackNumberFromFrequency : 1;
                $this->totalPrice = (float)$this->totalPrice * $giftPackNumberCharges;
            }
        } 
        $packCount = [];
        // if (isset($packCountRequest)) {
        //     $packCount = $this->getProductPackCount($packCountRequest);
        //     Log::channel('checkoutLog')->info("CreateCheckout: ProductPackCount => " .json_encode($packCount));
        // }
        if (isset($packCount) && !empty($packCount)) {
            $productPackCount = !empty($packCount['Pack']) ? "{$packCount['Pack']}" : (!empty($receivedData['checkout']['line_items'][0]['Packs']) ? 
            trim($receivedData['checkout']['line_items'][0]['Packs']) : "");
        } else {
            $productPackCount = !empty($receivedData['checkout']['line_items'][0]['Packs']) ? 
            trim($receivedData['checkout']['line_items'][0]['Packs']) : "";
        }
        if (!empty($productPackCount)) {
            $lineItemProperties['Packs'] = $productPackCount;
        }
        if ($melatoninCount > 4) {
            $status = true;
            $lineItemProperties['melatonin_warning'] = "{$status}";
        }
        if (!empty($utmParameter)) {
            $lineItemProperties['utm'] = $utmParameter;
        }
        if (!empty($shippingOverRideStatus)) {
            $lineItemProperties['override_free_shipping'] = $shippingOverRideStatus;
        }
        if (!empty($specialPackType)) {
            $lineItemProperties['special_pack_type'] = $specialPackType;
        }
        if (!empty($partnerValue)) {
            $lineItemProperties['partner'] = $partnerValue;
        }
        if (!empty($campaignValue)) {
            $lineItemProperties['campaign'] = $campaignValue;
        }
        if (!empty($firstName)) {
            $lineItemProperties['first_name'] = $firstName;
        }
        if (!empty($source)) {
            $lineItemProperties['source'] = $source;
        }
        if (!empty($medium)) {
            $lineItemProperties['medium'] = $medium;
        }
        if (!empty($content)) {
            $lineItemProperties['content'] = $content;
        }
        if (!empty($term)) {
            $lineItemProperties['term'] = $term;
        }
        if (!empty($quizId)) {
            $lineItemProperties['quiz_id'] = $quizId;
        }
        if (!empty($campaignId)) {
            $lineItemProperties['campaign_id'] = $campaignId;
        }
        if (!empty($templateId)) {
            $lineItemProperties['template_id'] = $templateId;
        }
        if (!empty($futureMonthFree)) {
            $lineItemProperties['future_month_free'] = $futureMonthFree;
        }
        if (!empty($corp_discount)) {
            $lineItemProperties['corp_discount'] = $corp_discount;
        }
        if (!empty($giftToSub)) {
            $lineItemProperties['giftToSub'] = $giftToSub;
        }
        if ($otp || $otp == 'true') {
            $lineItemProperties['one_time_purchase'] = 'true';
            $this->totalPrice = number_format((float)$this->totalPrice, 2, '.', '');
        }
        $lineItemProperties['free_shipping_threshold'] = $shopifyShippingMaxThreshold;
        if ($giftPackStatus == 1 && !empty($giftPackDetails)) {
            if (!empty($gifteeFirstName)) {
                $lineItemProperties['giftee_first_name'] = $gifteeFirstName;
            }
            if (!empty($gifteeLastName)) {
                $lineItemProperties['giftee_last_name'] = $gifteeLastName;
            }
            if (!empty($gitePackName)) {
                $lineItemProperties['gift_pack_name'] = $gitePackName;
            }
            if (!empty($gifteeEmail)) {
                $lineItemProperties['giftee_email'] = $gifteeEmail;
            }
            if (!empty($giftNumberChargesUntilExpiration)) {
                $lineItemProperties['gift_charge_interval_frequency'] = $giftNumberChargesUntilExpiration;
            }
            if (!empty($giftNotificationDate)) {
                $lineItemProperties['gift_notification_date'] = $giftNotificationDate;
            }
            if (!empty($giftNote)) {
                $lineItemProperties['gift_note'] = $giftNote;
            }
            if (!empty($giftSendersFirstName)) {
                $lineItemProperties['gift_senders_first_name'] = $giftSendersFirstName;
            }
            if (!empty($giftSendersLastName)) {
                $lineItemProperties['gift_senders_last_name'] = $giftSendersLastName;
            }
            $expireAfterSpecificNumberOfCharges = $giftSubscriptionNumberOfCharges;
        }
        $lineItems = [];
        if (isset($receivedData['checkout']['line_items'])) {
            foreach ($receivedData['checkout']['line_items'] as $checkoutValue) {
                $lineItemTemp = array(
                    "price"                                   => !empty($this->totalPrice) ? $this->totalPrice : '',
                    "product_id"                              => !empty($checkoutValue['product_id']) ? $checkoutValue['product_id'] : '',
                    "properties"                              => isset($lineItemProperties) ? $lineItemProperties : '',
                    "quantity"                                => !empty($checkoutValue['quantity']) ? $checkoutValue['quantity'] : 1,
                    "variant_id"                              => $checkoutValue['variant_id'],
                    "cutoff_day_of_month"                     => !empty($checkoutValue['cutoff_day_of_month'])
                        ? $checkoutValue['cutoff_day_of_month'] : '',
                    "cutoff_day_of_week"                      => !empty($checkoutValue['cutoff_day_of_week'])
                        ? $checkoutValue['cutoff_day_of_week'] : '',
                    "expire_after_specific_number_of_charges" => $expireAfterSpecificNumberOfCharges,
                    "fulfillment_service"                     => "manual",
                    "grams"                                   => !empty($checkoutValue['grams']) ? $checkoutValue['grams'] : '',
                    "line_price"                              => !empty($checkoutValue['line_price']) ? $checkoutValue['line_price'] : '',
                    "order_day_of_month"                      => !empty($checkoutValue['order_day_of_month'])
                        ? $checkoutValue['order_day_of_month'] : '',
                    "order_day_of_week"                       => !empty($checkoutValue['order_day_of_week'])
                        ? $checkoutValue['order_day_of_week'] : '',
                    "sku"                                     => !empty($checkoutValue['sku']) ? $checkoutValue['sku'] : '',
                    "title"                                   => !empty($checkoutValue['title']) ? $checkoutValue['title'] : '',
                    "variant_title"                           => !empty($checkoutValue['variant_title'])
                        ? $checkoutValue['variant_title'] : '',
                    "vendor"                                  => !empty($checkoutValue['vendor']) ? $checkoutValue['vendor'] : '',
                );
                if ((!$otp || $otp == 'false') && $giftPackStatus != 1) {
                    $lineItemTemp["charge_interval_frequency"] = $checkoutValue['charge_interval_frequency'];
                    $lineItemTemp["order_interval_frequency"]  = $checkoutValue['order_interval_frequency'];
                    $lineItemTemp["order_interval_unit"]       = $checkoutValue['order_interval_unit'];
                }
                if ($giftPackStatus == 1 && !empty($giftPackDetails)) {
                    // $lineItemTemp["charge_interval_frequency"] = $giftNumberChargesUntilExpiration;
                    $lineItemTemp["charge_interval_frequency"] = $checkoutValue['charge_interval_frequency'];
                    $lineItemTemp["order_interval_frequency"]  = !empty($checkoutValue['order_interval_frequency']) ?
                    $checkoutValue['order_interval_frequency'] : 1;
                    $lineItemTemp["order_interval_unit"]       = !empty($checkoutValue['order_interval_unit']) ?
                    $checkoutValue['order_interval_unit'] : "month";
                }
                foreach ($lineItemTemp as $key => $value) {
                    if (empty($value)) {
                        unset($lineItemTemp[$key]);
                    }
                }
                $lineItems[] = $lineItemTemp;
            }
        }
        if (!empty($receivedData['checkout']['shipping_address'])) {
            $shippingAddress = array(
                "address1"   => !empty($receivedData['checkout']['shipping_address']['address1'])
                    ? $receivedData['checkout']['shipping_address']['address1'] : '',
                "address2"   => !empty($receivedData['checkout']['shipping_address']['address2'])
                    ? $receivedData['checkout']['shipping_address']['address2'] : '',
                "city"       => !empty($receivedData['checkout']['shipping_address']['city'])
                    ? $receivedData['checkout']['shipping_address']['city'] : '',
                "company"    => !empty($receivedData['checkout']['shipping_address']['company'])
                    ? $receivedData['checkout']['shipping_address']['company'] : '',
                "country"    => !empty($receivedData['checkout']['shipping_address']['country'])
                    ? $receivedData['checkout']['shipping_address']['country'] : '',
                "first_name" => !empty($receivedData['checkout']['shipping_address']['first_name'])
                    ? $receivedData['checkout']['shipping_address']['first_name'] : '',
                "last_name"  => !empty($receivedData['checkout']['shipping_address']['last_name'])
                    ? $receivedData['checkout']['shipping_address']['last_name'] : '',
                "phone"      => !empty($receivedData['checkout']['shipping_address']['phone'])
                    ? $receivedData['checkout']['shipping_address']['phone'] : '',
                "province"   => !empty($receivedData['checkout']['shipping_address']['province'])
                    ? $receivedData['checkout']['shipping_address']['province'] : '',
                "zip"        => !empty($receivedData['checkout']['shipping_address']['zip'])
                    ? $receivedData['checkout']['shipping_address']['zip'] : ''

            );
            foreach ($shippingAddress as $key => $value) {
                if (empty($value)) {
                    unset($shippingAddress[$key]);
                }
            }
        }
        $checkoutRequest = array(
            "checkout" => array(
                "line_items"               => $lineItems,
                "shipping_address"         => $shippingAddress,
                "email"                    => !empty($receivedData['checkout']['email']) ? $receivedData['checkout']['email'] : '',
                "external_checkout_id"     => !empty($receivedData['checkout']['external_checkout_id'])
                    ? $receivedData['checkout']['external_checkout_id'] : '',
                "external_checkout_source" => !empty($receivedData['checkout']['external_checkout_source'])
                    ? $receivedData['checkout']['external_checkout_source'] : '',
                "discount_code"            => !empty($receivedData['checkout']['discount_code'])
                    ? $receivedData['checkout']['discount_code'] : ''
            )
        );
        foreach ($checkoutRequest['checkout'] as $key => $value) {
            if (empty($value)) {
                unset($checkoutRequest['checkout'][$key]);
            }
        }
        Log::channel('checkoutLog')->info("CreateCheckout: Checkout request => ".json_encode($checkoutRequest));

        $checkoutReturn = $this->modelObj->createCheckout(json_encode($checkoutRequest));
        
        Log::channel('checkoutLog')->info("CreateCheckout: Response => ".json_encode($checkoutReturn));        

        if (isset($checkoutReturn->error) || isset($checkoutReturn->errors) ||
            empty($checkoutReturn) || empty($checkoutReturn->checkout->token)) {
            $status = "failed";
            $error  = "Unknown error";
            if (isset($checkoutReturn->error)) {
                $error = json_encode($checkoutReturn->error);
            } else if (isset($checkoutReturn->errors)) {
                $error = json_encode($checkoutReturn->errors);
            } else if (empty($checkoutReturn) || empty($checkoutReturn->checkout->token)) {
                $error = "Checkout created response is empty";
            }
            Log::channel('checkoutLog')->info("CreateCheckout: CheckoutError => ".$error);
            $errorMessage = "Failed to create the Checkout";
            Log::channel('checkoutLog')->info("CreateCheckout: ".$errorMessage);
            return $this->returnData("Failed", $errorMessage, $request->input('callback'));
        }
        $checkoutUrl = env('NURISH_CHECKOUT_URL', true) . $checkoutReturn->checkout->token .
            '?myshopify_domain=' . env('SHOPIFY_STORE', true);
        $successResponse = $request->input('callback') . '({"status":"success","url": "' . $checkoutUrl . '",
            "override_free_shipping": "' .$shippingOverRideStatus. '",
            "special_pack_type": "'.$specialPackType.'","partner": "'.$partnerValue.'",
            "campaign": "'.$campaignValue.'","first_name": "'.$firstName.'",
            "source": "'.$source.'","medium": "'.$medium.'","content": "'.$content.'",
            "term": "'.$term.'","quiz_id": "'.$quizId.'","campaign_id": "'.$campaignId.'",
            "template_id": "'.$templateId.'","future_month_free": "'.$futureMonthFree.'", 
            "corp_discount": "'.$corp_discount.'","errorMessage":""})';

        echo $successResponse;
        exit();
    }

    /**
     * Response function
     */
    public function returnData($status, $errorMessage, $callBackToken) {

        $response = array(
            "status"       => $status,
            "errorMessage" => $errorMessage
        );
        echo $callBackToken . '(' . json_encode($response) . ')';
        exit();
    }

    /**
     * Update checkout
     */
    public function updateSubscriptionLineItemProperties(Request $request) {

        $requestData = $request->input('data');
        if (empty($requestData)) {
            $message = "Endpoint parameters are missing";
            $error = "Bad request";
            $status = "Failed";
            $response = array(
                "status"   => $status,
                "message" => $message,
                "error" => $error
            );
            Log::channel('checkoutLog')->info("updateSubscriptionLineItemProperties: Endpoint 
            parameters are missing");
            echo json_encode($response);
            exit();
        }
        Log::channel('checkoutLog')->info("updateSubscriptionLineItemProperties: Request => ".$requestData);
        $receivedData = json_decode($requestData);
        $subscriptionID = !empty($receivedData->subscription_id) ? $receivedData->subscription_id : "";
        if (empty($subscriptionID) && !isset($receivedData->properties)) {
            $errorMessage = "The endpoint request is not valid";
            Log::channel('checkoutLog')->info("updateSubscriptionLineItemProperties: ".$errorMessage);
            return $this->returnData("Failed", $errorMessage, $request->input('callback'));
        }
        if (empty($subscriptionID)) {
            $errorMessage = "The subscription id is missing in the request";
            Log::channel('checkoutLog')->info("updateSubscriptionLineItemProperties: ".$errorMessage);
            return $this->returnData("Failed", $errorMessage, $request->input('callback'));
        }
        if (!isset($receivedData->properties)) {
            $errorMessage = "Missing line item properties in the request";
            Log::channel('checkoutLog')->info("updateSubscriptionLineItemProperties: ".$errorMessage);
            return $this->returnData("Failed", $errorMessage, $request->input('callback'));
        }
        $pillsInProperties = 0;
        $propertyValidity = 1;
        $nonPillProperties = ['utm', 'override_free_shipping', 'special_pack_type', 'partner', 'campaign', 'first_name',
        'source', 'medium', 'content', 'term', 'quiz_id', 'campaign_id', 'template_id', 'future_month_free', 'corp_discount',
        'giftToSub'];
        foreach ($receivedData->properties as $propertyValue) {
            $nonPillPropertyStatus = 0;
            foreach ($nonPillProperties as $nonPillKey => $nonPillValue) {
                if (isset($propertyValue->$nonPillValue)) {
                    $nonPillPropertyStatus = 1;
                }
            }
            if ($nonPillPropertyStatus == 1) {
                continue;
            }
            $requestedProductId = !empty($propertyValue->product_id) ? trim($propertyValue->product_id) : "";
            $requestedPdtVariantId = !empty($propertyValue->variant_id) ? trim($propertyValue->variant_id) : "";
            $requestedProductName = !empty($propertyValue->name) ? trim($propertyValue->name) : "";
            $requestedProductValue = !empty($propertyValue->value) ? trim($propertyValue->value) : "";
            $requestedDosage = !empty($propertyValue->dosage) ? trim($propertyValue->dosage) : "";
            if (!empty($requestedProductId) || !empty($requestedPdtVariantId) || !empty($requestedProductName) ||
                !empty($requestedProductValue) ||  !empty($requestedDosage)) {
                $pillsInProperties = 1; 
            }
            if (empty($requestedProductId) || empty($requestedPdtVariantId) || empty($requestedProductName) ||
                empty($requestedProductValue) || empty($requestedDosage)) {
                    $propertyValidity = 0;
            }
        }
        if ($pillsInProperties == 0) {
            $errorMessage = "Not found any pills in the line item properties";
            Log::channel('checkoutLog')->info("updateSubscriptionLineItemProperties: ".$errorMessage);
            return $this->returnData("Failed", $errorMessage, $request->input('callback'));
        }
        if ($propertyValidity == 0) {
            $errorMessage = "Missing required parameters in the line item properties";
            Log::channel('checkoutLog')->info("updateSubscriptionLineItemProperties: ".$errorMessage);
            return $this->returnData("Failed", $errorMessage, $request->input('callback'));
        }
        if (isset($receivedData->properties)) {
            $rearrangeProperties = array();
            $i = 0;
            $j = 1;
            foreach ($receivedData->properties as $propertyValue) {
                $i++;
                if (!empty($propertyValue->variant_id)) {
                if (isset($rearrangeProperties[$propertyValue->variant_id])) {
                   $rearrangeProperties[$propertyValue->variant_id]->quantity += $propertyValue->quantity;
                } else {
                    $propertyValue->name =  'Vitamin'. ($j<10? '0'.$j++:$j++);
                    $rearrangeProperties[$propertyValue->variant_id] = $propertyValue;
                }
              } else {
                if (!empty($propertyValue->utm)) {
                    $rearrangeProperties['utm'] =  (object)["name" => "utm","value" => $propertyValue->utm];
                } elseif (isset($propertyValue->override_free_shipping)) {
                    $rearrangeProperties['override_free_shipping'] = ($propertyValue->override_free_shipping == "true") ?
                    (object)["name" => "override_free_shipping", "value" => "true"] :
                    (object)["name" => "override_free_shipping", "value" => "false"];
                } elseif (!empty($propertyValue->special_pack_type)) {
                    $rearrangeProperties['special_pack_type'] =
                    (object)["name" => "special_pack_type","value" => $propertyValue->special_pack_type];
                } elseif (!empty($propertyValue->partner)) {
                    $rearrangeProperties['partner'] =
                    (object)["name" => "partner","value" => $propertyValue->partner];
                } elseif (!empty($propertyValue->campaign)) {
                    $rearrangeProperties['campaign'] =
                    (object)["name" => "campaign","value" => $propertyValue->campaign];
                } elseif (!empty($propertyValue->first_name)) {
                    $rearrangeProperties['first_name'] =
                    (object)["name" => "first_name","value" => $propertyValue->first_name];
                } elseif (!empty($propertyValue->source)) {
                    $rearrangeProperties['source'] =
                    (object)["name" => "source","value" => $propertyValue->source];
                } elseif (!empty($propertyValue->medium)) {
                    $rearrangeProperties['medium'] =
                    (object)["name" => "medium","value" => $propertyValue->medium];
                } elseif (!empty($propertyValue->content)) {
                    $rearrangeProperties['content'] =
                    (object)["name" => "content","value" => $propertyValue->content];
                } elseif (!empty($propertyValue->term)) {
                    $rearrangeProperties['term'] =
                    (object)["name" => "term","value" => $propertyValue->term];
                } elseif (!empty($propertyValue->quiz_id)) {
                    $rearrangeProperties['quiz_id'] =
                    (object)["name" => "quiz_id","value" => $propertyValue->quiz_id];
                }
              }
            }
            unset($receivedData->properties);
            $receivedData->properties = array_values($rearrangeProperties);
        }
        $lineItemProperty = [];
        $productSKU       = [];
        $packCountRequest = [];
        $melatoninProductIds = [1837729808455, 1837729775687, 4329622405191];
        $melatoninCount = 0;
        $productVariantErrorStatus = 0;
        $variantError = '';
        $productValue = '';
        $elderberryQuantity = 0;
        $elderberryQuantityAdjustedRequest = [];
        $propertyParam = !empty($receivedData->properties) ? json_decode(json_encode($receivedData->properties), true) : '';
        $elderberryQuantity = $this->getElderBerryQuantity($propertyParam);
        if ($elderberryQuantity > 1) {
            $errorMessage = "Found more than 1 Elderberry bottle in the request";
            Log::channel('checkoutLog')->info("updateSubscriptionLineItemProperties: Error - ".$errorMessage);
            return $this->returnData("Failed", $errorMessage, $request->input('callback'));
        }

        $shopifyProductDetails = $this->getShopifyProductDetails();

        if (empty($shopifyProductDetails) || !isset($shopifyProductDetails)) {
            $errorMessage = "Failed to get the product details from Shopify";
            Log::channel('checkoutLog')->info("updateSubscriptionLineItemProperties: Error - ".$errorMessage);
            return $this->returnData("Failed", $errorMessage, $request->input('callback'));
        }
        if (!empty($receivedData->properties)) {
            foreach ($receivedData->properties as $itemValue) {
                if (!empty($itemValue->product_id) && !empty($itemValue->variant_id)) {
                    $variantDetails = [];
                    $requestedVariantId = $itemValue->variant_id;
                    $variantDetails = !empty($shopifyProductDetails[$requestedVariantId]) ? 
                    $shopifyProductDetails[$requestedVariantId] : '';
                    if (empty($variantDetails) || !isset($variantDetails)) {
                        $productVariantErrorStatus = 1;
                        $productValue = $itemValue->value;
                        $variantError = json_encode($itemValue);
                        break;
                    }
                    $productSKU[$itemValue->variant_id] = $variantDetails['sku'];
                    if (empty($itemValue->quantity)) {
                        $productQuantity = 1;
                    } else {
                        $productQuantity = $itemValue->quantity;
                    }
                    if (in_array($itemValue->product_id, $melatoninProductIds)) {
                        $melatoninCount = $melatoninCount + $productQuantity;
                    }
                    $packCountRequest[] = array("bulkID" => $variantDetails['sku'], "qty" => $productQuantity);
                    if (isset($itemValue->free_pill) && !empty($itemValue->free_pill)) {
                        continue;
                    }
                    if (isset($variantDetails['price'])) {
                        $this->totalPrice = $this->totalPrice + ($variantDetails['price'] * $productQuantity);
                    }
                }
            }
        }
        if ($productVariantErrorStatus == 1) {
            $errorMessage = "Failed to get the requested ".$productValue." details from Shopify";
            Log::channel('checkoutLog')->info("updateSubscriptionLineItemProperties: Error - ".$errorMessage.", Requested property: ".$variantError);
            return $this->returnData("Failed", $errorMessage, $request->input('callback'));
        }
        $productPackCount = '';
        $packCount = '';
        if (isset($packCountRequest)) {
            $packCount = $this->getProductPackCount($packCountRequest);
            Log::channel('checkoutLog')->info("updateSubscriptionLineItemProperties: ProductPackCount => " .json_encode($packCount));
        }
        if (isset($packCount)) {
            $productPackCount = !empty($packCount['Pack']) ? "{$packCount['Pack']}" : "";
        }
        foreach ($receivedData->properties as $propertyValue) {
            $value = '';
            $quantity = 1;
            if (isset($propertyValue->product_id) && isset($propertyValue->variant_id)) {
                $freePill = '';
                if (isset($propertyValue->free_pill) && !empty($propertyValue->free_pill)) {
                    $freePill = ' | free_pill';
                }
                if (!empty($propertyValue->quantity)) {
                    $quantity = $propertyValue->quantity;

                    $value = $propertyValue->value . (!empty($propertyValue->dosage) ?
                            ' | ' . $propertyValue->dosage : ' | '. 'Default Title') . ' | ' . $quantity . ' | ' .
                        $propertyValue->variant_id . ' | ' . $productSKU[$propertyValue->variant_id] . $freePill;
                } else {
                    $value = $propertyValue->value . (!empty($propertyValue->dosage) ?
                            ' | ' . $propertyValue->dosage : ' | '. 'Default Title') . ' | ' . $quantity . ' | ' .
                        $propertyValue->variant_id . ' | ' . $productSKU[$propertyValue->variant_id] . $freePill;
                }
                $property = array(
                    "name"  => $propertyValue->name,
                    "value" => $value
                );
                $lineItemProperty[] = $property;
            } else {
                $property = array(
                    "name"  => $propertyValue->name,
                    "value" => $propertyValue->value
                );
                $lineItemProperty[] = $property;
            }
        }
        if (!empty($productPackCount)) {
            $packCountArr = array(
                "name"  => "Packs",
                "value" => $productPackCount
            );
            $lineItemProperty[] = $packCountArr;
        }
        if ($melatoninCount > 4) {
            $melatoninWarning = array(
                "name"  => "melatonin_warning",
                "value" => true
            );
            $lineItemProperty[] = $melatoninWarning;
        }
        if (!empty($subscriptionID)) {
            $subscriptionDetails = $this->modelObj->getSubscription($subscriptionID);
            if (!empty($subscriptionDetails->error) || !empty($subscriptionDetails->errors) ||
                empty($subscriptionDetails)) {
                $status = "failed";
                $error  = "Unknown error";
                if (!empty($subscriptionDetails->error)) {
                    $error = json_encode($subscriptionDetails->error);
                } else if (!empty($subscriptionDetails->errors)) {
                    $error = json_encode($subscriptionDetails->errors);
                } else if (empty($subscriptionDetails)) {
                    $error = "Failed to get the subscription details by subscription ID";
                }
                $errorMessage = "Failed to get the subscription details by subscription ID";
                Log::channel('checkoutLog')->info("updateSubscriptionLineItemProperties: Failed => " .$error.", 
                SubscriptionId:".$subscriptionID);
                return $this->returnData("Failed", $errorMessage, $request->input('callback'));
            }
        }
        $lineItemProperties = [];
        $addedProperties = [];
        $vitaminUpdateStatus = 0;
        $checkOverRideFreeShippingStatus = 0;
        $freeShippingThreshold = 0;
        if (!empty($subscriptionDetails->subscription->properties)) {
            $subscriptionDetailsInArr = json_decode(json_encode($subscriptionDetails), true);
            foreach ($subscriptionDetailsInArr['subscription']['properties'] as $subKey => $subPropertyValue) {
                $changedPropertyData = '';
                $vitaminProductDetails = [];
                foreach ($lineItemProperty as $lineKey => $lineItemValue) {
                    $vitaminNameProperty = strtolower(trim($lineItemValue['name']));
                    if (strtolower(trim($subPropertyValue['name'])) == strtolower(trim($lineItemValue['name']))
                    && strpos($vitaminNameProperty, 'vitamin') === false) {
                        $changedPropertyData = $lineItemValue;
                    }
                    if (strpos($vitaminNameProperty, 'vitamin') !== false) {
                        $vitaminProductDetails[] = $lineItemValue;
                    }
                }
                $vitaminName = strtolower(trim($subPropertyValue['name']));
                if (!empty($changedPropertyData)) {
                    $lineItemProperties[] = $changedPropertyData;
                    $addedProperties[] = strtolower($changedPropertyData['name']);
                } else {
                    if (strpos($vitaminName, 'vitamin') === false) {
                        $lineItemProperties[] = $subPropertyValue;
                        $addedProperties[] = strtolower($subPropertyValue['name']);
                    }
                }
                if (strpos($vitaminName, 'vitamin') !== false && $vitaminUpdateStatus == 0) {
                    foreach ($vitaminProductDetails as $vitaminValue) {
                        $lineItemProperties[] = $vitaminValue;
                    }
                    $vitaminUpdateStatus = 1;
                }
                $subscriptionPropertyCheck = strtolower(trim($subPropertyValue['name']));
                if ($subscriptionPropertyCheck == 'override_free_shipping') {

                    $shippingOverrideValue = !empty($subPropertyValue['value']) ? $subPropertyValue['value'] : false;
                    if ($shippingOverrideValue && $shippingOverrideValue == "true") {
                        
                        $checkOverRideFreeShippingStatus = 1;
                    }
                }
                if ($subscriptionPropertyCheck == 'free_shipping_threshold') {
                    $shippingThresholdValue = !empty($subPropertyValue['value']) ? trim($subPropertyValue['value']) : 
                    number_format((float)0, 2, '.', '');
                    if (!empty($shippingThresholdValue)) {
                        
                        $freeShippingThreshold = $shippingThresholdValue;
                        $freeShippingThreshold = round($freeShippingThreshold);
                        $freeShippingThreshold = number_format((float)$freeShippingThreshold, 2, '.', '');
                    }
                }
            }
        }
        if (isset($addedProperties) && !empty($addedProperties)) {
            $notAddedProperty = [];
            foreach ($lineItemProperty as $lineKey => $lineItemValue) {
                $vitaminNameProperty = strtolower(trim($lineItemValue['name']));
                if (in_array($vitaminNameProperty, $addedProperties) == false
                && strpos($vitaminNameProperty, 'vitamin') === false) {
                    $notAddedProperty[] = $lineItemValue;
                }
            }
            if (!empty($notAddedProperty)) {
                foreach ($notAddedProperty as $notAddedValue) {
                    $lineItemProperties[] = $notAddedValue;
                }
            }
        }
        $updateRequest = array(
            "price"      => $this->totalPrice,
            "properties" => $lineItemProperties
        );
        $subscriptionShippingZoneIds = [96075317330,96075284562,94991646791,94991679559];
        if ($checkOverRideFreeShippingStatus == 0) {
            $usShippingRateAdjustableCountries = array(

                "alabama","arizona","arkansas","california","colorado","connecticut","delaware","florida","georgia",
                "idaho","illinois","indiana","iowa","kansas","kentucky","louisiana","maine","maryland","massachusetts",
                "michigan","minnesota","mississippi","missouri","montana","nebraska","nevada","new-hampshire",
                "new-jersey","new-mexico","new-york","north-carolina","north-dakota","ohio","oklahoma",
                "oregon","pennsylvania","rhode-island","south-carolina","south-dakota","tennessee","texas",
                "utah","vermont","virginia","washington","west-virginia","wisconsin","wyoming"
            );
            $usShippingRateNonAdjustableCountries = array(
                "alaska","hawaii"
            );
            $shopifyDomesticShippingPrice = '';
            $shopifyNonDomesticShippingPrice = '';
            $retry = false;
            $try = 1;
            $shippingZones = [];
            do {
                Log::channel('checkoutLog')->info("updateSubscriptionLineItemProperties".$try . ') try-get Shopify shipping zones');
                
                $shippingZones = $this->productObj->getShopifyShippingZone();

                if (!empty($shippingZones['errors']) || !empty($shippingZones['error']) || !isset($shippingZones)) {
                    sleep(1);
                    $retry = true;
                    $try++;
                }
            } while ($retry && $try <= 3);

            // $shippingZones = $this->productObj->getShopifyShippingZone();
            if (!empty($shippingZones['error']) || !empty($shippingZones['errors']) ||
                empty($shippingZones)) {
                $status = "failed";
                $error  = "Unknown error";
                if (!empty($shippingZones['error'])) {
                    $error = json_encode($shippingZones['error']);
                } else if (!empty($shippingZones['errors'])) {
                    $error = json_encode($shippingZones['errors']);
                } else if (empty($shippingZones)) {
                    $error = "Failed to get the Shopify shipping zone details";
                }
                $errorMessage = "Failed to get the Shopify shipping zone details";
                Log::channel('checkoutLog')->info("updateSubscriptionLineItemProperties: Failed => " .$error);
                return $this->returnData("Failed", $errorMessage, $request->input('callback'));
            }
            if (isset($shippingZones['shipping_zones'])) {
                foreach ($shippingZones['shipping_zones'] as $shipValue) {
                    $shippingZoneId = !empty($shipValue['id']) ? trim($shipValue['id']) : '';
                    $shippingZoneName = strtolower(trim($shipValue['name']));
                    if (strpos($shipValue['name'], ",") !== false ) {
                        $shippingZoneName = str_replace(" ", "", $shippingZoneName);
                        $shippingZoneName = str_replace(",", "-", $shippingZoneName);
                    }
                    if (($shippingZoneName == 'domestic' || $shippingZoneName == 'alaska-hawaii') && 
                    in_array($shippingZoneId, $subscriptionShippingZoneIds)) {
                        if (!empty($shipValue['price_based_shipping_rates'])) {
                            foreach ($shipValue['price_based_shipping_rates'] as $priceValue) {
                                $shippingName = strtolower(trim($priceValue['name']));
                                $shippingName = str_replace(" ", "-", $shippingName);

                                if ($shippingName == 'standard-shipping') {
                                    if ($shippingZoneName == 'domestic') {
                                        $shopifyDomesticShippingPrice = $priceValue['price'];
                                    } else {
                                        $shopifyNonDomesticShippingPrice = $priceValue['price'];
                                    }
                                }
                            }
                        }
                    }
                }
            }
            if (!empty($subscriptionDetails->subscription->address_id)) {

                $userAddressDetails = $this->modelObj->getAddressByAddressID($subscriptionDetails->subscription->address_id);

                if (!empty($userAddressDetails->error) || !empty($userAddressDetails->errors) ||
                empty($userAddressDetails)) {
                    $status = "failed";
                    $error  = "Unknown error";
                    if (!empty($userAddressDetails->error)) {
                        $error = json_encode($userAddressDetails->error);
                    } else if (!empty($userAddressDetails->errors)) {
                        $error = json_encode($userAddressDetails->errors);
                    } else if (empty($userAddressDetails)) {
                        $error = "Failed to get the address details by address ID";
                    }
                    $errorMessage = "Failed to get the address details to override the shipping details";
                    Log::channel('checkoutLog')->info("updateSubscriptionLineItemProperties: Failed => " .$error.", 
                    AddressId:".$subscriptionDetails->subscription->address_id);
                    return $this->returnData("Failed", $errorMessage, $request->input('callback'));
                }
            }
            $existingShippingPrice = '';
            if (!empty($userAddressDetails->address->original_shipping_lines)) {
                foreach ($userAddressDetails->address->original_shipping_lines as $shipPriceValue) {
                    $shippingName = strtolower(trim($shipPriceValue->title));
                    $shippingName = str_replace(" ", "-", $shippingName);
                    if ($shippingName == 'standard-shipping') {
                        $existingShippingPrice = $shipPriceValue->price;
                    }
                }
            }
            if (!empty($userAddressDetails->address->province)) {
                $addressProvince = strtolower(trim($userAddressDetails->address->province));
                $addressProvince = str_replace(" ", "-", $addressProvince);
            } else {
                $addressProvince = '';
            }
            $shippingUpdateStatus = 0;
            $shippingPrice = '';
            $shippingCode = '';
            $freeShippingThreshold = !empty($freeShippingThreshold) ? $freeShippingThreshold : number_format((float)20, 2, '.', '');
            if (!empty($addressProvince) && in_array($addressProvince, $usShippingRateAdjustableCountries)) {
                if ($this->totalPrice >= $freeShippingThreshold) {
                    $shippingPrice = 0;
                    $shippingCode = "Free Shipping";
                } else {
                    $shippingPrice = !empty($shopifyDomesticShippingPrice) ? $shopifyDomesticShippingPrice : 8;
                    $shippingCode = "Standard Shipping";
                }
                $shippingUpdateStatus = 1;
            } elseif (!empty($addressProvince) && in_array($addressProvince, $usShippingRateNonAdjustableCountries)) {
                $shippingPrice = !empty($shopifyNonDomesticShippingPrice) ? $shopifyNonDomesticShippingPrice : 8;
                $shippingCode = "Standard Shipping";
                $shippingUpdateStatus = 1;
            } else {
                if (!empty($freeShippingThreshold)) {
                    if ($this->totalPrice >= $freeShippingThreshold) {
                        $shippingPrice = 0;
                        $shippingCode = "Free Shipping";
                    }
                } else {
                    $shippingPrice = !empty($shopifyDomesticShippingPrice) ? $shopifyDomesticShippingPrice : 8;
                    $shippingCode = "Standard Shipping";
                }
                $shippingUpdateStatus = 1;
            }
            if ($shippingUpdateStatus == 1 && isset($subscriptionDetails) &&
            !empty($subscriptionDetails->subscription->address_id)) {
                $shippingUpdateRequest = array(
                    "shipping_lines_override" => array(
                    array(
                        "price" => $shippingPrice,
                        "code"  => $shippingCode,
                        "title" => "Standard Shipping"
                    )
                  )
                );

                $updatedShippingRateResponse = $this->modelObj->updateShippingRateByAddress(
                    $subscriptionDetails->subscription->address_id, json_encode($shippingUpdateRequest));

                if (isset($updatedShippingRateResponse->error) || isset($updatedShippingRateResponse->errors) ||
                empty($updatedShippingRateResponse)) {
                    $status = "failed";
                    $error  = "Unknown error";
                    if (isset($updatedShippingRateResponse->error)) {
                        $error = $updatedShippingRateResponse->error;
                    } else if (isset($updatedShippingRateResponse->errors)) {
                        $error = json_encode($updatedShippingRateResponse->errors);
                    } else if (empty($updatedShippingRateResponse)) {
                        $error = "Failed to update the Shipping amount";
                    }
                    $errorMessage = "Failed to update the shipping amount";
                    Log::channel('checkoutLog')->info("updateSubscriptionLineItemProperties: Failed => " .
                    $error.", AddressId:".$subscriptionDetails->subscription->address_id);
                    return $this->returnData("Failed", $errorMessage, $request->input('callback'));
                }
            }
        } else if ($checkOverRideFreeShippingStatus == 1) {
            $existingShippingPrice = '';
            $shippingPrice = '';
            $freeShippingThreshold = 0;
            $userAddressDetails = $this->modelObj->getAddressByAddressID($subscriptionDetails->subscription->address_id);
            if (!empty($userAddressDetails->error) || !empty($userAddressDetails->errors) ||
                empty($userAddressDetails)) {
                $status = "failed";
                $error  = "Unknown error";
                if (!empty($userAddressDetails->error)) {
                    $error = json_encode($userAddressDetails->error);
                } else if (!empty($userAddressDetails->errors)) {
                    $error = json_encode($userAddressDetails->errors);
                } else if (empty($userAddressDetails)) {
                    $error = "Failed to get the address details by address ID";
                }
                $errorMessage = "Failed to get the address details to override the shipping details";
                Log::channel('checkoutLog')->info("updateSubscriptionLineItemProperties: Failed => " .$error.", 
                AddressId:".$subscriptionDetails->subscription->address_id);
                return $this->returnData("Failed", $errorMessage, $request->input('callback'));
            }
            $addressOriginalShipping = !empty($userAddressDetails->address->original_shipping_lines[0]->price) ?
            (float)$userAddressDetails->address->original_shipping_lines[0]->price : 0;
            $addressOverRideFreeShipping = !empty($userAddressDetails->address->shipping_lines_override[0]->price) ? 
            (float)$userAddressDetails->address->shipping_lines_override[0]->price : 0;
            $existingShippingPrice = !empty($addressOverRideFreeShipping) ? $addressOverRideFreeShipping : 
            $addressOriginalShipping;
            if ($addressOverRideFreeShipping > 0 || $addressOriginalShipping > 0) {
                $shippingUpdateRequest = array(

                    "shipping_lines_override" => array(
                        array(
                            "code" => "Free shipping",
                            "price"=> "0.00",
                            "title"=> "Free shipping"
                        )
                    )
                );

                $updatedShippingRateResponse = $this->modelObj->updateShippingRateByAddress(
                $subscriptionDetails->subscription->address_id, json_encode($shippingUpdateRequest));

                if (isset($updatedShippingRateResponse->error) || isset($updatedShippingRateResponse->errors) ||
                empty($updatedShippingRateResponse)) {
                    $status = "failed";
                    $error  = "Unknown error";
                    if (isset($updatedShippingRateResponse->error)) {
                        $error = json_encode($updatedShippingRateResponse->error);
                    } else if (isset($updatedShippingRateResponse->errors)) {
                        $error = json_encode($updatedShippingRateResponse->errors);
                    } else if (empty($updatedShippingRateResponse)) {
                        $error = "Failed to update the Shipping amount";
                    }
                    $errorMessage = "Failed to update the shipping amount";
                    Log::channel('checkoutLog')->info("updateSubscriptionLineItemProperties: Failed => " .
                    $error.", AddressId:".$subscriptionDetails->subscription->address_id);
                    return $this->returnData("Failed", $errorMessage, $request->input('callback'));
                }
            }
            $shippingPrice = 0;
        }

        $subscriptionResponse = $this->modelObj->updateSubscription($subscriptionID, json_encode($updateRequest));

        if (isset($subscriptionResponse->error) || isset($subscriptionResponse->errors) ||
            empty($subscriptionResponse)) {
            $status = "failed";
            $error  = "Unknown error";
            if (isset($subscriptionResponse->error)) {
                $error = json_encode($subscriptionResponse->error);
            } else if (isset($subscriptionResponse->errors)) {
                $error = json_encode($subscriptionResponse->errors);
            }
            $errorMessage = "Failed to update the subscription";
            Log::channel('checkoutLog')->info("updateSubscriptionLineItemProperties: Error => " .$error);
            return $this->returnData("Failed", $errorMessage, $request->input('callback'));
        }
        $this->totalPrice = number_format((float)$this->totalPrice, 2, '.', '');
        $shippingPrice = number_format((float)$shippingPrice, 2, '.', '');
        $freeShippingThreshold = number_format((float)$freeShippingThreshold, 2, '.', '');
        $response         = array(
            "status"             => "success",
            "subscription_price" => $this->totalPrice,
            "melatonin_warning"  => $melatoninCount > 4 ? true : false,
            "shipping_amount"    => $shippingPrice,
            "free_shipping_threshold" => $freeShippingThreshold,
            "shipping_rate_update_status" => trim($existingShippingPrice) != trim($shippingPrice) ? true : false,
            "errorMessage"       => "",
        );
        Log::channel('checkoutLog')->info("updateSubscriptionLineItemProperties: Successfully updated the subscription, 
        Response => ".json_encode($response));
        echo $request->input('callback') . '(' . json_encode($response) . ')';
        exit();
    }

    /**
     * Get Elderberry quantity from the requested payload
     */
    public function getElderBerryQuantity($payloadProperties) {

        $elderberryQuantity = 0;
        $elderberryProductIds = env("ELDERBERRY_PRODUCT_IDS", true);
        $elderberryVariantIds = env("ELDERBERRY_VARIANT_IDS", true);
        $elderberryProductIds = explode(",", $elderberryProductIds);
        $elderberryVariantIds = explode(",", $elderberryVariantIds);
        if (isset($payloadProperties)) {
            foreach ($payloadProperties as $propKey => $propValue) {
                $lipProductId = !empty($propValue['product_id']) ? trim($propValue['product_id']) : '';
                $lipVariantId = !empty($propValue['variant_id']) ? trim($propValue['variant_id']) : '';
                if (in_array($lipProductId, $elderberryProductIds) && in_array($lipVariantId, $elderberryVariantIds)) {
                    $elderberryQuantity = $propValue['quantity'];
                }
            }
        }
        return $elderberryQuantity;
    }

    /**
     * Update Elderberry quantity in the requested payload
     */
    public function updateElderberryQuantity($payloadProperties) {

        $elderberryQuantityAdjustedArray = [];
        $elderberryProductIds = env("ELDERBERRY_PRODUCT_IDS", true);
        $elderberryVariantIds = env("ELDERBERRY_VARIANT_IDS", true);
        $elderberryProductIds = explode(",", $elderberryProductIds);
        $elderberryVariantIds = explode(",", $elderberryVariantIds);
        if (isset($payloadProperties)) {
            foreach ($payloadProperties as $propKey => $propValue) {
                $elderberryTempArray = [];
                $lipProductId = !empty($propValue['product_id']) ? trim($propValue['product_id']) : '';
                $lipVariantId = !empty($propValue['variant_id']) ? trim($propValue['variant_id']) : '';
                if (in_array($lipProductId, $elderberryProductIds) && in_array($lipVariantId, $elderberryVariantIds)) {
                    $elderberryTempArray = [
                        "name" => $propValue['name'],
                        "value" => $propValue['value'],
                        "product_id" => $propValue['product_id'],
                        "variant_id" => $propValue['variant_id'],
                        "quantity" => 1,
                        "dosage" => $propValue['dosage']
                    ];
                    $elderberryQuantityAdjustedArray[] = $elderberryTempArray;
                } else {

                    $elderberryQuantityAdjustedArray[] = $propValue;
                }
            }
        }
        return $elderberryQuantityAdjustedArray;
    }

    /**
     * Get product pack count
     */
    public function getProductPackCount($requestData) {

        $packRequest['ProductItems'] = array();
        $errorMessage = '';
        if (empty($requestData) || !isset($requestData)) {
            return null;
        }
        foreach ($requestData as $data) {
            $bulkID = isset($data['bulkID']) ? $data['bulkID'] : '';
            $qty = isset($data['qty']) ? $data['qty'] : '';
            if (empty($bulkID)) {
                $errorMessage = 'Empty parameter value - bulkID';
                return $errorMessage;
            }
            if (empty($qty)) {
                $errorMessage = 'Empty parameter value - qty';
                return $errorMessage;
            }
            if ($bulkID == 'vitamin-pack') {
                continue;
            }
            $tempPackRequest['BulkItem'] = $bulkID;
            $tempPackRequest['Quantity'] = $qty;
            array_push($packRequest['ProductItems'], $tempPackRequest);
        }
        if (empty($packRequest['ProductItems'])) {
            $errorMessage = 'Empty ProductItems';
            return $errorMessage;
        }

        $returnASCII = $this->encode_items($packRequest);

        $packDetails = $this->packCount->getPackCount($returnASCII);

        if (strpos($packDetails, 'Packs') !== false) {
            $stripData = explode("Packs", $packDetails);
            $packData = $stripData[1];
            $packData = ltrim($packData, ":");
            $packData = rtrim($packData, "}");
            $returnData = [
                'Pack' => $packData,
            ];
            return $returnData;
        } else {
            $errorMessage = 'No Pack details';
            return $errorMessage;
        }
    }

    /**
     * Encode items
     */
    public function encode_items($array) {

        foreach ($array as $key => $value) {
            if (is_array($value)) {
                $array[$key] = $this->encode_items($value);
            } else {
                $array[$key] = mb_convert_encoding($value, 'UTF-8');
            }
        }

        return $array;
    }
    
    /**
     * parseDom
     */
    public function parseDom() {

    	$url="https://nurish-usd-dev.myshopify.com/pages/subscription-discount";
    	try {
        	$ch = curl_init();
        	curl_setopt($ch, CURLOPT_URL, $url);
        	curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        	curl_setopt($ch,CURLOPT_USERAGENT,'Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.13) Gecko/20080311 Firefox/2.0.0.13');
        	$html = curl_exec($ch);
        	curl_close($ch);
    	} catch(Exception $e) {

            Log::channel('checkoutLog')->info("parseDom: Failed to load the Checkout for Token (" .
            $receivedData->checkout->token . "), Error: " .$error);
    		print_r($e->getMessage());
    	}
    	$dom = new \DOMDocument();
    	@$dom->loadHTML($html);
    	$xpath = new \DOMXPath($dom);
    	$inputNodes = $xpath->query('//input[@id="globalDiscountValue"]');
    	if (isset($inputNodes[0])) {
    	    return $inputNodes[0]->getAttribute('value');
    	}
    }
    
    /**
     * Get OTP extra details
     */
    public function getOTPExtra($email) {

    	$url="https://nurish-usd-dev.myshopify.com/pages/subscription-discount";
    	try {
        	$ch = curl_init();
        	curl_setopt($ch, CURLOPT_URL, $url);
        	curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        	curl_setopt($ch,CURLOPT_USERAGENT,'Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.13) Gecko/20080311 Firefox/2.0.0.13');
        	$html = curl_exec($ch);
        	curl_close($ch);
    	} catch(Exception $e) {
            Log::channel('checkoutLog')->info("getOTPExtra: Failed to load the URL from CMS for -".$email);
    	}
    	$dom = new \DOMDocument();
    	@$dom->loadHTML($html);
    	$xpath = new \DOMXPath($dom);
    	$inputNodes = $xpath->query('//input[@id="globalDiscountValue"]');
    	if (isset($inputNodes[0])) {
    	    $discount = $this->getExtraOTPAmount($inputNodes[0]->getAttribute('value'));
    	    return $discount;
    	}
        Log::channel('checkoutLog')->info("getOTPExtra: Failed to load the json URL from CMS -".$email);
    	return;
    }
    
    /**
     * Get the extra OTP amount
     */
    public function getExtraOTPAmount($url) {
        // $url = '//cdn.shopify.com/s/files/1/0263/1293/3458/t/48/assets/subscription_schema.json?v=9998742680377307201';
    	if (!empty($url)) {
    		$url  = 'https:'.$url;
    		$html = file_get_contents($url);
    		$discount = json_decode($html)->subscriptionDiscount;
    		if (trim($discount) != '') {
    		   if($discount > 0) {
    		       return $discount; 
    		   }
    		}
        }
        Log::channel('checkoutLog')->info("getOTPExtra: Failed to load the discount from JSON file");
    	return;
    }

    /**
     * Function to get all Shopify products
    */
    public function getShopifyProductDetails() {

        $shopifyProducts  = [];
        $pageNo = 1;
        $limit  = 200;
        do {
            try {
                if(!empty($products['link'])) {
                    $products = $this->productObj->getShopifyProducts($limit,$products['link']);
                 } else {
                    $products = $this->productObj->getShopifyProducts(
                        $limit);
                }
            }
            catch (Exception $e){
                Log::channel('checkoutLog')->info("getShopifyProductDetails: Error form Shopify when fetching the product details");
                exit();
            }
            $productsToSkip = array('4359176454226', '4359186350162', '4426715758674', '4716581486674', '4716582797394'); // Dev store
            //$productsToSkip = array('1809168695367', '4778895736903', '4778912219207', '4780554256455'); // Production
            
            if (!empty($products) && isset($products)) {
                foreach ($products['body'] as $product) {
                    if (in_array($product['id'], $productsToSkip)) {
                        continue;
                    }
                    if (!empty($product['variants'])) {
                        $variantDetails = [];
                        foreach ($product['variants'] as $variant) {
                            if (!empty($variant['id'])) {
                                $variantDetails = [
                                    'variant_id' => $variant['id'],
                                    'product_id' => $variant['product_id'],
                                    'variant_title' => strip_tags($variant['title']),
                                    'product_title' => strip_tags($product['title']),
                                    'sku' => $variant['sku'],
                                    'price' => $variant['price']
                                ];
                                $shopifyProducts[$variant['id']] = $variantDetails;
                            }
                        }
                    }
                }
                $productCount = count($products['body']);
            }
            $pageNo++;
            if (empty($products['link'])) {
                break;
            }
        } while($productCount == $limit);

        return $shopifyProducts;
    }
}
