<?php

namespace App\Http\Controllers;

use App\Models\RechargeApi;
use App\Models\Shopify\Product;
use Illuminate\Http\Request;
use Log;

class GiftRecipientCheckoutController extends Controller
{
    private $rechargeObj;
    private $productObj;

    public function __construct() {

        $this->rechargeObj = new RechargeApi();
        $this->productObj = new Product();
    }
    
    /**
     * Function to create the checkout of the gift recipient
     */
    public function createGiftRecipientCheckout(Request $request) {

        $requestData = $request->input('data');
        $CheckoutUrl = env('NURISH_CHECKOUT_URL', true);
        $shopifyRegularPackProductId = env('REGULAR_SUBSCRIPTION_PRODUCT_ID', true);
        $shopifyRegularPackVariantId = env('REGULAR_SUBSCRIPTION_VARIANT_ID', true);
        if (empty($requestData)) {
            $returnData = [
                'error' => 'Invalid endpoint request',
            ];
            Log::channel('giftRecipientCheckoutLog')->info("Invalid endpoint request");            
            return $this->returnData(600, "error", $returnData);
        }
        $receivedData = json_decode($requestData);
        if (empty($receivedData->subscriptionId)) {
            $returnData = [
                'error' => 'Invalid endpoint request',
            ];    
            Log::channel('giftRecipientCheckoutLog')->info("Subscription Id is empty in 
            the endpoint request");        
            return $this->returnData(600, "error", $returnData);
        }
        // Decode endpoint request
        $purchaserSubscriptionId = $this->decodeEndpointRequest($receivedData->subscriptionId);

        $productDetails = $this->getShopifyProductDetails();

        $subscriptionId = !empty($purchaserSubscriptionId) ? trim($purchaserSubscriptionId) : '';
        $giftSubscriptions = $this->rechargeObj->getSubscription($subscriptionId);
        if (!empty($giftSubscriptions->error) || !empty($giftSubscriptions->errors) || empty($giftSubscriptions) || 
            empty($giftSubscriptions->subscription)) {
            $rechargeError  = "Unknown error";
            if (!empty($giftSubscriptions->error)) {
                $rechargeError = json_encode($giftSubscriptions->error);
            } else if (!empty($giftSubscriptions->errors)) {
                $rechargeError = json_encode($giftSubscriptions->errors);
            } else if (empty($giftSubscriptions) || empty($giftSubscriptions->subscription)) {
                $rechargeError = 'Failed to get the subscription details by subscriptionId';
            }
            $returnData = [
                'error' => $rechargeError
            ];
            Log::channel('giftRecipientCheckoutLog')->info("Failed to get the subscription 
            details by subscriptionId(subscriptionId:".$subscriptionId."), Error: (".$rechargeError.")");
            return $this->returnData(600, "error", $returnData);
        }
        $giftSubscriptionDetails = '';
        $giftSubscriptionDetails = $giftSubscriptions->subscription;
        if (empty($giftSubscriptionDetails)) {
            $returnData = [
                'error' => 'No subscription found'
            ];
            Log::channel('giftRecipientCheckoutLog')->info("No subscription found 
            (SubscriptionId:".$subscriptionId.")");
            return $this->returnData(600, "error", $returnData);
        }
        if (empty($giftSubscriptionDetails->properties)) {
            $returnData = [
                'error' => 'No line item properties are found in the subscription'
            ];
            Log::channel('giftRecipientCheckoutLog')->info("No line item properties are 
            found in the subscription (subscriptionId:".$subscriptionId.")");
            return $this->returnData(600, "error", $returnData);
        }
        $properties = $giftSubscriptionDetails->properties;
        $requestProperties = [];
        $packs = "";
        $chargeIntervalFrequency = "";
        $gifteeEmail = "";
        $gifteeName = "";
        foreach ($properties as $property) {
            if (strpos(strtolower(trim($property->name)), 'vitamin') !== false) {
                $value = explode('|', $property->value);
                $variantId = trim($value[3]);
                $requestProperties[] = array(
                    'name' => $property->name,
                    'value' => trim($value[0]),
                    'product_id' => array_key_exists($variantId, $productDetails) ? 
                    $productDetails[$variantId]['product_id'] : '',
                    'variant_id' => $variantId,
                    'quantity' => trim($value[2]),
                    'dosage' => trim($value[1])
                );
            }
            if (strpos($property->name, 'Packs') !== false) {
                $packs = $property->value;
            }
            if (strpos($property->name, 'charge_interval_frequency') !== false) {
                $chargeIntervalFrequency = $property->value;
            }
            if (strpos($property->name, 'giftee_email') !== false) {
                $gifteeEmail = $property->value;
            }
            if (strpos($property->name, 'giftee_first_name') !== false) {
                $gifteeName = $property->value;
            }
        }
        $requestProperties[] = array(
            'giftToSub' => "true"
        );
        $checkoutRequest['checkout'] = array(
            'discount_code' => '',
            'line_items' => array(
                array(
                    'properties' => $requestProperties,
                    'quantity' => !empty($giftSubscriptionDetails->quantity) ? 
                    trim($giftSubscriptionDetails->quantity) : '',
                    'product_id' => $shopifyRegularPackProductId,
                    'variant_id' => $shopifyRegularPackVariantId,
                    'charge_interval_frequency' => $chargeIntervalFrequency,
                    'order_interval_frequency' => !empty($giftSubscriptionDetails->order_interval_frequency) ? 
                    trim($giftSubscriptionDetails->order_interval_frequency) : '',
                    'order_interval_unit' => !empty($giftSubscriptionDetails->order_interval_unit) ? 
                    trim($giftSubscriptionDetails->order_interval_unit) : '',
                    'Packs' => $packs
                )
            ),
            'email' => $gifteeEmail,
            'name' => $gifteeName
        );
        $laravelRequestObj = new \Illuminate\Http\Request();
        $laravelRequestObj->setMethod('POST');
        $this->checkoutControllerObj = new CheckoutController();
        $encodedEndpointRequest = json_encode($checkoutRequest);
        $laravelRequestObj->request->add([
            "callback" => "jQuery34106099294541143507_1638184748205",
            "data" => $encodedEndpointRequest
        ]);

        $checkoutResponse = $this->checkoutControllerObj->CreateCheckout($laravelRequestObj);

        if (strtolower($checkoutResponse['status']) != 'success') {
            $returnData = [
                'error' => 'Failed to create the checkout'
            ];
            Log::channel('giftRecipientCheckoutLog')->info("Failed to create the checkout for the 
            request ".json_encode($checkoutRequest));
            return $this->returnData(600, "error", $returnData);
        }
    }

    /**
     * Decode endpoint request
     */
    public function decodeEndpointRequest($endPointParameter) {

        $decodedValue = '';
        $decodedValue = $endPointParameter;
        for ($i = 0; $i < 5; $i++) { 
            $decodedValue = base64_decode($decodedValue);
        }
        return $decodedValue;
    }

    /**
     * Function to get all shopify products
    */
    public function getShopifyProductDetails() {

        $shopifyProducts  = [];
        $pageNo = 1;
        $limit  = 200;
        do {
            try {
                if (!empty($products['link'])) {
                    $products = $this->productObj->getShopifyProducts($limit,$products['link']);
                 } else {
                    $products = $this->productObj->getShopifyProducts($limit);
                }
            }
            catch (Exception $e) {
                Log::channel('giftRecipientCheckoutLog')->info("getShopifyProductDetails: Error form shopify when fetching the
                product details");
                exit();
            }
            $productsToSkip = array('4359176454226', '4359186350162', '4426715758674', '4716581486674', '4716582797394'); // Dev store
            //$productsToSkip = array('1809168695367', '4778895736903', '4778912219207', '4780554256455'); // Production
            
            if (!empty($products) && isset($products)) {
                foreach ($products['body'] as $product) {
                    if (in_array($product['id'], $productsToSkip)) {
                        continue;
                    }
                    if (!empty($product['variants'])) {
                        $variantDetails = [];
                        foreach ($product['variants'] as $variant) {
                            if (!empty($variant['id'])) {
                                $variantDetails = [
                                    'variant_id' => $variant['id'],
                                    'product_id' => $variant['product_id'],
                                ];
                                $shopifyProducts[$variant['id']] = $variantDetails;
                            }
                        }
                    }
                }
                $productCount = count($products['body']);
            }
            $pageNo++;
            if (empty($products['link'])) {
                break;
            }
        } while($productCount == $limit);

        return $shopifyProducts;
    }

    /**
     * Response function
     */
    public function returnData($status, $message, $error, $jsonCallback = null) {

        $returnData = array(
            'status' => $status,
            'message' => $message,
            'body' => $error
        );
        if (!empty($jsonCallback)) {
            echo $jsonCallback . '(' . json_encode($returnData) . ')';
            exit();
        } else {
            return response()->json($returnData);
        }
    }

}
