<?php

namespace App\Http\Controllers;

use App\Models\IterableApi;
use Illuminate\Http\Request;
use Log;

class IterableController extends Controller
{
    private $modelObj;

    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct() {

        $this->modelObj = new IterableApi();
    }

    /**
     * Get user Iterable details
     */
    public function getUserIterableStatus(Request $request) {

        $requestData = !empty($request->input('data')) ? $request->input('data') : "";
        if (empty($requestData)) {
            $response = array(
                "status" => "Failed",
                "errorMessage" => "Endpoint parameters are missing"
            );
            $response = !empty($request->input('callback')) ? $request->input('callback').'('.json_encode($response).')' : 
            json_encode($response);
            Log::channel('iterableLog')->info("getUserIterableStatus: Endpoint parameters are missing");
            echo $response;
            exit();
        }
        Log::channel('iterableLog')->info("getUserIterableStatus: Request =>".$requestData);
        $receivedData = json_decode($requestData);
        $emailAddress = !empty($receivedData->email) ? trim($receivedData->email) : "";
        if (empty($emailAddress)) {
            $errorMessage = "Missing required parameter - email";
            Log::channel('iterableLog')->info("getUserIterableStatus: ".$errorMessage);
            return $this->returnData("Failed", $errorMessage, $request->input('callback'));
         }

         $subscriptionResponse = $this->modelObj->getIterableSubscription($emailAddress);

         if (!empty($subscriptionResponse->msg) || !empty($subscriptionResponse->errors) || empty($subscriptionResponse)) {
            $status = "failed";
            $error = "Unknown error";
            if (!empty($subscriptionResponse->msg)) {
                $error = json_encode($subscriptionResponse->msg);
            } else if (!empty($subscriptionResponse->errors)) {
                $error = json_encode($subscriptionResponse->errors);
            }
            $errorMessage = "Failed to get the details from Iterable";
            Log::channel('iterableLog')->info("getUserIterableStatus: ".$errorMessage.", Error: " .$error);
            return $this->returnData("Failed", $errorMessage, $request->input('callback'));
        }
        $responseData = array(
            'unsubscribedChannelIds' => !empty($subscriptionResponse->user->dataFields->unsubscribedChannelIds) ? 
            $subscriptionResponse->user->dataFields->unsubscribedChannelIds : "",
            'subscribedMessageTypeIds' => !empty($subscriptionResponse->user->dataFields->subscribedMessageTypeIds) ? 
            $subscriptionResponse->user->dataFields->subscribedMessageTypeIds : "",
            'unsubscribedMessageTypeIds' => !empty($subscriptionResponse->user->dataFields->unsubscribedMessageTypeIds) ? 
            $subscriptionResponse->user->dataFields->unsubscribedMessageTypeIds : ""
        );
        $response = array(
            "status" => "success",
            "response" => $responseData,
            "errorMessage" => "",
        );
        Log::channel('iterableLog')->info("getUserIterableStatus: Successfully get the user iterable subscription details");
        echo $request->input('callback').'('.json_encode($response).')';
        exit();
    }

    /**
     * Update user Iterable details
     */
     public function updateIterableSubscription(Request $request) {

        $requestData = !empty($request->input('data')) ? $request->input('data') : "";
        if (empty($requestData)) {
            $response = array(
                "status" => "Failed",
                "errorMessage" => "Endpoint parameters are missing"
            );
            $response = !empty($request->input('callback')) ? $request->input('callback').'('.json_encode($response).')' : 
            json_encode($response);
            Log::channel('iterableLog')->info("updateIterableSubscription: Endpoint parameters are missing");
            echo $response;
            exit();
        }
        Log::channel('iterableLog')->info("updateIterableSubscription: Request =>".$requestData);
        $receivedData = json_decode($requestData);
        $emailAddress = !empty($receivedData->email) ? trim($receivedData->email) : "";
        $unSubscribeChannelIds = !empty($receivedData->unSubscribeChannelIds) ? $receivedData->unSubscribeChannelIds : "";
        $unSubscribePreferenceIds = !empty($receivedData->unSubscribePreferenceIds) ? $receivedData->unSubscribePreferenceIds : "";
        $subscribePreferenceIds = !empty($receivedData->subscribePreferenceIds) ? $receivedData->subscribePreferenceIds : "";
        if (empty($emailAddress) || empty($unSubscribeChannelIds) || empty($unSubscribePreferenceIds) ||
            empty($subscribePreferenceIds)) {
            $errorMessage = "Missing required parameters or the request is not correct";
            Log::channel('iterableLog')->info("updateIterableSubscription: ".$errorMessage);
            return $this->returnData("Failed", $errorMessage, $request->input('callback'));
        }
        $updateRequest = array(
            "email" => $emailAddress,
            "unsubscribedChannelIds" => array_map('intval', $unSubscribeChannelIds),
            "unsubscribedMessageTypeIds" => array_map('intval', $unSubscribePreferenceIds),
            "subscribedMessageTypeIds" => array_map('intval', $subscribePreferenceIds)
        );

        $subscriptionResponse = $this->modelObj->updateUserIterableSubscription(json_encode($updateRequest));

        if (!empty($subscriptionResponse->msg) || !empty($subscriptionResponse->errors) || empty($subscriptionResponse)) {
            $status = "failed";
            $error = "Unknown error";
            if (!empty($subscriptionResponse->msg)) {
                $error = json_encode($subscriptionResponse->msg);
            } else if (!empty($subscriptionResponse->errors)) {
                $error = json_encode($subscriptionResponse->errors);
            }
            $errorMessage = "Failed to update the Iterable subscription";
            Log::channel('iterableLog')->info("updateIterableSubscription: ".$errorMessage.", Error: " .$error);
            return $this->returnData("Failed", $errorMessage, $request->input('callback'));
        }
        $response = array(
            "status" => "success",
            "response" => 'updated',
            "errorMessage" => "",
        );
        Log::channel('iterableLog')->info("updateIterableSubscription: Successfully updated the iterable subscription");
        echo $request->input('callback').'('.json_encode($response).')';
        exit();
     }

    /**
     * add user to Iterable List
    */
    public function addUserToIterableList(Request $request) {

        $requestData = !empty($request->input('data')) ? $request->input('data') : "";
        if (empty($requestData)) {
            $response = array(
                "status" => "Failed",
                "errorMessage" => "Endpoint parameters are missing"
            );
            $response = !empty($request->input('callback')) ? $request->input('callback').'('.json_encode($response).')' : 
            json_encode($response);
            Log::channel('rechargeAddressLog')->info("addUserToIterableList: Endpoint parameters are missing");
            echo $response;
            exit();
        }
        Log::channel('iterableLog')->info("addUserToIterableList: Request =>".$requestData);
        $receivedData = json_decode($requestData);
        $emailAddress = !empty($receivedData->email) ? trim($receivedData->email) : "";
        $mobileNumber = !empty($receivedData->mobileNumber) ? trim($receivedData->mobileNumber) : "";
        $firstName = !empty($receivedData->firstName) ? trim($receivedData->firstName) : "";
        $lastName = !empty($receivedData->lastName) ? trim($receivedData->lastName) : "";
        $postalCode = !empty($receivedData->postalCode) ? trim($receivedData->postalCode) : "";
        if (empty($emailAddress) || empty($mobileNumber) || empty($firstName) || empty($lastName) || empty($postalCode)) {
            $errorMessage = "Missing required parameters or the request is not correct";
            Log::channel('iterableLog')->info("addUserToIterableList: ".$errorMessage);
            return $this->returnData("Failed", $errorMessage, $request->input('callback'));
         }
        $subscriptionRequest = array(
            "listId" => intval(env('ITERABLE_SMS_SUBSCRIBE_LIST_ID', true)),
            "subscribers" => array(
                array(
                    "email" => $emailAddress,
                    "dataFields" => array(
                    "phoneNumber" => $mobileNumber,
                    "First Name" => $firstName,
                    "Last Name" => $lastName,
                    "default_address" => array(
                        "zip" => $postalCode
                    )
                  )
               )
            )
        );

        $createUserSubscribeListResponse = 
        $this->modelObj->addSubscriberToIterableList(json_encode($subscriptionRequest));

        if (!empty($createUserSubscribeListResponse->msg) || !empty($createUserSubscribeListResponse->errors) ||
            empty($createUserSubscribeListResponse)) {
            $status = "failed";
            $error = "Unknown error";
            if (!empty($createUserSubscribeListResponse->msg)) {
                $error = json_encode($createUserSubscribeListResponse->msg);
            } else if (!empty($createUserSubscribeListResponse->errors)) {
                $error = json_encode($createUserSubscribeListResponse->errors);
            }
            $errorMessage = "Failed to update the Iterable subscription";
            Log::channel('iterableLog')->info("addUserToIterableList: Failed to update the Iterable subscription details for Email 
            (".$emailAddress."), Error: " .$error);
            return $this->returnData("Failed", $errorMessage, $request->input('callback'));
        }
        $response = array(
            "status" => "success",
            "response" => 'updated',
            "errorMessage" => "",
        );
        Log::channel('iterableLog')->info("addUserToIterableList: Successfully added the user to the iterable list");
        echo $request->input('callback').'('.json_encode($response).')';
        exit();
      }

    /**
     * Response function
     */
    public function returnData($status, $errorMessage, $callBackToken) {

        $response = array(
            "status"       => $status,
            "errorMessage" => $errorMessage
        );
        echo $callBackToken . '(' . json_encode($response) . ')';
        exit();
    }
}
