<?php

namespace App\Http\Controllers;

use App\Models\RechargeApi;
use App\Models\Shopify\Product;
use Illuminate\Http\Request;
use Log;

class RechargeAddressController extends Controller
{
    private $modelObj;
    private $productObj;

    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct() {

        $this->modelObj  = new RechargeApi();
        $this->productObj = new Product();
    }

    /**
     * Update subscription shipping rate when updating the address
     */
    public function updateSubscriptionByAddressID(Request $request) {

        $requestData = !empty($request->input('data')) ? $request->input('data') : "";
        if (empty($requestData)) {
            $message = "Endpoint parameters are missing";
            $error = "Bad request";
            $status = "Failed";
            $response = array(
                "status"       => $status,
                "message" => $message,
                "error" => $error
            );
            $response = !empty($request->input('callback')) ? $request->input('callback').'('.json_encode($response).')' : 
            json_encode($response);
            Log::channel('rechargeAddressLog')->info("updateSubscriptionByAddressID: Endpoint parameters are missing");
            echo $response;
            exit();
        }
        Log::channel('rechargeAddressLog')->info("updateSubscriptionByAddressID: Request =>".$requestData);
        $receivedData = json_decode($requestData);
        $customerId = !empty($receivedData->customer_id) ? trim($receivedData->customer_id) : "";
        $addressId = !empty($receivedData->address_id) ? trim($receivedData->address_id) : "";
        if (empty($customerId) || empty($addressId)) {
            $errorMessage = "The customerID or addressID is empty";
            Log::channel('rechargeAddressLog')->info("updateSubscriptionByAddressID: ".$errorMessage);
            return $this->returnData("Failed", $errorMessage, $request->input('callback'));
        } 

        $customerSubscriptions = $this->modelObj->getAllSubscriptionsByCustomerID($customerId);

        if (!empty($customerSubscriptions->error) || !empty($customerSubscriptions->errors) || 
            empty($customerSubscriptions->subscriptions)) {
            $status = "failed";
            $error  = "Unknown error";
            if (!empty($customerSubscriptions->error)) {
                $error = json_encode($customerSubscriptions->error);
            } else if (!empty($customerSubscriptions->errors)) {
                $error = json_encode($customerSubscriptions->errors);
            } else if (empty($customerSubscriptions->subscriptions)) {
                $error = "Failed to get the subscriptions";
            }
            $errorMessage = "Failed to get the subscription details, Error: ".$error;
            Log::channel('rechargeAddressLog')->info("updateSubscriptionByAddressID: ".$errorMessage);
            return $this->returnData("Failed", $errorMessage, $request->input('callback'));
        }
        if (empty($customerSubscriptions) || !isset($customerSubscriptions)) {
            $errorMessage = "There is no subscription in the given customer id";
            Log::channel('rechargeAddressLog')->info("updateSubscriptionByAddressID: ".$errorMessage);
            return $this->returnData("Failed", $errorMessage, $request->input('callback'));
        }
        $checkOverRideFreeShippingStatus = 0;
        $addressIdFoundStatus = 0;
        $subscriptionPrice = 0;
        $shippingRateUpdateStatus = 0;
        $freeShippingThreshold = 0;
        foreach ($customerSubscriptions->subscriptions as $subValue) {
            if (trim($subValue->address_id) == trim($receivedData->address_id)) {
                $addressIdFoundStatus = 1;
                $subscriptionPrice = trim($subValue->price);
                if (!empty($subValue->properties)) {
                    foreach ($subValue->properties as $propKey => $propValue) {
                        $subscriptionPropertyCheck = strtolower(trim($propValue->name));
                        if ($subscriptionPropertyCheck == 'override_free_shipping') {
                            $shippingOverrideValue = !empty($propValue->value) ? $propValue->value : false;
                            if ($shippingOverrideValue && $shippingOverrideValue == "true") {
                                $checkOverRideFreeShippingStatus = 1;
                            }
                        }
                        if ($subscriptionPropertyCheck == 'free_shipping_threshold') {
                            $shippingThresholdValue = !empty($propValue->value) ? trim($propValue->value) : 
                            number_format((float)0, 2, '.', '');
                            if (!empty($shippingThresholdValue)) {
                                $freeShippingThreshold = $shippingThresholdValue;
                                $freeShippingThreshold = round($freeShippingThreshold);
                                $freeShippingThreshold = number_format((float)$freeShippingThreshold, 2, '.', '');
                            }
                        }
                    }
                }
            }
        }
        if ($addressIdFoundStatus == 0) {
            $errorMessage = "The address Id that passed in the request is not found in the subscription";
            Log::channel('rechargeAddressLog')->info("updateSubscriptionByAddressID: ".$errorMessage);
            return $this->returnData("Failed", $errorMessage, $request->input('callback'));
        }
        // If override_free_shipping is not found in the subscription LIP
        if ($checkOverRideFreeShippingStatus == 0) {
            $usShippingRateAdjustableCountries = array(
                "alabama","arizona","arkansas","california","colorado","connecticut","delaware","florida","georgia",
                "idaho","illinois","indiana","iowa","kansas","kentucky","louisiana","maine","maryland","massachusetts",
                "michigan","minnesota","mississippi","missouri","montana","nebraska","nevada","new-hampshire",
                "new-jersey","new-mexico","new-york","north-carolina","north-dakota","ohio","oklahoma",
                "oregon","pennsylvania","rhode-island","south-carolina","south-dakota","tennessee","texas",
                "utah","vermont","virginia","washington","west-virginia","wisconsin","wyoming"
            );
            $usShippingRateNonAdjustableCountries = array("alaska","hawaii");
            $subscriptionShippingZoneIds = [96075317330,96075284562,94991646791,94991679559];
            $shopifyDomesticShippingPrice = '';
            $shopifyNonDomesticShippingPrice = '';

            $shippingZones = $this->productObj->getShopifyShippingZone();

            if (!empty($shippingZones['error']) || !empty($shippingZones['errors']) || 
                !isset($shippingZones['shipping_zones']) || empty($shippingZones)) {
                $status = "failed";
                $error  = "Unknown error";
                if (!empty($shippingZones['error'])) {
                    $error = json_encode($shippingZones['error']);
                } else if (!empty($shippingZones['errors'])) {
                    $error = json_encode($shippingZones['errors']);
                } else if (empty($shippingZones) || !isset($shippingZones['shipping_zones'])) {
                    $error = "Failed to get the Shopify shipping zone details";
                }
                $errorMessage = "Failed to get the Shopify shipping zone details";
                Log::channel('rechargeAddressLog')->info("updateSubscriptionByAddressID: ".$errorMessage.", Error:".$error);
                return $this->returnData("Failed", $errorMessage, $request->input('callback'));
            }
            if (isset($shippingZones['shipping_zones'])) {
                foreach ($shippingZones['shipping_zones'] as $shipValue) {
                    $shippingZoneId = !empty($shipValue['id']) ? trim($shipValue['id']) : '';
                    $shippingZoneName = strtolower(trim($shipValue['name']));
                    if (strpos($shipValue['name'], ",") !== false ) {
                        $shippingZoneName = str_replace(" ", "", $shippingZoneName);
                        $shippingZoneName = str_replace(",", "-", $shippingZoneName);
                    }
                    if (($shippingZoneName == 'domestic' || $shippingZoneName == 'alaska-hawaii') && 
                        in_array($shippingZoneId, $subscriptionShippingZoneIds)) {
                        if (!empty($shipValue['price_based_shipping_rates'])) {
                            foreach ($shipValue['price_based_shipping_rates'] as $priceValue) {
                                $shippingName = strtolower(trim($priceValue['name']));
                                $shippingName = str_replace(" ", "-", $shippingName);
                                if ($shippingName == 'standard-shipping') {
                                    if ($shippingZoneName == 'domestic') {
                                        $shopifyDomesticShippingPrice = $priceValue['price'];
                                    } else {
                                        $shopifyNonDomesticShippingPrice = $priceValue['price'];
                                    }
                                }
                            }
                        }
                    }
                }
            }

            $userAddressDetails = $this->modelObj->getAddressByAddressID($receivedData->address_id);

            if (!empty($userAddressDetails->error) || !empty($userAddressDetails->errors) ||
                empty($userAddressDetails)) {
                $status = "failed";
                $error  = "Unknown error";
                if (!empty($userAddressDetails->error)) {
                    $error = json_encode($userAddressDetails->error);
                } else if (!empty($userAddressDetails->errors)) {
                    $error = json_encode($userAddressDetails->errors);
                } else if (empty($userAddressDetails)) {
                    $error = "Failed to get the address details by address ID";
                }
                $errorMessage = "Failed to get the address details to override the shipping details";
                Log::channel('rechargeAddressLog')->info("updateSubscriptionByAddressID: ".$errorMessage.", Error:".$error);
                return $this->returnData("Failed", $errorMessage, $request->input('callback'));
            }
            $addressProvince = '';
            if (!empty($userAddressDetails->address->province)) {
                $addressProvince = strtolower(trim($userAddressDetails->address->province));
                $addressProvince = str_replace(" ", "-", $addressProvince);
            } else {
                $addressProvince = '';
            }
            $shippingUpdateStatus = 0;
            $shippingPrice = '';
            $shippingCode = '';
            $freeShippingThreshold = !empty($freeShippingThreshold) ? $freeShippingThreshold : 
            number_format((float)20, 2, '.', '');
            if (!empty($addressProvince) && in_array($addressProvince, $usShippingRateAdjustableCountries)) {
                if ($subscriptionPrice >= $freeShippingThreshold) {
                    $shippingPrice = 0;
                    $shippingCode = "Free Shipping";
                } else {
                    $shippingPrice = !empty($shopifyDomesticShippingPrice) ? $shopifyDomesticShippingPrice : 8;
                    $shippingCode = "Standard Shipping";
                }
                $shippingUpdateStatus = 1;

            } elseif (!empty($addressProvince) && in_array($addressProvince, $usShippingRateNonAdjustableCountries)) {
                $shippingPrice = !empty($shopifyNonDomesticShippingPrice) ? $shopifyNonDomesticShippingPrice : 8;
                $shippingCode = "Standard Shipping";
                $shippingUpdateStatus = 1;
            } else {
                if (!empty($freeShippingThreshold)) {
                    if ($subscriptionPrice >= $freeShippingThreshold) {
                        $shippingPrice = 0;
                        $shippingCode = "Free Shipping";
                    }
                } else {
                    $shippingPrice = !empty($shopifyDomesticShippingPrice) ? $shopifyDomesticShippingPrice : 8;
                    $shippingCode = "Standard Shipping";
                }
                $shippingUpdateStatus = 1;
            }
            if ($shippingUpdateStatus == 1) {
                $shippingUpdateRequest = array(
                    "shipping_lines_override" => array(
                        array(
                            "price" => $shippingPrice,
                            "code"  => $shippingCode,
                            "title" => "Standard Shipping"
                        )
                    )
                );

                $updatedShippingRateResponse = $this->modelObj->updateShippingRateByAddress($receivedData->address_id, 
                json_encode($shippingUpdateRequest));

                if (isset($updatedShippingRateResponse->error) || isset($updatedShippingRateResponse->errors) ||
                    empty($updatedShippingRateResponse)) {
                    $status = "failed";
                    $error  = "Unknown error";
                    if (isset($updatedShippingRateResponse->error)) {
                        $error = json_encode($updatedShippingRateResponse->error);
                    } else if (isset($updatedShippingRateResponse->errors)) {
                        $error = json_encode($updatedShippingRateResponse->errors);
                    } else if (empty($updatedShippingRateResponse)) {
                        $error = "Failed to update the Shipping amount";
                    }
                    $errorMessage = "Failed to update the shipping amount";
                    Log::channel('rechargeAddressLog')->info("updateSubscriptionByAddressID: ".$errorMessage.", Error:".$error);
                    return $this->returnData("Failed", $errorMessage, $request->input('callback'));
                }
            }
            $shippingRateUpdateStatus = 1;
        } else if ($checkOverRideFreeShippingStatus == 1) {
            $existingShippingPrice = '';
            $shippingPrice = '';
            $freeShippingThreshold = 0;

            $userAddressDetails = $this->modelObj->getAddressByAddressID($receivedData->address_id);

            if (!empty($userAddressDetails->error) || !empty($userAddressDetails->errors) ||
                empty($userAddressDetails)) {
                $status = "failed";
                $error  = "Unknown error";
                if (!empty($userAddressDetails->error)) {
                    $error = json_encode($userAddressDetails->error);
                } else if (!empty($userAddressDetails->errors)) {
                    $error = json_encode($userAddressDetails->errors);
                } else if (empty($userAddressDetails)) {
                    $error = "Failed to get the address details by address ID";
                }
                $errorMessage = "Failed to get the address details to override the shipping details";
                Log::channel('rechargeAddressLog')->info("updateSubscriptionByAddressID: ".$errorMessage.", Error:".$error);
                return $this->returnData("Failed", $errorMessage, $request->input('callback'));
            }
            $addressOriginalShipping = !empty($userAddressDetails->address->original_shipping_lines[0]->price) ?
            (float)$userAddressDetails->address->original_shipping_lines[0]->price : 0;
            $addressOverRideFreeShipping = !empty($userAddressDetails->address->shipping_lines_override[0]->price) ? 
            (float)$userAddressDetails->address->shipping_lines_override[0]->price : 0;
            $existingShippingPrice = !empty($addressOverRideFreeShipping) ? $addressOverRideFreeShipping : 
            $addressOriginalShipping;
            if ($addressOverRideFreeShipping > 0 || $addressOriginalShipping > 0) {
                $shippingUpdateRequest = array(
                    "shipping_lines_override" => array(
                        array(
                            "code" => "Free shipping",
                            "price"=> "0.00",
                            "title"=> "Free shipping"
                        )
                    )
                );

                $updatedShippingRateResponse = $this->modelObj->updateShippingRateByAddress($receivedData->address_id, 
                json_encode($shippingUpdateRequest));

                if (isset($updatedShippingRateResponse->error) || isset($updatedShippingRateResponse->errors) ||
                    empty($updatedShippingRateResponse)) {
                    $status = "failed";
                    $error  = "Unknown error";
                    if (isset($updatedShippingRateResponse->error)) {
                        $error = json_encode($updatedShippingRateResponse->error);
                    } else if (isset($updatedShippingRateResponse->errors)) {
                        $error = json_encode($updatedShippingRateResponse->errors);
                    } else if (empty($updatedShippingRateResponse)) {
                        $error = "Failed to update the Shipping amount";
                    }
                    $errorMessage = "Failed to update the shipping amount";
                    Log::channel('rechargeAddressLog')->info("updateSubscriptionByAddressID: ".$errorMessage.", Error:".$error);
                    return $this->returnData("Failed", $errorMessage, $request->input('callback'));
                }
                $shippingRateUpdateStatus = 1;
            }
        }
        if($shippingRateUpdateStatus == 0) {
            $response = array(
                "status"             => "success",
                "message"            => "There are no subscriptions at the given addressID",
                "errorMessage"       => "",
            );
            Log::channel('rechargeAddressLog')->info("updateSubscriptionByAddressID: There are no subscriptions at the given addressID");
            echo $request->input('callback') . '(' . json_encode($response) . ')';
            exit();
        }
        if ($shippingRateUpdateStatus == 1) {
            $response = array(
                "status"             => "success",
                "message"            => "Successfully updated the subscriptions shipping amount of the given user",
                "errorMessage"       => "",
            );
            Log::channel('rechargeAddressLog')->info("updateSubscriptionByAddressID: Successfully updated the subscriptions 
            shipping amount of the given user");
            echo $request->input('callback') . '(' . json_encode($response) . ')';
            exit();
        }
    }

    /**
     * Response function
     */
    public function returnData($status, $errorMessage, $callBackToken) {

        $response = array(
            "status"       => $status,
            "errorMessage" => $errorMessage
        );
        echo $callBackToken . '(' . json_encode($response) . ')';
        exit();
    }
}
