<?php

namespace App\Http\Controllers;

use App\Models\RechargeApi;
use App\Models\IterableApi;
use Illuminate\Http\Request;
use App\Models\Shopify\Product;
use Log;


class SubscriptionController extends Controller
{
    private $rechargeObj;
    private $iterableObj;
    private $productObj;
    private $checkoutControllerObj;

    public function __construct() {

        $this->rechargeObj = new RechargeApi();
        $this->iterableObj = new IterableApi();
        $this->productObj = new Product();
    }

    /**
     * Upsell endpoint to update subscription with addon
     */
    public function upsellSubscription(Request $request) {

        $requestData = !empty($request->input('data')) ? $request->input('data') : "";
        if (empty($requestData)) {
            $message = "Endpoint parameters are missing";
            $error = "Bad request";
            $status = "Failed";
            $response = array(
                "status"       => $status,
                "message" => $message,
                "error" => $error
            );
            $response = !empty($request->input('callback')) ? $request->input('callback').'('.json_encode($response).')' : 
            json_encode($response);
            Log::channel('subscriptionControllerLog')->info("upsellSubscription: Endpoint parameters are missing");
            echo $response;
            exit();
        }
        Log::channel('subscriptionControllerLog')->info("upsellSubscription: Request =>".$requestData);
        $receivedData = json_decode($requestData);
        $subscriptionID = !empty($receivedData->subscription_id) ? trim($receivedData->subscription_id) : "";
        $urlCustomerRechargeHash = !empty($receivedData->recharge_hash) ? trim($receivedData->recharge_hash) : "";
        if (empty($subscriptionID)) {
            $errorMessage = "The subscription id is missing in the request";
            Log::channel('subscriptionControllerLog')->info("upsellSubscription: ".$errorMessage);
            return $this->responseData("Failed", $errorMessage, $request->input('callback'));
        }
        if(empty($urlCustomerRechargeHash)) {
            $errorMessage = "The customer recharge hash is missing in the request";
            Log::channel('subscriptionControllerLog')->info("upsellSubscription: ".$errorMessage);
            return $this->responseData("Failed", $errorMessage, $request->input('callback'));
        }
        $propertyStatus = 0;
        if (empty($receivedData->addons)) {
            $propertyStatus = 1;
        } else {
            foreach ($receivedData->addons as $Value) {
                $addonProductId = !empty($Value->product_id) ? trim($Value->product_id) : "";
                $addonVariantId = !empty($Value->variant_id) ? trim($Value->variant_id) : "";
                $addonTitle = !empty($Value->title) ? trim($Value->title) : "";
                if (empty($addonProductId) || empty($addonVariantId) || empty($addonTitle)) {
                    $propertyStatus = 1;
                }
            }
        }
        if ($propertyStatus == 1) {
            $errorMessage = "Missing required parameters in the line item properties";
            Log::channel('subscriptionControllerLog')->info("upsellSubscription: ".$errorMessage.
            "(SubscriptionId:".$subscriptionID.")");
            return $this->responseData("Failed", $errorMessage, $request->input('callback'));
        }

        $subscriptionDetails =  $this->rechargeObj->getSubscription($subscriptionID);

        if (!empty($subscriptionDetails->error) || !empty($subscriptionDetails->errors) ||
            empty($subscriptionDetails) || empty($subscriptionDetails->subscription)) {
            $status = "failed";
            $error  = "Unknown error";
            if (!empty($subscriptionDetails->error)) {
                $error = json_encode($subscriptionDetails->error);
            } else if (!empty($subscriptionDetails->errors)) {
                $error = json_encode($subscriptionDetails->errors);
            } else {
                $error = "Failed to get the subscription details";
            }
            $errorMessage = "Failed to get the subscription details";
            Log::channel('subscriptionControllerLog')->info("upsellSubscription: ".$errorMessage.", Error:".$error);
            return $this->responseData("Failed", $errorMessage, $request->input('callback'));
        }
        $customerID = $subscriptionDetails->subscription->customer_id;

        $customerDetails = $this->rechargeObj->getRechargeCustomerDetailsByID($customerID);

        if (!empty($customerDetails->error) || !empty($customerDetails->errors) ||
            empty($customerDetails) || empty($customerDetails->customer)) {
            $status = "failed";
            $error  = "Unknown error";
            if (!empty($customerDetails->error)) {
                $error = json_encode($customerDetails->error);
            } else if (!empty($customerDetails->errors)) {
                $error = json_encode($customerDetails->errors);
            } else {
                $error = "Failed to get the customer details";
            }
            $errorMessage = "Failed to get the customer details";
            Log::channel('subscriptionControllerLog')->info("upsellSubscription: ".$errorMessage.", Error:".$error);
            return $this->responseData("Failed", $errorMessage, $request->input('callback'));
        }
        $customerRechargeHash = !empty($customerDetails->customer->hash) ? trim($customerDetails->customer->hash) : "";
	    $customerEmail = !empty($customerDetails->customer->email) ? trim($customerDetails->customer->email) : "";
        if ($customerRechargeHash != $urlCustomerRechargeHash) {
            $errorMessage = "Failed to validate the customer recharge hash";
            Log::channel('subscriptionControllerLog')->info("upsellSubscription: ".$errorMessage);
            return $this->responseData("Failed", $errorMessage, $request->input('callback'));
        } else {
            $endpointData = [];
            $vitaminCount = 1;
            $shopifyProductDetails = [];
            $subscriptionDetails = !empty($subscriptionDetails) ? json_decode(json_encode($subscriptionDetails), true) : '';

            $shopifyProductDetails = $this->getShopifyProductDetails();

            if (!isset($shopifyProductDetails)) {
                $errorMessage = "Failed to get the products from Shopify";
                Log::channel('subscriptionControllerLog')->info("upsellSubscription: ".$errorMessage.
                "(SubscriptionId:".$subscriptionID.")");
                return $this->responseData("Failed", $errorMessage, $request->input('callback'));
            }
            foreach ($subscriptionDetails['subscription']['properties'] as $subProperty) {
                foreach ($shopifyProductDetails as $shValue) {
                    foreach ($shValue as $variantValue) {
                        $propertyName = strtolower(trim($subProperty['name']));
                        $propertyValue = trim($subProperty['value']);
                        if (strpos($propertyName, 'vitamin') !== false && strpos($propertyValue, '|') !== false) {
                            $freePill = false;
                            if (strpos($propertyValue, 'free_pill') !== false) {
                                $freePill = true;
                            }
                            $explodeValue = explode('|', $propertyValue);
                            if($explodeValue[3] == $variantValue['id'] || $explodeValue[4] == $variantValue['sku']) {
                                $endpointData[] = array(
                                    "name" => trim($subProperty['name']),
                                    "value" => trim($explodeValue[0]),
                                    "product_id" => intval($variantValue['product_id']),
                                    "variant_id" => intval($explodeValue[3]),
                                    "quantity" => intval($explodeValue[2]),
                                    "dosage" => trim($explodeValue[1]),
                                    "free_pill" => $freePill
                                );
                                $vitaminCount++;
                            }
                        }
                    }
                }
            }
            foreach ($receivedData->addons as $Value) {
                $endpointData[] = array(
                    "name" => 'Vitamin'. ($vitaminCount<10 ? '0'.$vitaminCount++ : $vitaminCount++),
                    "value" => trim($Value->title),
                    "product_id" => intval($Value->product_id),
                    "variant_id" => intval($Value->variant_id),
                    "quantity" => !empty($Value->quantity) ? intval($Value->quantity) : 1,
                    "dosage" => trim($Value->dosage),
                    "free_pill" => false
                );
            }

            $subscriptionUpdateRequest = array(
                "subscription_id" => $subscriptionID,
                "properties" => $endpointData
            );

            $laravelRequestObj = new \Illuminate\Http\Request();
            $laravelRequestObj->setMethod('POST');
            $this->checkoutControllerObj = new CheckoutController();
            $encodedEndpointRequest = json_encode($subscriptionUpdateRequest);
            $laravelRequestObj->request->add([
                "callback" => "jQuery34109070734856041867_1640848631591",
                "data" => $encodedEndpointRequest
            ]);
            
            $updateResponse = $this->checkoutControllerObj->updateSubscriptionLineItemProperties($laravelRequestObj);
            
            if (strtolower($updateResponse['status']) == 'success') {
                $response = array(
                    "status"             => "success",
                    "subscription_price" => $updateResponse['subscription_price'],
                    "melatonin_warning"  => $updateResponse['melatonin_warning'] == 1 ? true : false,
                    "shipping_amount"    => $updateResponse['shipping_amount'],
                    "shipping_rate_update_status" => $updateResponse['shipping_rate_update_status'] == 1 ? true : false,
                    "errorMessage"       => "",
                );
            } else {
                $response = array(
                    "status"       => "Failed",
                    "errorMessage" => "Failed to update the subscription"."(".$subscriptionID.") - ".$customerEmail,
                );
                $error = "Failed to update the subscription"."(".$subscriptionID.") - ".$customerEmail;
                Log::channel('subscriptionControllerLog')->info("upsellSubscription: Error => ".$error);
            }
            echo $request->input('callback') . '(' . json_encode($response) . ')';
            exit();
        }
    }

    /**
     * Response function
     */
    public function responseData($status, $errorMessage, $callBackToken) {

        $response = array(
            "status"       => $status,
            "errorMessage" => $errorMessage
        );
        echo $callBackToken . '(' . json_encode($response) . ')';
        exit();
    }

    /**
     * Reactivate subscription by email address
     */
	public function reactivateSubscriptionByEmail(Request $request) {

        $email = !empty($request->input('email')) ? trim($request->input('email')) : "";
        if (empty($email)) {
            $message = "Failed to reactivate the subscription";
            $error = "Missing required parameter email";
            Log::channel('subscriptionControllerLog')->info("reactivateSubscriptionByEmail: 
            Missing required parameter email");
            $this->sendResponse("Failed", $message, $error);
        }
        Log::channel('subscriptionControllerLog')->info("reactivateSubscriptionByEmail: Request => ".$email);

        $customer = $this->rechargeObj->getRechargeCustomerDetailsByEmail($email);

        if (isset($customer->error) || isset($customer->errors) || empty($customer)) {
            $status = "failed";
            $error  = "Unknown error";
            if (isset($customer->error)) {
                $error = json_encode($customer->error);
            } else if (isset($customer->errors)) {
                $error = json_encode($customer->errors);
            } else {
                $error = "Failed to get the customer details from recharge";
            }
            $message = "Failed to reactivate the subscription";
            Log::channel('subscriptionControllerLog')->info("reactivateSubscriptionByEmail: Failed to get the customer 
            details by email, Email: ".$email.", Error => ".$error);
            $this->sendResponse("Failed", $message, $error);
        }
        if (empty($customer->customers[0]->id)) {
            $message = "Failed to reactivate the subscription";
            $error = "Customer not found";
            Log::channel('subscriptionControllerLog')->info("reactivateSubscriptionByEmail: Searched customer not 
            found in recharge, Email: ".$email);
            $this->sendResponse("Failed", $message, $error);
        }
        $customerDetails = '';
        foreach ($customer->customers as $customerDetail) {
            if (trim(strtolower($customerDetail->email)) == strtolower($email)) {
                $customerDetails = $customerDetail;
            }
        }
        $customer_id = $customerDetails->id;

        $subscription = $this->rechargeObj->getSubscriptionByCustomerID($customer_id);

        if (isset($subscriptions->error) || isset($subscription->errors) || empty($subscription)) {
            $status = "failed";
            $error  = "Unknown error";
            if (isset($subscription->error)) {
                $error = json_encode($subscription->error);
            } else if (isset($subscription->errors)) {
                $error = json_encode($subscription->errors);
            } else {
                $error = "Failed to get the subscriptions by customer Id";
            }
            $message = "Failed to reactivate the subscription";
            Log::channel('subscriptionControllerLog')->info("reactivateSubscriptionByEmail: Failed to get the subscriptions 
            by customerId, CustomerId:".$customer_id." Error => ".$error);
            $this->sendResponse("Failed", $message, $error);
        }
        if (empty($subscription->subscriptions)) {
            $message = "Failed to reactivate the subscription";
            $error = "Subscription not found";
            Log::channel('subscriptionControllerLog')->info("reactivateSubscriptionByEmail: No subscriptions found, 
            CustomerId:".$customer_id.", Error => ".$error);
            $this->sendResponse("Failed", $message, $error);
        }
        $subscription_id = trim($subscription->subscriptions[0]->id);

        $res = $this->rechargeObj->activateSubscription($subscription_id);

        if (isset($res->error) || isset($res->errors) || empty($res)) {
            $status = "failed";
            $error  = "Unknown error";
            if (isset($res->error)) {
                $error = json_encode($res->error);
            } else if (isset($res->errors)) {
                $error = json_encode($res->errors);
            } else {
                $error = "Failed to reactivate the subscription";
            }
            $message = "Failed to reactivate the subscription";
            Log::channel('subscriptionControllerLog')->info("reactivateSubscriptionByEmail: Failed to reactivate the subscription, 
            SubscriptionId: ".$subscription_id.", Error => ".$error);
            $this->sendResponse("Failed", $message, $error);
        } else {
            $this->sendResponse("Success", "Subscription reactivated successfully", "");
        }
    }

    /**
     * Reactivate cancelled recharge subscription
     */
    public function reactivateSubscriptionById(Request $request) {

        $requestData = !empty($request->input('data')) ? trim($request->input('data')) : "";
        if (empty($requestData)) {
            $message = "Endpoint parameters are missing";
            $error = "Bad request";
            $status = "Failed";
            $response = array(
                "status"       => $status,
                "message" => $message,
                "error" => $error
            );
            Log::channel('subscriptionControllerLog')->info("reactivateSubscriptionById: Endpoint parameters are missing");
            echo json_encode($response);
            exit();
        }
        Log::channel('subscriptionControllerLog')->info("reactivateSubscriptionById: Request =>".$requestData);
        $receivedData = json_decode($requestData);
        $receivedSubscriptions = !empty($receivedData->subscriptions) ? $receivedData->subscriptions : "";
        $receivedHash = !empty($receivedData->recharge_hash) ? trim($receivedData->recharge_hash) : "";
        if (empty($receivedSubscriptions)) {
            $message = "Failed to reactivate the subscription";
            $error = "Missing required parameter subscription_id";
            Log::channel('subscriptionControllerLog')->info("reactivateSubscriptionById: Missing required parameter 
            subscription_id");
            $this->sendResponse("Failed", $message, $error);
        }
        $subscriptionIdStatus = 0;
        $subscription_id = '';
        $subscriptionIDs = array();
        foreach ($receivedData->subscriptions as $dataValue) {
          if (empty($dataValue->subscription_id)) {
            $subscriptionIdStatus = 1;
          } else {
              if (empty($subscription_id)) {
                $subscription_id = $dataValue->subscription_id;
              }
              $subscriptionIDs[] = $dataValue->subscription_id;
          }
        }
        if ($subscriptionIdStatus == 1 || empty($receivedData->subscriptions)) {
            $message = "Failed to reactivate the subscription";
            $error = "Missing required parameter subscription_id";
            Log::channel('subscriptionControllerLog')->info("reactivateSubscriptionById: Missing required parameter 
            subscription_id");
            $this->sendResponse("Failed", $message, $error);
        }
        if (empty($receivedHash)) {
            $message = "Failed to reactivate the subscription";
            $error = "Missing required parameter recharge customer hash";
            Log::channel('subscriptionControllerLog')->info("reactivateSubscriptionById: Missing required parameter 
            recharge customer hash");
            $this->sendResponse("Failed", $message, $error);
        }
        $subscription_id = trim($subscription_id);
        $recharge_hash = trim($receivedHash);

        $subscriptionDetails = $this->rechargeObj->getSubscription($subscription_id);

        if (!isset($subscriptionDetails) || isset($subscriptionDetails->error) || isset($subscriptionDetails->errors) || 
            empty($subscriptionDetails->subscription->customer_id)) {
            $status = "failed";
            $error  = "Unknown error";
            if (isset($subscriptionDetails->error)) {
                $error = json_encode($subscriptionDetails->error);
            } else if (isset($subscriptionDetails->errors)) {
                $error = json_encode($subscriptionDetails->errors);
            } else {
                $error = "Not found the subscription details by Id";
            }
            $message = "Failed to get the subscription details";
            Log::channel('subscriptionControllerLog')->info("reactivateSubscriptionById: Failed to get the subscription details, 
            Error => ".$error);
            $this->sendResponse("Failed", $message, $error);
        }
        $customerId = trim($subscriptionDetails->subscription->customer_id);

        $customerDetails = $this->rechargeObj->getRechargeCustomerDetailsByID($customerId);

        if (!isset($customerDetails) || isset($customerDetails->error) || isset($customerDetails->errors) || 
            empty($customerDetails->customer->hash)) {
            $status = "failed";
            $error  = "Unknown error";
            if (isset($customerDetails->error)) {
                $error = json_encode($customerDetails->error);
            } else if (isset($customerDetails->errors)) {
                $error = json_encode($customerDetails->errors);
            } else {
                $error = "Failed to get the customer details from recharge";
            }
            $message = "Failed to get the customer details";
            Log::channel('subscriptionControllerLog')->info("reactivateSubscriptionById: Failed to get the customer details, 
            Error => ".$error);
            $this->sendResponse("Failed", $message, $error);
        }
        $rechargeCustomerHash = trim($customerDetails->customer->hash);
        if ($recharge_hash == $rechargeCustomerHash) {
            $subscriptionIDs = array_unique($subscriptionIDs);
            $userSubscriptionIDs = implode( ",", $subscriptionIDs);
            $iterableRequest = array(
                "email" => $customerDetails->customer->email,
                "dataFields" => array(
                    "subscriptionID" => $userSubscriptionIDs,
                    "rechargeHash" => $recharge_hash
                )
            );
            $errorResponses = array();
            $errorStatus = 0;
            foreach ($subscriptionIDs as $subscriptionID) {
                sleep(1);
                $reactivateResponse = $this->rechargeObj->activateSubscription($subscriptionID);

                if (isset($reactivateResponse->error) || isset($reactivateResponse->errors) ||
                    empty($reactivateResponse)) {
                    $errorStatus = 1;
                    $error  = "Unknown error";
                    if (isset($reactivateResponse->error)) {
                        $error  = json_encode($reactivateResponse->error);
                        $errorResponses[] = $subscriptionID.":".$error;

                    } else if (isset($reactivateResponse->errors)) {
                        $error = json_encode($reactivateResponse->errors);
                        $errorResponses[] = $subscriptionID.":".$error;
                    } else {
                        $error = "Failed to reactivate the subscription";
                        $errorResponses[] = $subscriptionID.":".$error;
                    }
                }
            }
            if ($errorStatus == 1) {
                $subscriptionErrors = implode( ",", $errorResponses);
                $error = "Failed subscriptions - ".$subscriptionErrors;
                $message = "Failed to reactivate the subscription";
                Log::channel('subscriptionControllerLog')->info("reactivateSubscriptionById: Error => ".$error);
                $this->sendResponse("Failed", $message, $error);
            } else {
                $updateIterableData = $this->iterableObj->updateUserDataFields(json_encode($iterableRequest));

                if (strtolower(trim($updateIterableData->code)) == 'success') {
                    Log::channel('iterableWebhookLog')->info("reactivateSubscriptionById: Successfully updated the subscriptionId and 
                    recharge hash (Email - ".strtolower($customerDetails->customer->email).")");
                } else {
                    Log::channel('iterableWebhookLog')->info("reactivateSubscriptionById: Failed to update the subscriptionId and 
                    recharge hash (Email - ".strtolower($customerDetails->customer->email).")");
                }
                $this->sendResponse("Success", "Subscription reactivated successfully", "");
            }
        } else {
            $message = "Failed to reactivate the subscription";
            $error = "Customer hashtags are not matched";
            Log::channel('subscriptionControllerLog')->info("reactivateSubscriptionById: Error => ".$error);
            $this->sendResponse("Failed", $message, $error);
        }
    }

    /**
     * Response function
     */
    public function sendResponse($status,$message, $error) {

        $response = array(
            "status" => $status,
            "message" => $message,
            "error" => $error
        );
        echo json_encode($response);
        exit();
    }

    /**
     * Function to get all shopify products
     */
    public function getShopifyProductDetails() {

        $shopifyProductDetails = array();
        $pageNo = 1;
        $limit  = 100;
        do {
            try {
                if (!empty($products['link'])) {
                    $products = $this->productObj->getShopifyProducts($limit,$products['link']);
                 } else {
                    $products = $this->productObj->getShopifyProducts($limit);
                }
            } catch (Exception $e) {
                Log::channel('subscriptionControllerLog')->info("getShopifyProductDetails: Failed to get the products from Shopify");
                exit();
            }
            if (!empty($products['error'])) {
                Log::channel('subscriptionControllerLog')->info("getShopifyProductDetails: Failed to get the products from Shopify");
                return $shopifyProductDetails;
            }
            if ($products) {
                foreach ($products['body'] as $product) {
                    $shopifyProductDetails[] = $product['variants'];
                }
                $productCount = count($products['body']);
            }
            $pageNo++;

            if (empty($products['link'])) {
                break;
            }
        } while($productCount == $limit);

        return $shopifyProductDetails;
    }

    /**
     * Pause(change charge date) a subscription and create a 'Subscription pause' event
     */
    public function pauseSubscription(Request $data) {

        $receivedData = !empty($data->input('data')) ? trim($data->input('data')) : "";

        if (empty($receivedData)) {
            $message = "Endpoint parameters are missing";
            $error = "Bad request";
            $status = "Failed";
            Log::channel('subscriptionControllerLog')->info("pauseSubscription: Endpoint parameters are missing");
            $this->requestResponse($status, $message, $error, $data->input('callback'));
        }
        Log::channel('subscriptionControllerLog')->info("pauseSubscription: Request => ".$receivedData);
        $receivedData = json_decode($receivedData);
        if (empty($receivedData)) {
            $message = "Endpoint parameters are missing";
            $error = "Bad request";
            $status = "Failed";
            Log::channel('subscriptionControllerLog')->info("pauseSubscription: Endpoint parameters are missing");
            $this->requestResponse($status, $message, $error, $data->input('callback'));
        }
        $emptyKey = null;
        $requiredRequestKeys = ["subscription_id", "paused_period", "hash_value"];
        $requestKeys = [];
        $valueStatus = 0;
        foreach ($receivedData as $key => $receiveValue) {
            $requestKeys[] = $key;
            if (empty($receiveValue) && $valueStatus == 0) {
                $emptyKey = $key;
                $valueStatus = 1;
            }
        }
        $keyCompareStatus = array_diff($requiredRequestKeys, $requestKeys);
        if(!empty($keyCompareStatus)) {
            $missedParameters = implode(",",$keyCompareStatus);
            $message = $missedParameters." endpoint parameters are missing";
            $error = "Some parameters are missing in the endpoint request";
            $status = "Failed";
            Log::channel('subscriptionControllerLog')->info("pauseSubscription: Some parameters are missing in the 
            endpoint request");
            $this->requestResponse($status, $message, $error, $data->input('callback'));
        }
        if (!empty($emptyKey)) {
            $message = $emptyKey." is missing";
            $error = "Bad request";
            $status = "Failed";
            Log::channel('subscriptionControllerLog')->info("pauseSubscription: Parameters are missing in the request, 
            Error => ".$message);
            $this->requestResponse($status, $message, $error, $data->input('callback'));
        }
        $subscriptionID = trim($receivedData->subscription_id);

        $subscriptionDetails =  $this->rechargeObj->getSubscription($subscriptionID);

        if (!empty($subscriptionDetails->error) || !empty($subscriptionDetails->errors) ||
            empty($subscriptionDetails) || empty($subscriptionDetails->subscription)) {
            $status = "failed";
            $error  = "Unknown error";
            if (!empty($subscriptionDetails->error)) {
                $error = json_encode($subscriptionDetails->error);
            } else if (!empty($subscriptionDetails->errors)) {
                $error = json_encode($subscriptionDetails->errors);
            } else {
                $error = "Failed to get the subscription details by using SubscriptionID:".$subscriptionID;
            }
            $message = "Failed to get the subscription details from recharge";
            Log::channel('subscriptionControllerLog')->info("pauseSubscription: Error => ".$error);
            $this->requestResponse("Failed", $message, $error, $data->input('callback'));
        }
        $customerID = $subscriptionDetails->subscription->customer_id;

        $customerDetails = $this->rechargeObj->getRechargeCustomerDetailsByID($customerID);

        if (!empty($customerDetails->error) || !empty($customerDetails->errors) ||
            empty($customerDetails) || empty($customerDetails->customer)) {
            $status = "failed";
            $error  = "Unknown error";
            if (!empty($customerDetails->error)) {
                $error = json_encode($customerDetails->error);
            } else if (!empty($customerDetails->errors)) {
                $error = json_encode($customerDetails->errors);
            } else {
                $error = "Failed to get the customer details by using CustomerID:".$customerID;
            }
            $message = "Failed to get the customer details from recharge";
            Log::channel('subscriptionControllerLog')->info("pauseSubscription: Error => ".$error);
            $this->requestResponse("Failed", $message, $error, $data->input('callback'));
        }
        $customerHash = strtolower(trim($receivedData->hash_value));
        $originalCustomerHash = strtolower(trim($customerDetails->customer->hash));
        if ($customerHash != $originalCustomerHash) {
            $message = "Customer hash values are not matching";
            $error = "Security breach";
            $status = "Failed";
            Log::channel('subscriptionControllerLog')->info("pauseSubscription: Customer hash values are not matching");
            $this->requestResponse($status, $message, $error, $data->input('callback'));
        }
        if (empty($subscriptionDetails->subscription->next_charge_scheduled_at)) {
            $message = "The next charge date is not found in the subscription - ".$subscriptionID;
            $error = "Next charge date not found";
            $status = "Failed";
            Log::channel('subscriptionControllerLog')->info("pauseSubscription: Next charge date not found, 
            SubscriptionId:".$subscriptionID);
            $this->requestResponse($status, $message, $error, $data->input('callback'));
        }
        $subscriptionCurrentChargeDate = $subscriptionDetails->subscription->next_charge_scheduled_at;
        $subscriptionCurrentChargeDate = date('Y-m-d', strtotime($subscriptionCurrentChargeDate));
        $pausePeriod = (int)$receivedData->paused_period;
        $newChargeDate = date('Y-m-d',strtotime('+'.$pausePeriod.' days',strtotime($subscriptionCurrentChargeDate)));
        $payload = array(
            "date" => $newChargeDate
        );

        $subscriptionUpdateResponse = $this->rechargeObj->updateSubscriptionChargeDate($subscriptionID, json_encode($payload));

        if (!empty($subscriptionUpdateResponse->error) || !empty($subscriptionUpdateResponse->errors) ||
            empty($subscriptionUpdateResponse)) {
            $status = "failed";
            $error  = "Unknown error";
            if (!empty($subscriptionUpdateResponse->error)) {
                $error = json_encode($subscriptionUpdateResponse->error);
            } else if (!empty($subscriptionUpdateResponse->errors)) {
                $error = json_encode($subscriptionUpdateResponse->errors);
            } else {
                $error = "Failed to update the subscription charge date - ".$subscriptionID;
            }
            $message = "Failed to pause the subscription in recharge";
            Log::channel('subscriptionControllerLog')->info("pauseSubscription: Error => ".$error);
            $this->requestResponse("Failed", $message, $error, $data->input('callback'));
        }
        $eventRequest = array(
            "email" => strtolower($customerDetails->customer->email), 
            "eventName" => "Subscription paused",
            "dataFields" => array(
                "delay" => trim($receivedData->paused_period)
            )
        );

        $eventResponse = $this->iterableObj->createIterableEvent(json_encode($eventRequest));

        if (isset($eventResponse->code) || trim(strtolower($eventResponse->code)) == 'success') {
            Log::channel('subscriptionControllerLog')->info("pauseSubscription: Paused subscription iterable event - 
            Successfully created event for Email (".strtolower($customerDetails->customer->email).")");
        } else {
            Log::channel('subscriptionControllerLog')->info("pauseSubscription: Cancelled subscription iterable event - 
            Failed to create event for Email (".strtolower($customerDetails->customer->email).")");
        }
        $message = "Successfully paused the subscription";
        $error = "";
        $status = "Success";
        $this->requestResponse($status, $message, $error, $data->input('callback'));
    }

    /**
     * Response function
     */
    public function requestResponse($status, $message, $error, $callback) {
        $response = array(
            "status"       => $status,
            "message" => $message,
            "error" => $error
        );
        echo $callback . '(' . json_encode($response) . ')';
        exit();
    }

    /**
     * Convert OTP to subscription
     */
    public function otpToSubscription(Request $data) {

        $receivedData = !empty($data->input('data')) ? trim($data->input('data')) : "";
        if (empty($data['data'])) {
            $message = "Endpoint parameters are missing";
            $error = "Bad request";
            $status = "Failed";
            Log::channel('subscriptionControllerLog')->info("otpToSubscription: Endpoint parameters are missing");
            $this->requestResponse($status, $message, $error, $data->input('callback'));
        }
        Log::channel('subscriptionControllerLog')->info("otpToSubscription: Request => ".$receivedData);
        $receivedData = json_decode($receivedData);
        if (empty($receivedData)) {
            $message = "Endpoint parameters are missing";
            $error = "Bad request";
            $status = "Failed";
            Log::channel('subscriptionControllerLog')->info("otpToSubscription: Endpoint parameters are missing");
            $this->requestResponse($status, $message, $error, $data->input('callback'));
        }
        $emptyKey = null;
        $requiredRequestKeys = ["address_id", "hash_value"];
        $requestKeys = [];
        $valueStatus = 0;
        foreach ($receivedData as $key => $receiveValue) {

            $requestKeys[] = $key;
            if (empty($receiveValue) && $valueStatus == 0) {
                $emptyKey = $key;
                $valueStatus = 1;
            }
        }
        $keyCompareStatus = array_diff($requiredRequestKeys, $requestKeys);
        if(!empty($keyCompareStatus)) {
            $missedParameters = implode(",",$keyCompareStatus);
            $message = $missedParameters." endpoint parameters are missing";
            $error = "Some parameters are missing in the endpoint request";
            $status = "Failed";
            Log::channel('subscriptionControllerLog')->info("otpToSubscription: Some parameters are missing in the 
            endpoint request");
            $this->requestResponse($status, $message, $error, $data->input('callback'));
        }
        if (!empty($emptyKey)) {
            $message = $emptyKey." is missing";
            $error = "Bad request";
            $status = "Failed";
            Log::channel('subscriptionControllerLog')->info("otpToSubscription: Parameters are missing in the request");
            $this->requestResponse($status, $message, $error, $data->input('callback'));
        }
        $lineItemParameterStatus = 1;
        if (empty($receivedData->line_items)) {
            $lineItemParameterStatus = 0;
        } else {
            foreach ($receivedData->line_items as $key => $receiveValue) {
                if (empty($receiveValue->variant_id) || empty($receiveValue)) {
                    $lineItemParameterStatus = 0;
                }
            }
        }
        $discountCode = null;
        $addressID = trim($receivedData->address_id);

        $otpChargeDetails = $this->rechargeObj->getOtpChargeDetails($addressID);

        if (!empty($otpChargeDetails->error) || !empty($otpChargeDetails->errors) ||
            empty($otpChargeDetails)) {
            $status = "failed";
            $error  = "Unknown error";
            if (!empty($otpChargeDetails->error)) {
                $error = json_encode($otpChargeDetails->error);
            } else if (!empty($otpChargeDetails->errors)) {
                $error = json_encode($otpChargeDetails->error);
            } else {
                $error = "Failed to get the charge details by using addressID:".$addressID;
            }
            $message = "Failed to get the charge details from recharge";
            Log::channel('subscriptionControllerLog')->info("otpToSubscription: Error => ".$error);
            $this->requestResponse("Failed", $message, $error, $data->input('callback'));
        }
        $customerHash = strtolower(trim($receivedData->hash_value));
        $originalCustomerHash = null;
        $customerEmail = null;
        $username = null;
        $rechargeChargeDetails = null;
        $subscriptionExistingStatus = null;
        foreach ($otpChargeDetails->charges as $chargeValue) {
            if ($addressID == trim($chargeValue->address_id) &&
                $customerHash == strtolower(trim($chargeValue->customer_hash))) {
                $originalCustomerHash = $chargeValue->customer_hash;
                $customerEmail = $chargeValue->email;
                $username  = !empty($chargeValue->first_name) ? $chargeValue->first_name.' ' : '';
                $username .= !empty($chargeValue->last_name) ? $chargeValue->last_name : '';
                $chargeTags = strtolower(trim($chargeValue->tags));
                if (strpos($chargeTags, 'otp' ) !== false) {
                    $rechargeChargeDetails = $chargeValue;
                }
                if (strpos($chargeTags, 'subscription') !== false &&
                    strtolower(trim($chargeValue->status)) == 'queued') {
                    $subscriptionExistingStatus = $chargeValue->status;
                }
            }
        }
        if (empty($originalCustomerHash)) {
            $message = "Customer hash value is not matching";
            $error = "Security breach";
            $status = "Failed";
            Log::channel('subscriptionControllerLog')->info("otpToSubscription: Customer hash value is not matching");
            $this->requestResponse($status, $message, $error, $data->input('callback'));
        }
        if (!empty($subscriptionExistingStatus)) {
            $message = "There is already a queued charge in the given addressID:".$addressID;
            $error = "Existing an active queued charge";
            $status = "Failed";
            Log::channel('subscriptionControllerLog')->info("otpToSubscription: Already existing an active queued charge");
            $this->requestResponse($status, $message, $error, $data->input('callback'));
        }
        $chargeTags = strtolower(trim($otpChargeDetails->charges[0]->tags));
        if (empty($rechargeChargeDetails)) {
            $message = "The charge in the given addressID is not an OTP charge";
            $error = "OTP charge not found";
            $status = "Failed";
            Log::channel('subscriptionControllerLog')->info("otpToSubscription: OTP charge not found");
            $this->requestResponse($status, $message, $error, $data->input('callback'));
        }
        $otpLineItemQuantity = [];
        $lineItemProperties = [];
        $chargeVariantIDs = [];
        $otpFreePill = [];
        foreach ($rechargeChargeDetails->line_items[0]->properties as $propertyValue) {
            if (!empty($propertyValue->name) && $propertyValue->name != "one_time_purchase") {
                $lineItemProperties[] = $propertyValue;
                $propertyKey = strtolower(trim($propertyValue->name));
                if (strpos($propertyKey, 'vitamin') !== false) {
                    $explodeValue = explode("|", $propertyValue->value);
                    $otpLineItemQuantity[$explodeValue[3]] = $explodeValue[2];
                    $chargeVariantIDs[] = $explodeValue[3];
                    if (strpos($propertyValue->value, "free_pill") !== false) {
                        $otpFreePill[] = $explodeValue[3];
                    }
                }
            }
        }
        if (empty($chargeVariantIDs)) {
            $message = "The line items in the OTP charge is empty addressID:".$addressID;
            $error = "Line items not found in the OTP charge";
            $status = "Failed";
            Log::channel('subscriptionControllerLog')->info("otpToSubscription: Line items not found in the OTP charge");
            $this->requestResponse($status, $message, $error, $data->input('callback'));
        }
        if ($lineItemParameterStatus == 1) {
            $endpointVariantIDMismatchStatus = null;
            foreach ($receivedData->line_items as $lineValue) {

                if (in_array($lineValue->variant_id, $chargeVariantIDs) == false) {
                    $endpointVariantIDMismatchStatus = 1;
                }
                if (count($receivedData->line_items) != count($chargeVariantIDs)) {
                    $endpointVariantIDMismatchStatus = 1;
                }
            }
            if ($endpointVariantIDMismatchStatus == 1) {
                $message = "The requested endpoint variantID and OTP charge variantID are mismatched";
                $error = "Product mismatch found";
                $status = "Failed";
                Log::channel('subscriptionControllerLog')->info("otpToSubscription: Product mismatch found");
                $this->requestResponse($status, $message, $error, $data->input('callback'));
            }
        }
        $lineItemProperties[] = (object)array(
            "name" => "charge_interval_frequency",
            "value" => 28
        );
        $lineItemProperties[] = (object)array(
            "name" => "shipping_interval_frequency",
            "value" => 28
        );
        $lineItemProperties[] = (object)array(
            "name" => "shipping_interval_unit_type",
            "value" => "day"
        );
        $productVariantIDs = null;
        if ($lineItemParameterStatus == 1) {
            $productVariantIDs = $receivedData->line_items;
        } else {
            foreach ($chargeVariantIDs as $chargeValue) {
                $productVariantIDs[] = (object) ["variant_id" => intval($chargeValue)];
            }
        }
        $variantErrorStatus = 0;
        $subscriptionPrice = null;
        foreach ($productVariantIDs as $lineValue) {
            $variantDetails = $this->productObj->getShopifyVariantByID($lineValue->variant_id);
            if (!empty($variantDetails['errors']) || !empty($variantDetails['error'])) {
                $variantErrorStatus = 1;
                $variantMismatchID = $lineValue->variant_id;
                $variantError = !empty($variantDetails['errors']) ? $variantDetails['errors'] :
                $variantDetails['error'];
                break;
            }
            foreach ($otpLineItemQuantity as $propertyKey => $otpPropertyValue) {
                if (in_array($propertyKey, $otpFreePill)) {
                    continue;
                }
                if (trim($propertyKey) == trim($variantDetails['variant']['id'])) {
                    $subscriptionPrice = floatval($subscriptionPrice) +
                    (floatval($variantDetails['variant']['price']) * intval($otpPropertyValue));
                }
            }
        }
        if ($variantErrorStatus == 1) {
            $message = "The variantID in the endpoint request is not valid variantID: ".$variantMismatchID.", Error: ".$variantError;
            $error = "Invalid variantID found";
            $status = "Failed";
            Log::channel('subscriptionControllerLog')->info("otpToSubscription: Invalid variantID found, Error => ".$message);
            $this->requestResponse($status, $message, $error, $data->input('callback'));
        }
        $subscriptionSKU = "vitamin-pack";
        $today = time();
        $todayDate = date('Y-m-d');
        $defaultSubscriptionPeriod = 28;
        $otpChargeProcessedDate = date('Y-m-d', strtotime($rechargeChargeDetails->processed_at));
        $otpChargeProcessedDate = strtotime($otpChargeProcessedDate);
        $dateDiff = $today - $otpChargeProcessedDate;
        $dateDiffDays = round($dateDiff / (60 * 60 * 24));
        $nextChargePeriod = $defaultSubscriptionPeriod - $dateDiffDays;
        if ($nextChargePeriod <= 0) {
            $nextChargeDate = $todayDate;
        } else {
            $nextChargeDate = date('Y-m-d',strtotime('+'.$nextChargePeriod.' days',strtotime($todayDate)));
        }
        $shopifyVariantId = env("REGULAR_SUBSCRIPTION_VARIANT_ID", true);
        $intervalUnit = "day";

        $subscriptionRequest = array(
            "address_id" => $rechargeChargeDetails->address_id,
            "next_charge_scheduled_at" => $nextChargeDate,
            //"next_charge_scheduled_at" => $todayDate,
            "quantity" => !empty($rechargeChargeDetails->line_items[0]->quantity) ?
            $rechargeChargeDetails->line_items[0]->quantity : 1,
            "price" => floatval($subscriptionPrice),
            "sku" => $subscriptionSKU,
            "shopify_variant_id" => $shopifyVariantId,
            "order_interval_unit" => $intervalUnit,
            "order_interval_frequency" => $defaultSubscriptionPeriod,
            "charge_interval_frequency" => $defaultSubscriptionPeriod,
            "properties" => $lineItemProperties
        );

        $subscriptionCreateResponse = $this->rechargeObj->createSubscription(json_encode($subscriptionRequest));

        if (!empty($subscriptionCreateResponse->error) || !empty($subscriptionCreateResponse->errors) ||
            empty($subscriptionCreateResponse)) {
            $status = "failed";
            $error  = "Unknown error";
            if (!empty($subscriptionCreateResponse->error)) {
                $error = json_encode($subscriptionCreateResponse->error);
            } else if (!empty($subscriptionCreateResponse->errors)) {
                $error = json_encode($subscriptionCreateResponse->errors);
            } else {
                $error = "Failed to create the subscription under the addressID:".$addressID;
            }
            $message = "Failed to create the subscription in recharge";
            Log::channel('subscriptionControllerLog')->info("otpToSubscription: Error => ".$error);
            $this->requestResponse("Failed", $message, $error, $data->input('callback'));
        }

        // Create iterable event convertToSubscription
        $eventRequest = array(
            "email" => strtolower($customerEmail), 
            "eventName" => "convertToSubscription",
            "dataFields" => array(
                "userName" => $username
            )
        );

        $eventResponse = $this->iterableObj->createIterableEvent(json_encode($eventRequest));

        if (isset($eventResponse->code) || trim(strtolower($eventResponse->code)) == 'success') {
            Log::channel('subscriptionControllerLog')->info("otpToSubscription: convertToSubscription iterable event - 
            Successfully created event for Email (".strtolower($customerEmail).")");
        } else {
            Log::channel('subscriptionControllerLog')->info("otpToSubscription: convertToSubscription iterable event - 
            Failed to create event for Email (".strtolower($customerEmail).")");
        }
        // Applying the discount to the created subscription charge
        $subscriptionID = $subscriptionCreateResponse->subscription->id;
        $subscriptionChargeDetails = $this->rechargeObj->getChargeDetailsBySubscriptionID($subscriptionID);
        if (!empty($subscriptionChargeDetails->error) || !empty($subscriptionChargeDetails->errors) ||
            empty($subscriptionChargeDetails)) {
            $status = "failed";
            $error  = "Unknown error";
            if (!empty($subscriptionChargeDetails->error)) {
                $error = json_encode($subscriptionChargeDetails->error);
            } else if (!empty($subscriptionChargeDetails->errors)) {
                $error = json_encode($subscriptionChargeDetails->errors);
            } else {
                $error = "Failed to get the charge details by using subscriptionID: ".$subscriptionID;
            }
            $message = "Failed to get the subscription charge details from recharge";
            Log::channel('subscriptionControllerLog')->info("otpToSubscription: Error => ".$error);
            $this->requestResponse("Failed", $message, $error, $data->input('callback'));
        }
        $subscriptionChargeStatus = null;
        $subscriptionchargeID = null;
        foreach ($subscriptionChargeDetails->charges as $chargeValue) {
            $subscriptionChargeTags = strtolower(trim($chargeValue->tags));
            if (strpos($subscriptionChargeTags, 'subscription' ) !== false &&
            strtolower(trim($chargeValue->status)) == 'queued' ) {

                $subscriptionchargeID = $chargeValue->id;
                $subscriptionChargeStatus = $chargeValue->status;
            } 
        }
        if (empty($subscriptionChargeStatus)) {
            $message = "The subscription queued charge not found for apply the discount subscriptionID: "
            .$subscriptionID;
            $error = "Queued charge not found";
            $status = "Failed";
            Log::channel('subscriptionControllerLog')->info("otpToSubscription: Queued charge not found, Error => ".$message);
            $this->requestResponse($status, $message, $error, $data->input('callback'));
        }
        if (!empty($receivedData->discountCode)) {
            $discountCode = trim($receivedData->discountCode);
            $discountCodeDetails = $this->rechargeObj->getDiscountDetailsByDiscountCode($discountCode);
            if (!empty($discountCodeDetails->error) || !empty($discountCodeDetails->errors) ||
                empty($discountCodeDetails)) {
                $status = "failed";
                $error  = "Unknown error";
                if (!empty($discountCodeDetails->error)) {
                    $error = json_encode($discountCodeDetails->error);
                } else if (!empty($discountCodeDetails->errors)) {
                    $error = json_encode($discountCodeDetails->errors);
                } else {
                    $error = "Failed to get the discount code details from recharge";
                }
                $message = "Failed to get the discount details from recharge by using the provided discountCode: "
                .$discountCode;
                Log::channel('subscriptionControllerLog')->info("otpToSubscription: Error => ".$error);
                $this->requestResponse("Failed", $message, $error, $data->input('callback'));
            }
            $discountRequest = array(
                "discount_code" => $discountCode
            );

            $discountApplyResponse = $this->rechargeObj->applyDiscountToChargeByDiscountCode($subscriptionchargeID, 
            json_encode($discountRequest));

            if (!empty($discountApplyResponse->error) || !empty($discountApplyResponse->errors) ||
                empty($discountApplyResponse)) {
                $status = "failed";
                $error  = "Unknown error";
                if (!empty($discountApplyResponse->error)) {
                    $error = json_encode($discountApplyResponse->error);
                } else if (!empty($discountApplyResponse->errors)) {
                    $error = json_encode($discountApplyResponse->errors);
                } else {
                    $error = "Failed to apply the discount to the charge";
                }
                $message = "Failed to apply the discount to the charge chargeID: ".$subscriptionchargeID;
                Log::channel('subscriptionControllerLog')->info("otpToSubscription: ChargeId => ".$subscriptionchargeID.", 
                Error => ".$error);
                $this->requestResponse("Failed", $message, $error, $data->input('callback'));
            }
            $message = "Successfully created the subscription and applied the discount";
            $error = "";
            $status = "Success";
            $this->requestResponse($status, $message, $error, $data->input('callback'));
        }
        $message = "Successfully created the subscription";
        $error = "";
        $status = "Success";
        $this->requestResponse($status, $message, $error, $data->input('callback'));
    }
    
    /**
     * Reactivate cancelled recharge subscription with coupon
    */
    public function reactivateSubscriptionWithCoupon(Request $request) {

        $requestData = !empty($request->input('data')) ? trim($request->input('data')) : "";
        if (empty($requestData)) {
            $message = "Endpoint parameters are missing";
            $error = "Bad request";
            $status = "Failed";
            $response = array(
                "status"       => $status,
                "message" => $message,
                "error" => $error
            );
            $response = !empty($request->input('callback')) ? $request->input('callback').'('.json_encode($response).')' : 
            json_encode($response);
            Log::channel('subscriptionControllerLog')->info("reactivateSubscriptionWithCoupon: Endpoint parameters are missing");
            echo $response;
            exit();
        }
        Log::channel('subscriptionControllerLog')->info("reactivateSubscriptionWithCoupon: Request => ".$requestData);
        $receivedData = json_decode($requestData);
        $subscriptionId = !empty($receivedData->subscriptionId) ? trim($receivedData->subscriptionId) : "";
        $customerHash = !empty($receivedData->customerHash) ? trim($receivedData->customerHash) : "";
        if (empty($subscriptionId)) {
            $message = "Failed to reactivate the subscription";
            $error = "Missing required parameter subscription_id";
            $status = "Failed";
            Log::channel('subscriptionControllerLog')->info("reactivateSubscriptionWithCoupon: Missing required parameter 
            subscription_id");
            $this->requestResponse($status, $message, $error, $request->input('callback'));
        }
        if (empty($customerHash)) {
            $message = "Failed to reactivate the subscription";
            $error = "Missing required parameter recharge customer hash";
            $status = "Failed";
            Log::channel('subscriptionControllerLog')->info("reactivateSubscriptionWithCoupon: Missing required parameter 
            recharge customer hash");
            $this->requestResponse($status, $message, $error, $request->input('callback'));
        }

        $subscriptionDetails = $this->rechargeObj->getSubscription($subscriptionId);

        if (!isset($subscriptionDetails) || isset($subscriptionDetails->error)|| isset($subscriptionDetails->errors) || 
            empty($subscriptionDetails->subscription->customer_id)) {
            $status = "Failed";
            $error  = "Unknown error";
            if (isset($subscriptionDetails->error)) {
                $error = json_encode($subscriptionDetails->error);
            } else if (isset($subscriptionDetails->errors)) {
                $error = json_encode($subscriptionDetails->errors);
            } else {
                $error = "Failed to get the subscription details by Id";
            }
            $message = "Failed to get the subscription details";
            Log::channel('subscriptionControllerLog')->info("reactivateSubscriptionWithCoupon: Failed to get the subscription details, 
            Error => ".$error);
            $this->requestResponse($status, $message, $error, $request->input('callback'));
        }

        $customerDetails = $this->rechargeObj->getRechargeCustomerDetailsByID($subscriptionDetails->subscription->customer_id);

        if (!isset($customerDetails) || isset($customerDetails->error) || isset($customerDetails->errors) || 
            empty($customerDetails->customer->hash)) {
            $status = "Failed";
            $error  = "Unknown error";
            if (isset($customerDetails->error)) {
                $error = json_encode($customerDetails->error);
            } else if (isset($customerDetails->errors)) {
                $error = json_encode($customerDetails->errors);
            } else {
                $error = "Failed to get the customer details";
            }
            $message = "Failed to get the customer details from recharge";
            Log::channel('subscriptionControllerLog')->info("reactivateSubscriptionWithCoupon: Failed to get the customer details 
            from recharge, Error => ".$error);
            $this->requestResponse($status, $message, $error, $request->input('callback'));
        }
        $rechargeCustomerHash = trim($customerDetails->customer->hash);
        if ($customerHash == $rechargeCustomerHash) {

            $reactivateResponse = $this->rechargeObj->activateSubscription($subscriptionId);

            if (isset($reactivateResponse->error) || isset($reactivateResponse->errors) || empty($reactivateResponse)) {
                $status = "Failed";
                $error  = "Unknown error";
                if (isset($reactivateResponse->error)) {
                    $error = json_encode($reactivateResponse->error);
                } else if (isset($reactivateResponse->errors)) {
                    $error = json_encode($reactivateResponse->errors);
                } else {
                    $error = "Failed to reactivate the subscription";
                }
                $message = "Failed to reactivate the subscription";
                Log::channel('subscriptionControllerLog')->info("reactivateSubscriptionWithCoupon: Failed to reactivate the subscription, 
                Error => ".$error);
                $this->requestResponse($status, $message, $error, $request->input('callback'));
            } else {
                /* Iterable event for reactivating the subscription*/
                $subscriptionProperties = '';

                if (isset($subscriptionDetails->subscription) && !empty($subscriptionDetails->subscription)) {
                    $subscriptionProperties = !empty($subscriptionDetails->subscription->properties) ? 
                    $subscriptionDetails->subscription->properties : "";
                }
                if (!empty($customerDetails->customer->email)) {
                    $lineItemDetails = [];
                    if (!empty($subscriptionProperties)) {
                        foreach ($subscriptionProperties as $item) {
                            if (isset($item->name) && strpos($item->name, 'Vitamin') !== false) {
                                $explodeValue = explode('|', $item->value);
                                $lineItem = new \stdClass();
                                if (!empty($explodeValue)) {
                                    $lineItem->name = $item->name;
                                    $lineItem->value = $explodeValue[0];
                                    $lineItem->variant_id = $explodeValue[3];
                                    $lineItem->quantity = $explodeValue[2];
                                    $lineItem->sku = $explodeValue[4];
                                    $lineItemDetails[] = $lineItem;
                                }
                            }
                        }
                    }
                    $customerEmail = $customerDetails->customer->email;
                    $eventRequest = array(
                        "email" => strtolower($customerEmail), 
                        "eventName" => "subscriptionReactivated",
                        "dataFields" => array(
                            "lineItems" => $lineItemDetails
                        )
                    );

                    $eventResponse = $this->iterableObj->createIterableEvent(json_encode($eventRequest));

                    if (isset($eventResponse->code) || trim(strtolower($eventResponse->code)) == 'success') {
                        Log::channel('subscriptionControllerLog')->info("reactivateSubscriptionWithCoupon: 
                        Subscription reactivation event - Successfully created for Email (".
                            strtolower($customerEmail).")".
                            " | customerHash -> ".$customerHash.
                            " | Subscription -> ".$subscriptionId
                        );
                    } else {
                        Log::channel('subscriptionControllerLog')->info("reactivateSubscriptionWithCoupon: 
                        Subscription reactivation event - Failed to create event for Email (".
                            strtolower($customerEmail).")".
                            " | customerHash -> ".$customerHash.
                            " | Subscription -> ".$subscriptionId
                        );
                    }
                }

                /* apply coupon for the next charge */
                if (isset($receivedData->couponCode) && !empty($receivedData->couponCode)) {

                    $subscriptionChargeDetails = $this->rechargeObj->getChargeDetailsBySubscriptionID($subscriptionId);

                    if (!empty($subscriptionChargeDetails->error) || !empty($subscriptionChargeDetails->errors) ||
                        empty($subscriptionChargeDetails)) {
                        $status = "Failed";
                        $error  = "Unknown error";
                        if (!empty($subscriptionChargeDetails->error)) {
                            $error = json_encode($subscriptionChargeDetails->error);
                        } else if (!empty($subscriptionChargeDetails->errors)) {
                            $error = json_encode($subscriptionChargeDetails->errors);
                        } else {
                            $error = "Failed to get the charge details by using subscriptionId: ".$subscriptionId;
                        }
                        $message = "Failed to get the subscription charge details from recharge";
                        Log::channel('subscriptionControllerLog')->info("reactivateSubscriptionWithCoupon: 
                        Failed to get the charge details by using subscriptionId:".
                            " | customerHash -> ".$customerHash.
                            " | Subscription -> ".$subscriptionId.
                            " | Error -> ".$error
                        );
                        $this->requestResponse($status, $message, $error, $request->input('callback'));
                    }
                    $subscriptionChargeStatus = null;
                    $subscriptionchargeID = null;
                    foreach ($subscriptionChargeDetails->charges as $chargeValue) {
                        $subscriptionChargeTags = strtolower(trim($chargeValue->tags));
                        if (strpos($subscriptionChargeTags, 'subscription') !== false &&
                        strtolower(trim($chargeValue->status)) == 'queued') {
                            $subscriptionchargeID = $chargeValue->id;
                            $subscriptionChargeStatus = $chargeValue->status;
                        }
                    }
                    if (empty($subscriptionChargeStatus)) {
                        $message = "The subscription queued charge was not found to apply the discount, subscriptionID: "
                        .$subscriptionId;
                        $error = "Queued charge not found";
                        $status = "Failed";
                        Log::channel('subscriptionControllerLog')->info("reactivateSubscriptionWithCoupon: 
                        Queued charge not found:".
                            " | customerHash -> ".$customerHash.
                            " | Subscription -> ".$subscriptionId
                        );
                        $this->requestResponse($status, $message, $error, $request->input('callback'));
                    }
                    $discountCode = trim($receivedData->couponCode);

                    $discountCodeDetails = $this->rechargeObj->getDiscountDetailsByDiscountCode($discountCode);

                    if (!empty($discountCodeDetails->error) || !empty($discountCodeDetails->errors) ||
                        empty($discountCodeDetails)) {
                        $status = "Failed";
                        $error  = "Unknown error";
                        if (!empty($discountCodeDetails->error)) {
                            $error = json_encode($discountCodeDetails->error);
                        } else if (!empty($discountCodeDetails->errors)) {
                            $error = json_encode($discountCodeDetails->errors);
                        } else {
                            $error = "Failed to get the discount code details from recharge";
                        }

                        $message = "Failed to get the discount details from recharge by using the provided discountCode: "
                        .$discountCode;

                        Log::channel('subscriptionControllerLog')->info("reactivateSubscriptionWithCoupon: 
                        Failed to get the discount details from recharge by using the provided discountCode:".
                            " | customerHash -> ".$customerHash.
                            " | Subscription -> ".$subscriptionId.
                            " | DiscountCode -> ".$discountCode.
                            " Error -> ".$error
                        );
                        $this->requestResponse($status, $message, $error, $request->input('callback'));
                    }
                    $discountRequest = array(
                        "discount_code" => $discountCode
                    );

                    $discountApplyResponse = $this->rechargeObj->applyDiscountToChargeByDiscountCode($subscriptionchargeID,
                    json_encode($discountRequest));

                    if (!empty($discountApplyResponse->error) || !empty($discountApplyResponse->errors) ||
                        empty($discountApplyResponse)) {
                        $status = "Failed";
                        $error  = "Unknown error";
                        if (!empty($discountApplyResponse->error)) {
                            $error = json_encode($discountApplyResponse->error);
                        } else if (!empty($discountApplyResponse->errors)) {
                            $error = json_encode($discountApplyResponse->errors);
                        } else {
                            $error = "Failed to apply the discount to the charge";
                        }
                        $message = "Failed to apply the discount to the charge chargeID: ".$subscriptionchargeID;
                        Log::channel('subscriptionControllerLog')->info("reactivateSubscriptionWithCoupon: 
                        Failed to apply the discount to the charge: ".
                            " | Subscription -> ".$subscriptionId.
                            " | customerHash -> ".$customerHash.
                            " | chargeID: ".$subscriptionchargeID.
                            " | DiscountCode: ".$discountCode.
                            " | Error -> ".$error
                        );
                        $this->requestResponse($status, $message, $error, $request->input('callback'));
                    }
                    $message = "Successfully activated the subscription and applied the discount";
                    $error = "";
                    $status = "Success";
                    Log::channel('subscriptionControllerLog')->info("reactivateSubscriptionWithCoupon: 
                    Successfully activated the subscription and applied the discount.".
                    " | Subscription -> ".$subscriptionId.
                    " | customerHash -> ".$customerHash.
                    " | chargeID: ".$subscriptionchargeID.
                    " | DiscountCode: ".$discountCode
                    );
                    $this->requestResponse($status, $message, $error, $request->input('callback'));
                }

                /* apply coupon for next charge */
                Log::channel('subscriptionControllerLog')->info("reactivateSubscriptionWithCoupon: 
                Subscription reactivated successfully.".
                " | customerHash -> ".$customerHash.
                " | Subscription -> ".$subscriptionId
                );
                $message = "Subscription reactivated successfully";
                $error = "";
                $status = "Success";
                $this->requestResponse($status, $message, $error, $request->input('callback'));
            }
        } else {
            $message = "Failed to reactivate the subscription";
            $error = "Customer hash tags are not matched";
            $status = "Failed";

            Log::channel('subscriptionControllerLog')->info("reactivateSubscriptionWithCoupon: 
            Failed to reactivate the subscription. Customer hashtags are not matched:".
                " | customerHash -> ".$customerHash.
                " | Subscription -> ".$subscriptionId
            );
            $this->requestResponse($status, $message, $error, $request->input('callback'));
        }
    }
    
    /**
     * Adjust subscription order date
    */
    public function adjustNextOrderDate(Request $request) {

        $requestData = !empty($request->input('data')) ? trim($request->input('data')) : "";
        if (empty($requestData)) {
            $message = "Endpoint parameters are missing";
            $error = "Bad request";
            $status = "Failed";
            $response = array(
                "status"       => $status,
                "message" => $message,
                "error" => $error
            );
            $response = !empty($request->input('callback')) ? $request->input('callback').'('.json_encode($response).')' : 
            json_encode($response);
            Log::channel('subscriptionControllerLog')->info("adjustNextOrderDate: Endpoint parameters are missing");
            echo $response;
            exit();
        }
        Log::channel('subscriptionControllerLog')->info("adjustNextOrderDate: Request => ".$requestData);
        $receivedData = json_decode($requestData);
        $subscriptionId = !empty($receivedData->subscriptionId) ? trim($receivedData->subscriptionId) : "";
        $customerHash = !empty($receivedData->customerHash) ? trim($receivedData->customerHash) : "";
        if (empty($subscriptionId)) {
            $message = "Failed to adjust the order date.";
            $error = "Missing required parameter subscription id";
            $status = "Failed";
            Log::channel('subscriptionControllerLog')->info("adjustNextOrderDate: Missing required parameter subscription id");
            $this->requestResponse($status, $message, $error, $request->input('callback'));
        }
        if (empty($customerHash)) {
            $message = "Failed to adjust the order date.";
            $error = "Missing required parameter recharge customer hash";
            $status = "Failed";
            Log::channel('subscriptionControllerLog')->info("adjustNextOrderDate: Missing required parameter recharge customer hash");
            $this->requestResponse($status, $message, $error, $request->input('callback'));
        }

        $subscriptionDetails = $this->rechargeObj->getSubscription($subscriptionId);

        if (!isset($subscriptionDetails) || isset($subscriptionDetails->error) || isset($subscriptionDetails->errors) || 
            empty($subscriptionDetails->subscription->customer_id)) {
            $status = "Failed";
            $error  = "Unknown error";
            if (isset($subscriptionDetails->error)) {
                $error = json_encode($subscriptionDetails->error);
            } else if (isset($subscriptionDetails->errors)) {
                $error = json_encode($subscriptionDetails->errors);
            } else {
                $error = "Failed to get the subscription details";
            }
            $message = "Failed to adjust the order date. Invalid subscription ID";

            Log::channel('subscriptionControllerLog')->info("adjustNextOrderDate: 
            Failed to adjust the order date: Invalid subscription ID: ".
                " | customerHash -> ".$customerHash.
                " | Subscription -> ".$subscriptionId.
                " | Error -> ".$error
            );
            $this->requestResponse($status, $message, $error, $request->input('callback'));
        }

        $customerDetails = $this->rechargeObj->getRechargeCustomerDetailsByID($subscriptionDetails->subscription->customer_id);

        if (!isset($customerDetails) || isset($customerDetails->error) || isset($customerDetails->errors) || 
            empty($customerDetails->customer->hash)) {
            $status = "Failed";
            $error  = "Unknown error";
            if (isset($customerDetails->error)) {
                $error = json_encode($customerDetails->error);
            } else if (isset($customerDetails->errors)) {
                $error = json_encode($customerDetails->errors);
            } else {
                $error = "Failed to get the customer details by Id";
            }
            $message = "Failed to get the customer details from recharge";

            Log::channel('subscriptionControllerLog')->info("adjustNextOrderDate: 
            Failed to get the customer details from recharge: ".
                " | customerHash -> ".$customerHash.
                " | Subscription -> ".$subscriptionId.
                " | Error -> ".$error
            );
            $this->requestResponse($status, $message, $error, $request->input('callback'));
        }
        $rechargeCustomerHash = trim($customerDetails->customer->hash);
        if ($customerHash == $rechargeCustomerHash) {
            /*Change next charge date by subscription ID */
            if (empty($subscriptionDetails->subscription->next_charge_scheduled_at)) {
                $message = "The next charge date is not found in the subscription - ".$subscriptionId;
                $error = "Next charge date not found";
                $status = "Failed";
                Log::channel('subscriptionControllerLog')->info("adjustNextOrderDate: 
                Next charge date not found: ".
                    " | customerHash -> ".$customerHash.
                    " | Subscription -> ".$subscriptionId.
                    " | Error -> ".$error
                );
                $this->requestResponse($status, $message, $error, $request->input('callback'));
            }
            $adjustDays = 1;
            $subscriptionCurrentChargeDate = $subscriptionDetails->subscription->next_charge_scheduled_at;
            $subscriptionCurrentChargeDate = date('Y-m-d', strtotime($subscriptionCurrentChargeDate));
            $adjustDays = (int)$adjustDays;
            $currentDate = date('Y-m-d');
            if ($adjustDays > 0) {
                $newChargeDate = date('Y-m-d',strtotime('+'.$adjustDays.' days',strtotime($currentDate)));
            }
            $payload = array(
                "date" => $newChargeDate
            );
            
            $subscriptionUpdateResponse = $this->rechargeObj->updateSubscriptionChargeDate($subscriptionId, json_encode($payload));

            if (isset($subscriptionUpdateResponse->error) || isset($subscriptionUpdateResponse->errors) || 
                empty($subscriptionUpdateResponse)) {
                $status = "Failed";
                $error  = "Unknown error";
                if (isset($subscriptionUpdateResponse->error)) {
                    $error = json_encode($subscriptionUpdateResponse->error);
                } else if (isset($subscriptionUpdateResponse->errors)) {
                    $error = json_encode($subscriptionUpdateResponse->errors);
                } else {
                    $error = "Failed to update the subscription";
                }
                $message = "Failed to change the next charge order date";
                $status = "Failed";
                Log::channel('subscriptionControllerLog')->info("adjustNextOrderDate: 
                Failed to change the next charge order date: ".
                    " | customerHash -> ".$customerHash.
                    " | Subscription -> ".$subscriptionId.
                    " | Error -> ".$error
                );
                $this->requestResponse($status, $message, $error, $request->input('callback'));
            }
            /*Change next charge date by subscription ID */
            else {
                /* apply coupon for the next charge */
                if (isset($receivedData->couponCode) && !empty($receivedData->couponCode)) {

                    $subscriptionChargeDetails = $this->rechargeObj->getChargeDetailsBySubscriptionID($subscriptionId);

                    if (!empty($subscriptionChargeDetails->error) || !empty($subscriptionChargeDetails->errors) ||
                        empty($subscriptionChargeDetails)) {
                        $status = "Failed";
                        $error  = "Unknown error";
                        if (!empty($subscriptionChargeDetails->error)) {
                            $error = json_encode($subscriptionChargeDetails->error);
                        } else if (!empty($subscriptionChargeDetails->errors)) {
                            $error = json_encode($subscriptionChargeDetails->errors);
                        } else {
                            $error = "Failed to get the charge details by using subscriptionId: ".$subscriptionId;
                        }
                        $message = "Failed to get the subscription charge details from recharge";
                        Log::channel('subscriptionControllerLog')->info("adjustNextOrderDate: 
                        Failed to get the charge details by using subscriptionId:".
                            " | customerHash -> ".$customerHash.
                            " | Subscription -> ".$subscriptionId.
                            " | Error -> ".$error
                        );
                        $this->requestResponse($status, $message, $error, $request->input('callback'));
                    }
                    $subscriptionChargeStatus = null;
                    $subscriptionchargeID = null;
                    foreach ($subscriptionChargeDetails->charges as $chargeValue) {
                        $subscriptionChargeTags = strtolower(trim($chargeValue->tags));
                        if (strpos($subscriptionChargeTags, 'subscription') !== false &&
                        strtolower(trim($chargeValue->status)) == 'queued') {
                            $subscriptionchargeID = $chargeValue->id;
                            $subscriptionChargeStatus = $chargeValue->status;
                        }
                    }
                    if (empty($subscriptionChargeStatus)) {
                        $message = "The upcoming charge not found for applying the discount, subscriptionID: "
                        .$subscriptionId;
                        $error = "Queued charge not found";
                        $status = "Failed";
                        Log::channel('subscriptionControllerLog')->info("adjustNextOrderDate: 
                        Queued charge not found:".
                            " | customerHash -> ".$customerHash.
                            " | Subscription -> ".$subscriptionId
                        );
                        $this->requestResponse($status, $message, $error, $request->input('callback'));
                    }
                    $discountCode = trim($receivedData->couponCode);

                    $discountCodeDetails = $this->rechargeObj->getDiscountDetailsByDiscountCode($discountCode);

                    if (!empty($discountCodeDetails->error) || !empty($discountCodeDetails->errors) ||
                        empty($discountCodeDetails)) {
                        $status = "Failed";
                        $error  = "Unknown error";
                        if (!empty($discountCodeDetails->error)) {
                            $error = json_encode($discountCodeDetails->error);
                        } else if (!empty($discountCodeDetails->errors)) {
                            $error = json_encode($discountCodeDetails->errors);
                        } else {
                            $error = "Failed to get the discount code details from the recharge";
                        }
                        $message = "Failed to get the discount details from recharge by using the provided discountCode: "
                        .$discountCode;

                        Log::channel('subscriptionControllerLog')->info("adjustNextOrderDate: 
                        Failed to get the discount details from recharge by using the provided discountCode:".
                            " | customerHash -> ".$customerHash.
                            " | Subscription -> ".$subscriptionId.
                            " | DiscountCode -> ".$discountCode.
                            " Error -> ".$error
                        );
                        $this->requestResponse($status, $message, $error, $request->input('callback'));
                    }
                    $discountRequest = array(
                        "discount_code" => $discountCode
                    );

                    $discountApplyResponse = $this->rechargeObj->applyDiscountToChargeByDiscountCode($subscriptionchargeID,
                    json_encode($discountRequest));

                    if (!empty($discountApplyResponse->error) || !empty($discountApplyResponse->errors) ||
                        empty($discountApplyResponse)) {
                        $status = "Failed";
                        $error  = "Unknown error";
                        if (!empty($discountApplyResponse->error)) {
                            $error = json_encode($discountApplyResponse->error);
                        } else if (!empty($discountApplyResponse->errors)) {
                            $error = json_encode($discountApplyResponse->errors);
                        } else {
                            $error = "Failed to apply the discount to the charge";
                        }
                        $message = "Failed to apply the discount to the charge, chargeID: ".$subscriptionchargeID;
                        Log::channel('subscriptionControllerLog')->info("adjustNextOrderDate: 
                        Failed to apply the discount to the charge: ".
                            " | Subscription -> ".$subscriptionId.
                            " | customerHash -> ".$customerHash.
                            " | chargeID: ".$subscriptionchargeID.
                            " | DiscountCode: ".$discountCode.
                            " | Error -> ".$error
                        );
                        $this->requestResponse($status, $message, $error, $request->input('callback'));
                    }
                    $message = "Successfully changed the charge date of the subscription and applied the discount.";
                    $error = "";
                    $status = "Success";
                    Log::channel('subscriptionControllerLog')->info("adjustNextOrderDate: 
                    Successfully changes the charge date of subscription and applies the discount.".
                        " | Subscription -> ".$subscriptionId.
                        " | customerHash -> ".$customerHash.
                        " | chargeID: ".$subscriptionchargeID.
                        " | DiscountCode: ".$discountCode
                    );
                    $this->requestResponse($status, $message, $error, $request->input('callback'));
                }
                /* apply coupon for the next charge */
                Log::channel('subscriptionControllerLog')->info("adjustNextOrderDate: 
                The subscription date changed successfully.".
                    " | customerHash -> ".$customerHash.
                    " | Subscription -> ".$subscriptionId
                );
                $message = "The subscription date changed successfully.";
                $error = "";
                $status = "Success";
                $response = array(
                        "status"  => $status,
                        "message" => $message,
                        "error"   => $error
                );
                $this->requestResponse($status, $message, $error, $request->input('callback'));
            }
        } else {
            $message = "Failed to change the order date.";
            $error = "Customer hashtags are not matched";
            $status = "Failed";
            Log::channel('subscriptionControllerLog')->info("adjustNextOrderDate: 
            Failed to change the order date of the subscription. Customer hashtags are not matched:".
                " | customerHash -> ".$customerHash.
                " | Subscription -> ".$subscriptionId
            );
            $this->requestResponse($status, $message, $error, $request->input('callback'));
        }
    }

    /**
     * Endpoint to change subscription next charge date
     */
    public function setNextChargeDate(Request $request) {

        $requestData = !empty($request->input('data')) ? $request->input('data') : "";
        if (empty($requestData)) {
            $returnData = [
                'error' => 'Invalid endpoint request',
            ];       
            Log::channel('nextChargeDateChangeLog')->info("Invalid endpoint request");     
            return $this->returnData(600, "failed", $returnData);
        }
        Log::channel('nextChargeDateChangeLog')->info("Request => ".$requestData);
        $receivedData = json_decode($requestData);
        if (empty($receivedData->hash) && empty($receivedData->subscriptionId) && empty($receivedData->nextChargeDate)) {
            $returnData = [
                'error' => 'Invalid endpoint request',
            ];        
            Log::channel('nextChargeDateChangeLog')->info("Invalid endpoint request");    
            return $this->returnData(600, "failed", $returnData);
        }
        $rechargeToken = !empty($receivedData->hash) ? trim($receivedData->hash) : '';
        $subscriptionId = !empty($receivedData->subscriptionId) ? trim($receivedData->subscriptionId) : '';
        $nextChargeDate = !empty($receivedData->nextChargeDate) ? trim($receivedData->nextChargeDate) : '';
        if (empty($rechargeToken)) {
            $returnData = [
                'error' => 'Required parameter recharge hash is missing in the endpoint request',
            ];
            Log::channel('nextChargeDateChangeLog')->info("Required parameter recharge hash is missing 
            in the endpoint request, request: ".json_encode($receivedData));
            return $this->returnData(600, "failed", $returnData);
        }
        if (empty($subscriptionId)) {
            $returnData = [
                'error' => 'Required parameter subscriptionId is missing in the endpoint request',
            ];
            Log::channel('nextChargeDateChangeLog')->info("Required parameter subscriptionId is missing 
            in the endpoint request, request: ".json_encode($receivedData));
            return $this->returnData(600, "failed", $returnData);
        }
        if (empty($nextChargeDate)) {
            $returnData = [
                'error' => 'Required parameter next charge date is missing in the endpoint request',
            ];
            Log::channel('nextChargeDateChangeLog')->info("Required parameter next charge date is missing in 
            the endpoint request, request: ".json_encode($receivedData));
            return $this->returnData(600, "failed", $returnData);
        }
        
        $queuedChargeDetails = $this->rechargeObj->getQueuedChargeBySubscriptionID($subscriptionId);

        if (!empty($queuedChargeDetails->error) || !empty($queuedChargeDetails->errors) ||
            empty($queuedChargeDetails) || empty($queuedChargeDetails->charges)) {
            $rechargeError  = "Unknown error";
            if (!empty($queuedChargeDetails->error)) {
                $rechargeError = json_encode($queuedChargeDetails->error);
            } else if (!empty($queuedChargeDetails->errors)) {
                $rechargeError = json_encode($queuedChargeDetails->errors);
            } else if (empty($queuedChargeDetails) || empty($queuedChargeDetails->charges)) {
                $rechargeError = 'Not found any queued charge for the subscription';
            }
            $returnData = [
                'error' => $rechargeError
            ];
            Log::channel('nextChargeDateChangeLog')->info("Failed to get the queued charge details from recharge, 
            subscriptionId: ".$subscriptionId.", Error:".$rechargeError);
            return $this->returnData(600, "failed", $returnData);
        }
        $currentNextChargeDate = !empty($queuedChargeDetails->charges[0]->scheduled_at) ? 
        trim($queuedChargeDetails->charges[0]->scheduled_at) : '';
        $rechargeHashToken = !empty($queuedChargeDetails->charges[0]->customer_hash) ?
        trim($queuedChargeDetails->charges[0]->customer_hash) : '';
        $emailAddress = !empty($queuedChargeDetails->charges[0]->email) ?
        trim($queuedChargeDetails->charges[0]->email) : '';
        $customerFirstName = !empty($queuedChargeDetails->charges[0]->first_name) ?
        trim($queuedChargeDetails->charges[0]->first_name) : '';
        $lipProperties = !empty($queuedChargeDetails->charges[0]->line_items[0]->properties) ?
        $queuedChargeDetails->charges[0]->line_items[0]->properties : '';
        if ($rechargeHashToken != $rechargeToken) {
            $returnData = [
                'error' => 'Failed the security authentication'
            ];
            Log::channel('nextChargeDateChangeLog')->info("Failed the security authentication
            (The recharge hash tokens are mismatched), request: ".json_encode($receivedData));
            return $this->returnData(600, "failed", $returnData);
        }
        $today = date('Y-m-d');
        $currentDate = date('Y-m-d',strtotime($today));
        $newNextChargeDate = date('Y-m-d',strtotime($nextChargeDate));
        if ($newNextChargeDate <= $currentDate) {
            $returnData = [
                'error' => "The new charge date in the request is less than or equal to today's date"
            ];
            Log::channel('nextChargeDateChangeLog')->info("The new charge date in the request is less 
            than or equal to the current charge date, request: ".json_encode($receivedData));
            return $this->returnData(600, "failed", $returnData);
        }
        $payload = array(
            "date" => $newNextChargeDate
        );

        $subscriptionUpdateResponse = $this->rechargeObj->updateSubscriptionChargeDate($subscriptionId, json_encode($payload));

        if (isset($subscriptionUpdateResponse->error) || isset($subscriptionUpdateResponse->errors) || 
            empty($subscriptionUpdateResponse)) {
            $subscriptionUpdateError  = "Unknown error";
            if (!empty($subscriptionUpdateResponse->error)) {
                $subscriptionUpdateError = json_encode($subscriptionUpdateResponse->error);
            } else if (!empty($subscriptionUpdateResponse->errors)) {
                $subscriptionUpdateError = json_encode($subscriptionUpdateResponse->errors);
            } else if (empty($subscriptionUpdateResponse)) {
                $subscriptionUpdateError = 'Failed to update the subscription next charge date';
            }
            $returnData = [
                'error' => $subscriptionUpdateError
            ];
            Log::channel('nextChargeDateChangeLog')->info("Failed to update the subscription next charge date, 
            subscriptionId: ".$subscriptionId.", Error:".$subscriptionUpdateError);
            return $this->returnData(600, "failed", $returnData);
        }
        // Create iterable event
        $iterableEventCreateRequest = [];
        $subscriptionVitamins = $this->createIterableVitaminDataField($lipProperties);
        $iterableNextChargeDate = date('F d, Y', strtotime($newNextChargeDate));
        $iterableEventCreateRequest = array(
            "email" => strtolower($emailAddress), 
            "eventName" => "nextChargeDate Updated",
            "dataFields" => array(
                "email" => $emailAddress,
                "first_name" => $customerFirstName,
                "vitamins" => $subscriptionVitamins,
                "pack_shipping_date" => $iterableNextChargeDate
            )
        );

        $eventResponse = $this->iterableObj->createIterableEvent(json_encode($iterableEventCreateRequest));

        if (isset($eventResponse->code) || trim(strtolower($eventResponse->code)) == 'success') {
            Log::channel('nextChargeDateChangeLog')->info("Iterable event(nextChargeDate Updated): - Successfully 
            created the event for Email(".strtolower($emailAddress).")");
        } else {
            Log::channel('nextChargeDateChangeLog')->info("Iterable event(nextChargeDate Updated): - Failed to create 
            the event for Email (".strtolower($emailAddress).")");
        }
        $returnData = [
            'message' => 'Successfully updated the subscription next charge date'
        ];
        Log::channel('nextChargeDateChangeLog')->info("Successfully updated the subscription next charge date, 
        subscriptionId: ".$subscriptionId);
        return $this->returnData(200, "success", $returnData);
    }

    /**
     * Create iterable vitamin data field
     */
    public function createIterableVitaminDataField($lipProperties) {

        $subscriptionVitamins = [];
        if (!empty($lipProperties)) {
            foreach ($lipProperties as $vitaminKey => $vitaminValue) {
                $vitaminTempArray = [];
                $propertyName = !empty($vitaminValue->name) ? strtolower(trim($vitaminValue->name)) : '';
                if (strpos($propertyName, 'vitamin') !== false) {
                    $propertyValue = !empty($vitaminValue->value) ? trim($vitaminValue->value) : '';
                    if (strpos($propertyValue, '|') !== false) {
                        $explodePropertyValue = explode('|', $propertyValue);
                        if (count($explodePropertyValue) == 5) {
                            $vitaminTempArray = [
                                "vitamin_name" => trim($explodePropertyValue[0]),
                                "quantity" => trim($explodePropertyValue[2]),
                                "variant" => trim($explodePropertyValue[1]),
                            ];
                            $subscriptionVitamins[] = $vitaminTempArray;
                        }
                    }
                }
            }
        }
        return $subscriptionVitamins;
    }

    /**
     * Response function
     */
    public function returnData($status, $message, $error, $jsonCallback = null) {

        $returnData = array(
            'status' => $status,
            'message' => $message,
            'body' => $error
        );
        if (!empty($jsonCallback)) {
            echo $jsonCallback . '(' . json_encode($returnData) . ')';
            exit();
        } else {
            echo json_encode($returnData);
            exit();
        }
    }

}
