<?php

namespace App\Jobs;

use Illuminate\Bus\Queueable;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Contracts\Queue\ShouldQueue;
use App\Models\RechargeApi;
use App\Models\IterableApi;
use Log;

class IterableSubscriptionCancelEvent implements ShouldQueue
{
    use InteractsWithQueue, Queueable;

    private $iterableObj;
    private $rechargeObj;
    private $customerID;

    /**
     * Create a new job instance.
     *
     * @return void
     */
    public function __construct($customerID) {

        $this->customerID = $customerID;
    }
    
    public function handle() {
        
        $this->iterableObj = new IterableApi();
        $this->rechargeObj = new RechargeApi();
        
        $customerDetails = $this->rechargeObj->getRechargeCustomerDetailsByID($this->customerID);

        if (isset($customerDetails->error) || isset($customerDetails->errors) || empty($customerDetails->customer->email) || 
            empty($customerDetails)) {
            $status = "failed";
            $error = "Unknown error";
            if (isset($customerDetails->error)) {
                $error = json_encode($customerDetails->error);
            } else if (isset($customerDetails->errors)) {
                $error = json_encode($customerDetails->errors);
            } else {
                $error = "Not found any customer details by customerId";
            }
            Log::channel('iterableWebhookLog')->info("IterableSubscriptionCancelEvent: Failed to get the Recharge user details for 
            CustomerID (".$this->customerID."), Error: " . $error);
            exit();
        }
        $eventRequest = array(
            "email" => strtolower($customerDetails->customer->email), 
            "eventName" => "Subscription cancelled",
            "dataFields" => array(
                "userName" => $customerDetails->customer->first_name.' '.$customerDetails->customer->last_name
            )
        );
        $eventResponse = $this->iterableObj->createIterableEvent(json_encode($eventRequest));

        if (isset($eventResponse->code) || trim(strtolower($eventResponse->code)) == 'success') {
            Log::channel('iterableWebhookLog')->info("IterableSubscriptionCancelEvent: Cancelled subscription iterable event - 
            Successfully created event for Email (".strtolower($customerDetails->customer->email).")");
        } else {
            Log::channel('iterableWebhookLog')->info("IterableSubscriptionCancelEvent: Cancelled subscription iterable event - 
            Failed to create event for Email (".strtolower($customerDetails->customer->email).")");
        }
    }
}
