<?php

namespace App\Jobs;

use Illuminate\Bus\Queueable;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Contracts\Queue\ShouldQueue;
use App\Models\IterableApi;
use Log;

class IterableUserChannelUpdateEvent implements ShouldQueue
{
    use InteractsWithQueue, Queueable;

    private $iterableObj;
    private $customerEmail;

    /**
     * Create a new job instance.
     *
     * @return void
     */
    public function __construct($customerEmail) {

        $this->customerEmail = $customerEmail;
    }

    public function handle() {

        $this->iterableObj = new IterableApi();

        $customerDetails = $this->iterableObj->getIterableSubscription($this->customerEmail);

        if (isset($customerDetails->error) || isset($customerDetails->errors) || empty($customerDetails->user->email) ||
            empty($customerDetails)) {
            $status = "failed";
            $error = "Unknown error";
            if (isset($customerDetails->error)) {
                $error = json_encode($customerDetails->error);
            } else if (isset($customerDetails->errors)) {
                $error = json_encode($customerDetails->errors);
            } else {
                $error = "The response is empty or missing the required parameters";
            }
            Log::channel('iterableWebhookLog')->info("IterableUserChannelUpdateEvent: Failed to get the Iterable user details for 
            CustomerEmail (".$this->customerEmail."), Error: " . $error);
            exit();
        }
        $channelIds = env('ITERABLE_DEFAULT_OFF_CHANNEL_IDS',true);
        $channelIdExplode = explode(",",$channelIds);
        $updateRequest = array(
            "email" => $this->customerEmail,
            "unsubscribedChannelIds" => array_map('intval', $channelIdExplode)
        );

        $updateResponse = $this->iterableObj->updateUserIterableSubscription(json_encode($updateRequest));

        if(empty($updateResponse->code) || strtolower(trim($updateResponse->code)) == 'success') {
            Log::channel('iterableWebhookLog')->info("IterableUserChannelUpdateEvent: Update iterable user channel status - 
            Successfully Updated the channel status (".strtolower($customerDetails->user->email).")");
        } else {
            Log::channel('iterableWebhookLog')->info("IterableUserChannelUpdateEvent: Update iterable user channel status - 
            Failed to  Updated the channel status (".strtolower($customerDetails->user->email).")");
        }
    }
}
