<?php
namespace App\Models;
use Illuminate\Database\Eloquent\Model;

class RechargeApi extends Model {

    private $rechargeUrl;
    private $accessToken;
    private $rechargeObj;
    private $rechargeApiKey;

    function __construct() {

        $this->rechargeUrl = env('RECHARGE_URL', true);
        $this->rechargeApiKey = env('RECHARGE_API_KEY',true);
    }

    public function getChargeById($id){

        $method = 'GET';
        $path = $this->rechargeUrl . 'charges/'.$id;
        $response = $this->call($method, $path);
        return json_decode($response);
    }

    /**
     * update subscription by ID
     */
    function updateSubscription($subscription_id, $data = NULL) {

        $method = 'PUT';
        $path = env('RECHARGE_URL', true).'/subscriptions/'.$subscription_id;
        $response = $this->call($method, $path, $data);
        return json_decode($response);
    }

    /**
     * get subscription by subscription_id
     */
    function getSubscription($subscription_id) {

        $method = 'GET';
        $path = $this->rechargeUrl . '/subscriptions/' . $subscription_id;
        $response = $this->call($method, $path);
        return json_decode($response);
    }

    /**
     * get subscription by subscription_id
     */
    function activateSubscription($subscription_id) {

        $method = 'POST';
        $path = $this->rechargeUrl . '/subscriptions/' . $subscription_id . '/activate';
        $data="{}";
        $response = $this->call($method, $path, $data);
        return json_decode($response);
    }

    /**
     * create recharge checkout
     */
    function createCheckout($data) {

        $method = 'POST';
        $path = env('RECHARGE_URL', true).'/checkouts';
        $response = $this->call($method, $path, $data);
        return json_decode($response);
    }

    /**
     * update recharge checkout
     */
    function updateCheckout($token, $data) {

        $method = 'PUT';
        $path = env('RECHARGE_URL', true).'/checkouts/'.$token;
        $response = $this->call($method, $path, $data);
        return json_decode($response);
    }

    /**
     * get recharge checkout
     */
    function retrieveCheckout($token, $status = 'checkout') {

        $method = 'GET';
        if($status == 'checkout') {
            $path = env('RECHARGE_URL', true).'/checkouts/'.
            $token;
        } else {
            $path = env('RECHARGE_URL', true).'/checkouts/'.
                $token.'/shipping_rates';
        }
        $response = $this->call($method, $path);
        return json_decode($response);
    }

    /**
     * update shipping line item
     */
    function updateShippingLine($token, $data) {

        $method = 'PUT';
        $path = env('RECHARGE_URL', true).'/checkouts/'.$token;
        $response = $this->call($method, $path, $data);
        return json_decode($response);
    }

    /**
     * get customer details by customer id
     */
    function getRechargeCustomerDetailsByID($customerID) {

        $method = 'GET';
        $path = $this->rechargeUrl . '/customers/'.$customerID;
        $response = $this->call($method, $path);
        return json_decode($response);
    }

    /**
     * get customer details by customer email
     */
	function getRechargeCustomerDetailsByEmail($email) {

        $method = 'GET';
        $path = $this->rechargeUrl . '/customers?email='.$email;
        $response = $this->call($method, $path);
        return json_decode($response);
    }

    /**
     * get subscription by customer id
     */
    function getSubscriptionByCustomerID($customerID) {

        $method = 'GET';
        $path = $this->rechargeUrl . '/subscriptions?customer_id='.$customerID;
        $response = $this->call($method, $path);
        return json_decode($response);
    }

    /**
     * get customer address by address id
     */
    function getAddressByAddressID($addressId) {

        $method = 'GET';
        $path = env('RECHARGE_URL', true).'/addresses/'.$addressId;
        $response = $this->call($method, $path);
        return json_decode($response);
    }

    /**
     * get shipping rate by address
     */
    function updateShippingRateByAddress($addressId, $data) {

        $method = 'PUT';
        $path = env('RECHARGE_URL', true).'/addresses/'.$addressId;
        $response = $this->call($method, $path, $data);
        return json_decode($response);
    }

    /**
     * get all subscription by customer id
     */
    function getAllSubscriptionsByCustomerID($customerID) {

        $method = 'GET';
        $path = env('RECHARGE_URL', true).'/subscriptions?customer_id='.$customerID;
        $response = $this->call($method, $path);
        return json_decode($response);
    }

    /**
     * update subscription using upsell addon
     */
    function updateUpsellSubscription($data = NULL) {

        $method = 'GET';
        $path = env('UPDATE_SUBSCRIPTION_URL', true).'updateSubscriptionLineItemProperties';
        $response = $this->callEndpoint($method, $path, $data);
	$response = trim($response,")");
        $response = ltrim($response,"update(");
        return json_decode($response,true);
    }

    /**
     * get all active subscriptions by customer id
     */
    function getAllActiveSubscriptionsByCustomerID($customerID) {

        $method = 'GET';
        $path = env('RECHARGE_URL', true).'/subscriptions?customer_id='.$customerID.'&status=active';
        $response = $this->call($method, $path);
        return json_decode($response);
    }

    /**
     * change subscription next charge date
     */
    function updateSubscriptionChargeDate($subscriptionID, $data) {

        $method = 'POST';
        $path = env('RECHARGE_URL', true).'/subscriptions/'.$subscriptionID.'/set_next_charge_date';
        $response = $this->call($method, $path, $data);
        return json_decode($response);
    }

    /**
     * get charge details by addressID
     */
    function getOtpChargeDetails($addressID) {

        $method = 'GET';
        $path = env('RECHARGE_URL', true).'/charges?address_id='.$addressID;
        $response = $this->call($method, $path);
        return json_decode($response);
    }

    /**
     * create subscription in recharge
     */
    function createSubscription($data) {

        $method = 'POST';
        $path = env('RECHARGE_URL', true).'/subscriptions';
        $response = $this->call($method, $path, $data);
        return json_decode($response);
    }

    /**
     * get charge details by subscriptionID
     */
    function getChargeDetailsBySubscriptionID($subscriptionID) {

        $method = 'GET';
        $path = env('RECHARGE_URL', true).'/charges?subscription_id='.$subscriptionID;
        $response = $this->call($method, $path);
        return json_decode($response);
    }

    /**
     * get discount details by discount code
     */
    function getDiscountDetailsByDiscountCode($discountCode) {

        $method = 'GET';
        $path = env('RECHARGE_URL', true).'/discounts?discount_code='.$discountCode;
        $response = $this->call($method, $path);
        return json_decode($response);
    }

    /**
     * apply discount to a charge by discount code
     */
    function applyDiscountToChargeByDiscountCode($chargeID, $data) {

        $method = 'POST';
        $path = env('RECHARGE_URL', true).'/charges/'.$chargeID.'/apply_discount';
        $response = $this->call($method, $path, $data);
        return json_decode($response);
    }

    /**
     * Get queued charge By subscription id
     */
    function getQueuedChargeBySubscriptionID($subscriptionID) {

        $method = 'GET';
        $path = env('RECHARGE_URL', true).'/charges?status=queued&subscription_id='.$subscriptionID;
        $response = $this->call($method, $path);
        return json_decode($response);
    }

    /**
     * CURL call
     */
    function call($method, $path, $data = []) {

        $curl = curl_init();
        curl_setopt($curl, CURLOPT_URL, $path);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_ENCODING, '');
        curl_setopt($curl, CURLOPT_MAXREDIRS, 10);
        curl_setopt($curl, CURLOPT_TIMEOUT, 30);
        curl_setopt($curl, CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_1_1);
        curl_setopt($curl, CURLOPT_CUSTOMREQUEST, $method);

        if ($method !== 'GET') {
            curl_setopt($curl, CURLOPT_POSTFIELDS, $data);
        }
        curl_setopt($curl, CURLOPT_HTTPHEADER, array(
            "cache-control: no-cache",
            "x-recharge-access-token: " . env('RECHARGE_API_KEY',true),
            "content-type: application/json",
        ));
        $response = curl_exec($curl);
        return $response;
    }

    /**
     * CURL call for update the upsell subscription
     */
    function callEndpoint($method, $path, $data = []) {

        $curl = curl_init();
        curl_setopt($curl, CURLOPT_URL, $path.'?callback=update&data='.urlencode($data));
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_CUSTOMREQUEST, $method);
        curl_setopt($curl, CURLOPT_HTTPHEADER, array(
            "content-type: application/json",
        ));
        $response = curl_exec($curl);
        return $response;

    }
}
